@@ -1,7 +1,4 @@
-//! Implementation of `[stable_mir::compiler_interface::Context]` trait.
-//!
-//! This trait is currently the main interface between the Rust compiler,
-//! and the `stable_mir` crate.
+//! Implementation of StableMIR Context.
 
 #![allow(rustc::usage_of_qualified_ty)]
 
@@ -9,7 +6,7 @@ use std::cell::RefCell;
 use std::iter;
 
 use rustc_abi::HasDataLayout;
-use rustc_hir::LangItem;
+use rustc_hir::{Attribute, LangItem};
 use rustc_middle::ty::layout::{
     FnAbiOf, FnAbiOfHelpers, HasTyCtxt, HasTypingEnv, LayoutOf, LayoutOfHelpers,
 };
@@ -20,7 +17,6 @@ use rustc_middle::ty::{
 use rustc_middle::{mir, ty};
 use rustc_span::def_id::LOCAL_CRATE;
 use stable_mir::abi::{FnAbi, Layout, LayoutShape};
-use stable_mir::compiler_interface::Context;
 use stable_mir::mir::alloc::GlobalAlloc;
 use stable_mir::mir::mono::{InstanceDef, StaticDef};
 use stable_mir::mir::{BinOp, Body, Place, UnOp};
@@ -35,9 +31,16 @@ use stable_mir::{Crate, CrateDef, CrateItem, CrateNum, DefId, Error, Filename, I
 use crate::rustc_internal::RustcInternal;
 use crate::rustc_smir::builder::BodyBuilder;
 use crate::rustc_smir::{Stable, Tables, alloc, filter_def_ids, new_item_kind, smir_crate};
+use crate::stable_mir;
 
-impl<'tcx> Context for TablesWrapper<'tcx> {
-    fn target_info(&self) -> MachineInfo {
+/// Provides direct access to rustc's internal queries.
+///
+/// The [`crate::stable_mir::compiler_interface::SmirInterface`] must go through
+/// this context to obtain rustc-level information.
+pub struct SmirCtxt<'tcx>(pub RefCell<Tables<'tcx>>);
+
+impl<'tcx> SmirCtxt<'tcx> {
+    pub fn target_info(&self) -> MachineInfo {
         let mut tables = self.0.borrow_mut();
         MachineInfo {
             endian: tables.tcx.data_layout.endian.stable(&mut *tables),
@@ -47,31 +50,35 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
         }
     }
 
-    fn entry_fn(&self) -> Option<stable_mir::CrateItem> {
+    pub fn entry_fn(&self) -> Option<stable_mir::CrateItem> {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         Some(tables.crate_item(tcx.entry_fn(())?.0))
     }
 
-    fn all_local_items(&self) -> stable_mir::CrateItems {
+    /// Retrieve all items of the local crate that have a MIR associated with them.
+    pub fn all_local_items(&self) -> stable_mir::CrateItems {
         let mut tables = self.0.borrow_mut();
         tables.tcx.mir_keys(()).iter().map(|item| tables.crate_item(item.to_def_id())).collect()
     }
 
-    fn mir_body(&self, item: stable_mir::DefId) -> stable_mir::mir::Body {
+    /// Retrieve the body of a function.
+    /// This function will panic if the body is not available.
+    pub fn mir_body(&self, item: stable_mir::DefId) -> stable_mir::mir::Body {
         let mut tables = self.0.borrow_mut();
         let def_id = tables[item];
         tables.tcx.instance_mir(rustc_middle::ty::InstanceKind::Item(def_id)).stable(&mut tables)
     }
 
-    fn has_body(&self, def: DefId) -> bool {
+    /// Check whether the body of a function is available.
+    pub fn has_body(&self, def: DefId) -> bool {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         let def_id = def.internal(&mut *tables, tcx);
         tables.item_has_body(def_id)
     }
 
-    fn foreign_modules(&self, crate_num: CrateNum) -> Vec<stable_mir::ty::ForeignModuleDef> {
+    pub fn foreign_modules(&self, crate_num: CrateNum) -> Vec<stable_mir::ty::ForeignModuleDef> {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         tcx.foreign_modules(crate_num.internal(&mut *tables, tcx))
@@ -80,21 +87,23 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
             .collect()
     }
 
-    fn crate_functions(&self, crate_num: CrateNum) -> Vec<FnDef> {
+    /// Retrieve all functions defined in this crate.
+    pub fn crate_functions(&self, crate_num: CrateNum) -> Vec<FnDef> {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         let krate = crate_num.internal(&mut *tables, tcx);
         filter_def_ids(tcx, krate, |def_id| tables.to_fn_def(def_id))
     }
 
-    fn crate_statics(&self, crate_num: CrateNum) -> Vec<StaticDef> {
+    /// Retrieve all static items defined in this crate.
+    pub fn crate_statics(&self, crate_num: CrateNum) -> Vec<StaticDef> {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         let krate = crate_num.internal(&mut *tables, tcx);
         filter_def_ids(tcx, krate, |def_id| tables.to_static(def_id))
     }
 
-    fn foreign_module(
+    pub fn foreign_module(
         &self,
         mod_def: stable_mir::ty::ForeignModuleDef,
     ) -> stable_mir::ty::ForeignModule {
@@ -104,7 +113,7 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
         mod_def.stable(&mut *tables)
     }
 
-    fn foreign_items(&self, mod_def: stable_mir::ty::ForeignModuleDef) -> Vec<ForeignDef> {
+    pub fn foreign_items(&self, mod_def: stable_mir::ty::ForeignModuleDef) -> Vec<ForeignDef> {
         let mut tables = self.0.borrow_mut();
         let def_id = tables[mod_def.def_id()];
         tables
@@ -118,12 +127,12 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
             .collect()
     }
 
-    fn all_trait_decls(&self) -> stable_mir::TraitDecls {
+    pub fn all_trait_decls(&self) -> stable_mir::TraitDecls {
         let mut tables = self.0.borrow_mut();
         tables.tcx.all_traits().map(|trait_def_id| tables.trait_def(trait_def_id)).collect()
     }
 
-    fn trait_decls(&self, crate_num: CrateNum) -> stable_mir::TraitDecls {
+    pub fn trait_decls(&self, crate_num: CrateNum) -> stable_mir::TraitDecls {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         tcx.traits(crate_num.internal(&mut *tables, tcx))
@@ -132,14 +141,14 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
             .collect()
     }
 
-    fn trait_decl(&self, trait_def: &stable_mir::ty::TraitDef) -> stable_mir::ty::TraitDecl {
+    pub fn trait_decl(&self, trait_def: &stable_mir::ty::TraitDef) -> stable_mir::ty::TraitDecl {
         let mut tables = self.0.borrow_mut();
         let def_id = tables[trait_def.0];
         let trait_def = tables.tcx.trait_def(def_id);
         trait_def.stable(&mut *tables)
     }
 
-    fn all_trait_impls(&self) -> stable_mir::ImplTraitDecls {
+    pub fn all_trait_impls(&self) -> stable_mir::ImplTraitDecls {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         iter::once(LOCAL_CRATE)
@@ -149,7 +158,7 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
             .collect()
     }
 
-    fn trait_impls(&self, crate_num: CrateNum) -> stable_mir::ImplTraitDecls {
+    pub fn trait_impls(&self, crate_num: CrateNum) -> stable_mir::ImplTraitDecls {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         tcx.trait_impls_in_crate(crate_num.internal(&mut *tables, tcx))
@@ -158,21 +167,21 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
             .collect()
     }
 
-    fn trait_impl(&self, impl_def: &stable_mir::ty::ImplDef) -> stable_mir::ty::ImplTrait {
+    pub fn trait_impl(&self, impl_def: &stable_mir::ty::ImplDef) -> stable_mir::ty::ImplTrait {
         let mut tables = self.0.borrow_mut();
         let def_id = tables[impl_def.0];
         let impl_trait = tables.tcx.impl_trait_ref(def_id).unwrap();
         impl_trait.stable(&mut *tables)
     }
 
-    fn generics_of(&self, def_id: stable_mir::DefId) -> stable_mir::ty::Generics {
+    pub fn generics_of(&self, def_id: stable_mir::DefId) -> stable_mir::ty::Generics {
         let mut tables = self.0.borrow_mut();
         let def_id = tables[def_id];
         let generics = tables.tcx.generics_of(def_id);
         generics.stable(&mut *tables)
     }
 
-    fn predicates_of(&self, def_id: stable_mir::DefId) -> stable_mir::ty::GenericPredicates {
+    pub fn predicates_of(&self, def_id: stable_mir::DefId) -> stable_mir::ty::GenericPredicates {
         let mut tables = self.0.borrow_mut();
         let def_id = tables[def_id];
         let GenericPredicates { parent, predicates } = tables.tcx.predicates_of(def_id);
@@ -190,7 +199,7 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
         }
     }
 
-    fn explicit_predicates_of(
+    pub fn explicit_predicates_of(
         &self,
         def_id: stable_mir::DefId,
     ) -> stable_mir::ty::GenericPredicates {
@@ -211,17 +220,20 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
         }
     }
 
-    fn local_crate(&self) -> stable_mir::Crate {
+    /// Get information about the local crate.
+    pub fn local_crate(&self) -> stable_mir::Crate {
         let tables = self.0.borrow();
         smir_crate(tables.tcx, LOCAL_CRATE)
     }
 
-    fn external_crates(&self) -> Vec<stable_mir::Crate> {
+    /// Retrieve a list of all external crates.
+    pub fn external_crates(&self) -> Vec<stable_mir::Crate> {
         let tables = self.0.borrow();
         tables.tcx.crates(()).iter().map(|crate_num| smir_crate(tables.tcx, *crate_num)).collect()
     }
 
-    fn find_crates(&self, name: &str) -> Vec<stable_mir::Crate> {
+    /// Find a crate with the given name.
+    pub fn find_crates(&self, name: &str) -> Vec<stable_mir::Crate> {
         let tables = self.0.borrow();
         let crates: Vec<stable_mir::Crate> = [LOCAL_CRATE]
             .iter()
@@ -234,7 +246,8 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
         crates
     }
 
-    fn def_name(&self, def_id: stable_mir::DefId, trimmed: bool) -> Symbol {
+    /// Returns the name of given `DefId`.
+    pub fn def_name(&self, def_id: stable_mir::DefId, trimmed: bool) -> Symbol {
         let tables = self.0.borrow();
         if trimmed {
             with_forced_trimmed_paths!(tables.tcx.def_path_str(tables[def_id]))
@@ -243,7 +256,14 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
         }
     }
 
-    fn get_attrs_by_path(
+    /// Return registered tool attributes with the given attribute name.
+    ///
+    /// FIXME(jdonszelmann): may panic on non-tool attributes. After more attribute work, non-tool
+    /// attributes will simply return an empty list.
+    ///
+    /// Single segmented name like `#[clippy]` is specified as `&["clippy".to_string()]`.
+    /// Multi-segmented name like `#[rustfmt::skip]` is specified as `&["rustfmt".to_string(), "skip".to_string()]`.
+    pub fn tool_attrs(
         &self,
         def_id: stable_mir::DefId,
         attr: &[stable_mir::Symbol],
@@ -251,44 +271,58 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         let did = tables[def_id];
-        let attr_name: Vec<_> =
-            attr.iter().map(|seg| rustc_span::symbol::Symbol::intern(&seg)).collect();
+        let attr_name: Vec<_> = attr.iter().map(|seg| rustc_span::Symbol::intern(&seg)).collect();
         tcx.get_attrs_by_path(did, &attr_name)
-            .map(|attribute| {
-                let attr_str = rustc_ast_pretty::pprust::attribute_to_string(attribute);
-                let span = attribute.span;
-                stable_mir::crate_def::Attribute::new(attr_str, span.stable(&mut *tables))
+            .filter_map(|attribute| {
+                if let Attribute::Unparsed(u) = attribute {
+                    let attr_str = rustc_hir_pretty::attribute_to_string(&tcx, attribute);
+                    Some(stable_mir::crate_def::Attribute::new(
+                        attr_str,
+                        u.span.stable(&mut *tables),
+                    ))
+                } else {
+                    None
+                }
             })
             .collect()
     }
 
-    fn get_all_attrs(&self, def_id: stable_mir::DefId) -> Vec<stable_mir::crate_def::Attribute> {
+    /// Get all tool attributes of a definition.
+    pub fn all_tool_attrs(
+        &self,
+        def_id: stable_mir::DefId,
+    ) -> Vec<stable_mir::crate_def::Attribute> {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         let did = tables[def_id];
-        let filter_fn = move |a: &&rustc_ast::ast::Attribute| {
-            matches!(a.kind, rustc_ast::ast::AttrKind::Normal(_))
-        };
         let attrs_iter = if let Some(did) = did.as_local() {
-            tcx.hir().attrs(tcx.local_def_id_to_hir_id(did)).iter().filter(filter_fn)
+            tcx.hir_attrs(tcx.local_def_id_to_hir_id(did)).iter()
         } else {
-            tcx.item_attrs(did).iter().filter(filter_fn)
+            tcx.attrs_for_def(did).iter()
         };
         attrs_iter
-            .map(|attribute| {
-                let attr_str = rustc_ast_pretty::pprust::attribute_to_string(attribute);
-                let span = attribute.span;
-                stable_mir::crate_def::Attribute::new(attr_str, span.stable(&mut *tables))
+            .filter_map(|attribute| {
+                if let Attribute::Unparsed(u) = attribute {
+                    let attr_str = rustc_hir_pretty::attribute_to_string(&tcx, attribute);
+                    Some(stable_mir::crate_def::Attribute::new(
+                        attr_str,
+                        u.span.stable(&mut *tables),
+                    ))
+                } else {
+                    None
+                }
             })
             .collect()
     }
 
-    fn span_to_string(&self, span: stable_mir::ty::Span) -> String {
+    /// Returns printable, human readable form of `Span`.
+    pub fn span_to_string(&self, span: stable_mir::ty::Span) -> String {
         let tables = self.0.borrow();
         tables.tcx.sess.source_map().span_to_diagnostic_string(tables[span])
     }
 
-    fn get_filename(&self, span: &Span) -> Filename {
+    /// Return filename from given `Span`, for diagnostic purposes.
+    pub fn get_filename(&self, span: &Span) -> Filename {
         let tables = self.0.borrow();
         tables
             .tcx
@@ -299,23 +333,27 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
             .to_string()
     }
 
-    fn get_lines(&self, span: &Span) -> LineInfo {
+    /// Return lines corresponding to this `Span`.
+    pub fn get_lines(&self, span: &Span) -> LineInfo {
         let tables = self.0.borrow();
         let lines = &tables.tcx.sess.source_map().span_to_location_info(tables[*span]);
         LineInfo { start_line: lines.1, start_col: lines.2, end_line: lines.3, end_col: lines.4 }
     }
 
-    fn item_kind(&self, item: CrateItem) -> ItemKind {
+    /// Returns the `kind` of given `DefId`.
+    pub fn item_kind(&self, item: CrateItem) -> ItemKind {
         let tables = self.0.borrow();
         new_item_kind(tables.tcx.def_kind(tables[item.0]))
     }
 
-    fn is_foreign_item(&self, item: DefId) -> bool {
+    /// Returns whether this is a foreign item.
+    pub fn is_foreign_item(&self, item: DefId) -> bool {
         let tables = self.0.borrow();
         tables.tcx.is_foreign_item(tables[item])
     }
 
-    fn foreign_item_kind(&self, def: ForeignDef) -> ForeignItemKind {
+    /// Returns the kind of a given foreign item.
+    pub fn foreign_item_kind(&self, def: ForeignDef) -> ForeignItemKind {
         let mut tables = self.0.borrow_mut();
         let def_id = tables[def.def_id()];
         let tcx = tables.tcx;
@@ -330,32 +368,37 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
         }
     }
 
-    fn adt_kind(&self, def: AdtDef) -> AdtKind {
+    /// Returns the kind of a given algebraic data type.
+    pub fn adt_kind(&self, def: AdtDef) -> AdtKind {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         def.internal(&mut *tables, tcx).adt_kind().stable(&mut *tables)
     }
 
-    fn adt_is_box(&self, def: AdtDef) -> bool {
+    /// Returns if the ADT is a box.
+    pub fn adt_is_box(&self, def: AdtDef) -> bool {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         def.internal(&mut *tables, tcx).is_box()
     }
 
-    fn adt_is_simd(&self, def: AdtDef) -> bool {
+    /// Returns whether this ADT is simd.
+    pub fn adt_is_simd(&self, def: AdtDef) -> bool {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         def.internal(&mut *tables, tcx).repr().simd()
     }
 
-    fn adt_is_cstr(&self, def: AdtDef) -> bool {
+    /// Returns whether this definition is a C string.
+    pub fn adt_is_cstr(&self, def: AdtDef) -> bool {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         let def_id = def.0.internal(&mut *tables, tcx);
         tables.tcx.is_lang_item(def_id, LangItem::CStr)
     }
 
-    fn fn_sig(&self, def: FnDef, args: &GenericArgs) -> PolyFnSig {
+    /// Retrieve the function signature for the given generic arguments.
+    pub fn fn_sig(&self, def: FnDef, args: &GenericArgs) -> PolyFnSig {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         let def_id = def.0.internal(&mut *tables, tcx);
@@ -364,7 +407,8 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
         sig.stable(&mut *tables)
     }
 
-    fn intrinsic(&self, def: DefId) -> Option<IntrinsicDef> {
+    /// Retrieve the intrinsic definition if the item corresponds one.
+    pub fn intrinsic(&self, def: DefId) -> Option<IntrinsicDef> {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         let def_id = def.internal(&mut *tables, tcx);
@@ -372,14 +416,16 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
         intrinsic.map(|_| IntrinsicDef(def))
     }
 
-    fn intrinsic_name(&self, def: IntrinsicDef) -> Symbol {
+    /// Retrieve the plain function name of an intrinsic.
+    pub fn intrinsic_name(&self, def: IntrinsicDef) -> Symbol {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         let def_id = def.0.internal(&mut *tables, tcx);
         tcx.intrinsic(def_id).unwrap().name.to_string()
     }
 
-    fn closure_sig(&self, args: &GenericArgs) -> PolyFnSig {
+    /// Retrieve the closure signature for the given generic arguments.
+    pub fn closure_sig(&self, args: &GenericArgs) -> PolyFnSig {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         let args_ref = args.internal(&mut *tables, tcx);
@@ -387,25 +433,28 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
         sig.stable(&mut *tables)
     }
 
-    fn adt_variants_len(&self, def: AdtDef) -> usize {
+    /// The number of variants in this ADT.
+    pub fn adt_variants_len(&self, def: AdtDef) -> usize {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         def.internal(&mut *tables, tcx).variants().len()
     }
 
-    fn variant_name(&self, def: VariantDef) -> Symbol {
+    /// The name of a variant.
+    pub fn variant_name(&self, def: VariantDef) -> Symbol {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         def.internal(&mut *tables, tcx).name.to_string()
     }
 
-    fn variant_fields(&self, def: VariantDef) -> Vec<FieldDef> {
+    pub fn variant_fields(&self, def: VariantDef) -> Vec<FieldDef> {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         def.internal(&mut *tables, tcx).fields.iter().map(|f| f.stable(&mut *tables)).collect()
     }
 
-    fn eval_target_usize(&self, cnst: &MirConst) -> Result<u64, Error> {
+    /// Evaluate constant as a target usize.
+    pub fn eval_target_usize(&self, cnst: &MirConst) -> Result<u64, Error> {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         let mir_const = cnst.internal(&mut *tables, tcx);
@@ -413,7 +462,7 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
             .try_eval_target_usize(tables.tcx, ty::TypingEnv::fully_monomorphized())
             .ok_or_else(|| Error::new(format!("Const `{cnst:?}` cannot be encoded as u64")))
     }
-    fn eval_target_usize_ty(&self, cnst: &TyConst) -> Result<u64, Error> {
+    pub fn eval_target_usize_ty(&self, cnst: &TyConst) -> Result<u64, Error> {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         let mir_const = cnst.internal(&mut *tables, tcx);
@@ -422,7 +471,8 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
             .ok_or_else(|| Error::new(format!("Const `{cnst:?}` cannot be encoded as u64")))
     }
 
-    fn try_new_const_zst(&self, ty: Ty) -> Result<MirConst, Error> {
+    /// Create a new zero-sized constant.
+    pub fn try_new_const_zst(&self, ty: Ty) -> Result<MirConst, Error> {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         let ty_internal = ty.internal(&mut *tables, tcx);
@@ -447,25 +497,26 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
             .stable(&mut *tables))
     }
 
-    fn new_const_str(&self, value: &str) -> MirConst {
+    /// Create a new constant that represents the given string value.
+    pub fn new_const_str(&self, value: &str) -> MirConst {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         let ty = ty::Ty::new_static_str(tcx);
         let bytes = value.as_bytes();
-        let val_tree = ty::ValTree::from_raw_bytes(tcx, bytes);
-
-        let ct = ty::Const::new_value(tcx, val_tree, ty);
-        super::convert::mir_const_from_ty_const(&mut *tables, ct, ty)
+        let valtree = ty::ValTree::from_raw_bytes(tcx, bytes);
+        let cv = ty::Value { ty, valtree };
+        let val = tcx.valtree_to_const_val(cv);
+        mir::Const::from_value(val, ty).stable(&mut tables)
     }
 
-    fn new_const_bool(&self, value: bool) -> MirConst {
+    /// Create a new constant that represents the given boolean value.
+    pub fn new_const_bool(&self, value: bool) -> MirConst {
         let mut tables = self.0.borrow_mut();
-        let ct = ty::Const::from_bool(tables.tcx, value);
-        let ty = tables.tcx.types.bool;
-        super::convert::mir_const_from_ty_const(&mut *tables, ct, ty)
+        mir::Const::from_bool(tables.tcx, value).stable(&mut tables)
     }
 
-    fn try_new_const_uint(&self, value: u128, uint_ty: UintTy) -> Result<MirConst, Error> {
+    /// Create a new constant that represents the given value.
+    pub fn try_new_const_uint(&self, value: u128, uint_ty: UintTy) -> Result<MirConst, Error> {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         let ty = ty::Ty::new_uint(tcx, uint_ty.internal(&mut *tables, tcx));
@@ -474,15 +525,13 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
             .layout_of(ty::TypingEnv::fully_monomorphized().as_query_input(ty))
             .unwrap()
             .size;
-
-        // We don't use Const::from_bits since it doesn't have any error checking.
         let scalar = ScalarInt::try_from_uint(value, size).ok_or_else(|| {
             Error::new(format!("Value overflow: cannot convert `{value}` to `{ty}`."))
         })?;
-        let ct = ty::Const::new_value(tables.tcx, ValTree::from_scalar_int(scalar), ty);
-        Ok(super::convert::mir_const_from_ty_const(&mut *tables, ct, ty))
+        Ok(mir::Const::from_scalar(tcx, mir::interpret::Scalar::Int(scalar), ty)
+            .stable(&mut tables))
     }
-    fn try_new_ty_const_uint(
+    pub fn try_new_ty_const_uint(
         &self,
         value: u128,
         uint_ty: UintTy,
@@ -500,31 +549,39 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
         let scalar = ScalarInt::try_from_uint(value, size).ok_or_else(|| {
             Error::new(format!("Value overflow: cannot convert `{value}` to `{ty}`."))
         })?;
-        Ok(ty::Const::new_value(tables.tcx, ValTree::from_scalar_int(scalar), ty)
+        Ok(ty::Const::new_value(tcx, ValTree::from_scalar_int(tcx, scalar), ty)
             .stable(&mut *tables))
     }
 
-    fn new_rigid_ty(&self, kind: RigidTy) -> stable_mir::ty::Ty {
+    /// Create a new type from the given kind.
+    pub fn new_rigid_ty(&self, kind: RigidTy) -> stable_mir::ty::Ty {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         let internal_kind = kind.internal(&mut *tables, tcx);
         tables.tcx.mk_ty_from_kind(internal_kind).stable(&mut *tables)
     }
 
-    fn new_box_ty(&self, ty: stable_mir::ty::Ty) -> stable_mir::ty::Ty {
+    /// Create a new box type, `Box<T>`, for the given inner type `T`.
+    pub fn new_box_ty(&self, ty: stable_mir::ty::Ty) -> stable_mir::ty::Ty {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         let inner = ty.internal(&mut *tables, tcx);
         ty::Ty::new_box(tables.tcx, inner).stable(&mut *tables)
     }
 
-    fn def_ty(&self, item: stable_mir::DefId) -> stable_mir::ty::Ty {
+    /// Returns the type of given crate item.
+    pub fn def_ty(&self, item: stable_mir::DefId) -> stable_mir::ty::Ty {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         tcx.type_of(item.internal(&mut *tables, tcx)).instantiate_identity().stable(&mut *tables)
     }
 
-    fn def_ty_with_args(&self, item: stable_mir::DefId, args: &GenericArgs) -> stable_mir::ty::Ty {
+    /// Returns the type of given definition instantiated with the given arguments.
+    pub fn def_ty_with_args(
+        &self,
+        item: stable_mir::DefId,
+        args: &GenericArgs,
+    ) -> stable_mir::ty::Ty {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         let args = args.internal(&mut *tables, tcx);
@@ -539,33 +596,38 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
             .stable(&mut *tables)
     }
 
-    fn mir_const_pretty(&self, cnst: &stable_mir::ty::MirConst) -> String {
+    /// Returns literal value of a const as a string.
+    pub fn mir_const_pretty(&self, cnst: &stable_mir::ty::MirConst) -> String {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         cnst.internal(&mut *tables, tcx).to_string()
     }
 
-    fn span_of_an_item(&self, def_id: stable_mir::DefId) -> Span {
+    /// `Span` of an item.
+    pub fn span_of_an_item(&self, def_id: stable_mir::DefId) -> Span {
         let mut tables = self.0.borrow_mut();
         tables.tcx.def_span(tables[def_id]).stable(&mut *tables)
     }
 
-    fn ty_pretty(&self, ty: stable_mir::ty::Ty) -> String {
+    /// Obtain the representation of a type.
+    pub fn ty_pretty(&self, ty: stable_mir::ty::Ty) -> String {
         let tables = self.0.borrow_mut();
         tables.types[ty].to_string()
     }
 
-    fn ty_kind(&self, ty: stable_mir::ty::Ty) -> TyKind {
+    /// Obtain the representation of a type.
+    pub fn ty_kind(&self, ty: stable_mir::ty::Ty) -> TyKind {
         let mut tables = self.0.borrow_mut();
         tables.types[ty].kind().stable(&mut *tables)
     }
 
-    fn ty_const_pretty(&self, ct: stable_mir::ty::TyConstId) -> String {
+    pub fn ty_const_pretty(&self, ct: stable_mir::ty::TyConstId) -> String {
         let tables = self.0.borrow_mut();
         tables.ty_consts[ct].to_string()
     }
 
-    fn rigid_ty_discriminant_ty(&self, ty: &RigidTy) -> stable_mir::ty::Ty {
+    /// Get the discriminant Ty for this Ty if there's one.
+    pub fn rigid_ty_discriminant_ty(&self, ty: &RigidTy) -> stable_mir::ty::Ty {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         let internal_kind = ty.internal(&mut *tables, tcx);
@@ -573,7 +635,8 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
         internal_ty.discriminant_ty(tables.tcx).stable(&mut *tables)
     }
 
-    fn instance_body(&self, def: InstanceDef) -> Option<Body> {
+    /// Get the body of an Instance which is already monomorphized.
+    pub fn instance_body(&self, def: InstanceDef) -> Option<Body> {
         let mut tables = self.0.borrow_mut();
         let instance = tables.instances[def];
         tables
@@ -581,63 +644,67 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
             .then(|| BodyBuilder::new(tables.tcx, instance).build(&mut *tables))
     }
 
-    fn instance_ty(&self, def: InstanceDef) -> stable_mir::ty::Ty {
+    /// Get the instance type with generic instantiations applied and lifetimes erased.
+    pub fn instance_ty(&self, def: InstanceDef) -> stable_mir::ty::Ty {
         let mut tables = self.0.borrow_mut();
         let instance = tables.instances[def];
         assert!(!instance.has_non_region_param(), "{instance:?} needs further instantiation");
         instance.ty(tables.tcx, ty::TypingEnv::fully_monomorphized()).stable(&mut *tables)
     }
 
-    fn instance_args(&self, def: InstanceDef) -> GenericArgs {
+    /// Get the instantiation types.
+    pub fn instance_args(&self, def: InstanceDef) -> GenericArgs {
         let mut tables = self.0.borrow_mut();
         let instance = tables.instances[def];
         instance.args.stable(&mut *tables)
     }
 
-    fn instance_abi(&self, def: InstanceDef) -> Result<FnAbi, Error> {
+    /// Get an instance ABI.
+    pub fn instance_abi(&self, def: InstanceDef) -> Result<FnAbi, Error> {
         let mut tables = self.0.borrow_mut();
         let instance = tables.instances[def];
         Ok(tables.fn_abi_of_instance(instance, List::empty())?.stable(&mut *tables))
     }
 
-    fn fn_ptr_abi(&self, fn_ptr: PolyFnSig) -> Result<FnAbi, Error> {
+    /// Get the ABI of a function pointer.
+    pub fn fn_ptr_abi(&self, fn_ptr: PolyFnSig) -> Result<FnAbi, Error> {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         let sig = fn_ptr.internal(&mut *tables, tcx);
         Ok(tables.fn_abi_of_fn_ptr(sig, List::empty())?.stable(&mut *tables))
     }
 
-    fn instance_def_id(&self, def: InstanceDef) -> stable_mir::DefId {
+    /// Get the instance.
+    pub fn instance_def_id(&self, def: InstanceDef) -> stable_mir::DefId {
         let mut tables = self.0.borrow_mut();
         let def_id = tables.instances[def].def_id();
         tables.create_def_id(def_id)
     }
 
-    fn instance_mangled_name(&self, instance: InstanceDef) -> Symbol {
+    /// Get the instance mangled name.
+    pub fn instance_mangled_name(&self, instance: InstanceDef) -> Symbol {
         let tables = self.0.borrow_mut();
         let instance = tables.instances[instance];
         tables.tcx.symbol_name(instance).name.to_string()
     }
 
-    fn is_empty_drop_shim(&self, def: InstanceDef) -> bool {
+    /// Check if this is an empty DropGlue shim.
+    pub fn is_empty_drop_shim(&self, def: InstanceDef) -> bool {
         let tables = self.0.borrow_mut();
         let instance = tables.instances[def];
         matches!(instance.def, ty::InstanceKind::DropGlue(_, None))
     }
 
-    fn is_empty_async_drop_ctor_shim(&self, def: InstanceDef) -> bool {
-        let tables = self.0.borrow_mut();
-        let instance = tables.instances[def];
-        matches!(instance.def, ty::InstanceKind::AsyncDropGlueCtorShim(_, None))
-    }
-
-    fn mono_instance(&self, def_id: stable_mir::DefId) -> stable_mir::mir::mono::Instance {
+    /// Convert a non-generic crate item into an instance.
+    /// This function will panic if the item is generic.
+    pub fn mono_instance(&self, def_id: stable_mir::DefId) -> stable_mir::mir::mono::Instance {
         let mut tables = self.0.borrow_mut();
         let def_id = tables[def_id];
         Instance::mono(tables.tcx, def_id).stable(&mut *tables)
     }
 
-    fn requires_monomorphization(&self, def_id: stable_mir::DefId) -> bool {
+    /// Item requires monomorphization.
+    pub fn requires_monomorphization(&self, def_id: stable_mir::DefId) -> bool {
         let tables = self.0.borrow();
         let def_id = tables[def_id];
         let generics = tables.tcx.generics_of(def_id);
@@ -645,7 +712,8 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
         result
     }
 
-    fn resolve_instance(
+    /// Resolve an instance from the given function definition and generic arguments.
+    pub fn resolve_instance(
         &self,
         def: stable_mir::ty::FnDef,
         args: &stable_mir::ty::GenericArgs,
@@ -665,7 +733,8 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
         }
     }
 
-    fn resolve_drop_in_place(&self, ty: stable_mir::ty::Ty) -> stable_mir::mir::mono::Instance {
+    /// Resolve an instance for drop_in_place for the given type.
+    pub fn resolve_drop_in_place(&self, ty: stable_mir::ty::Ty) -> stable_mir::mir::mono::Instance {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         let internal_ty = ty.internal(&mut *tables, tcx);
@@ -673,7 +742,8 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
         instance.stable(&mut *tables)
     }
 
-    fn resolve_for_fn_ptr(
+    /// Resolve instance for a function pointer.
+    pub fn resolve_for_fn_ptr(
         &self,
         def: FnDef,
         args: &GenericArgs,
@@ -691,7 +761,8 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
         .stable(&mut *tables)
     }
 
-    fn resolve_closure(
+    /// Resolve instance for a closure with the requested type.
+    pub fn resolve_closure(
         &self,
         def: ClosureDef,
         args: &GenericArgs,
@@ -708,7 +779,8 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
         )
     }
 
-    fn eval_instance(&self, def: InstanceDef, const_ty: Ty) -> Result<Allocation, Error> {
+    /// Try to evaluate an instance into a constant.
+    pub fn eval_instance(&self, def: InstanceDef, const_ty: Ty) -> Result<Allocation, Error> {
         let mut tables = self.0.borrow_mut();
         let instance = tables.instances[def];
         let tcx = tables.tcx;
@@ -728,21 +800,24 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
             .map_err(|e| e.stable(&mut *tables))?
     }
 
-    fn eval_static_initializer(&self, def: StaticDef) -> Result<Allocation, Error> {
+    /// Evaluate a static's initializer.
+    pub fn eval_static_initializer(&self, def: StaticDef) -> Result<Allocation, Error> {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         let def_id = def.0.internal(&mut *tables, tcx);
         tables.tcx.eval_static_initializer(def_id).stable(&mut *tables)
     }
 
-    fn global_alloc(&self, alloc: stable_mir::mir::alloc::AllocId) -> GlobalAlloc {
+    /// Retrieve global allocation for the given allocation ID.
+    pub fn global_alloc(&self, alloc: stable_mir::mir::alloc::AllocId) -> GlobalAlloc {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         let alloc_id = alloc.internal(&mut *tables, tcx);
         tables.tcx.global_alloc(alloc_id).stable(&mut *tables)
     }
 
-    fn vtable_allocation(
+    /// Retrieve the id for the virtual table.
+    pub fn vtable_allocation(
         &self,
         global_alloc: &GlobalAlloc,
     ) -> Option<stable_mir::mir::alloc::AllocId> {
@@ -753,12 +828,14 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
         let tcx = tables.tcx;
         let alloc_id = tables.tcx.vtable_allocation((
             ty.internal(&mut *tables, tcx),
-            trait_ref.internal(&mut *tables, tcx),
+            trait_ref
+                .internal(&mut *tables, tcx)
+                .map(|principal| tcx.instantiate_bound_regions_with_erased(principal)),
         ));
         Some(alloc_id.stable(&mut *tables))
     }
 
-    fn krate(&self, def_id: stable_mir::DefId) -> Crate {
+    pub fn krate(&self, def_id: stable_mir::DefId) -> Crate {
         let tables = self.0.borrow();
         smir_crate(tables.tcx, tables[def_id].krate)
     }
@@ -766,7 +843,7 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
     /// Retrieve the instance name for diagnostic messages.
     ///
     /// This will return the specialized name, e.g., `Vec<char>::new`.
-    fn instance_name(&self, def: InstanceDef, trimmed: bool) -> Symbol {
+    pub fn instance_name(&self, def: InstanceDef, trimmed: bool) -> Symbol {
         let tables = self.0.borrow_mut();
         let instance = tables.instances[def];
         if trimmed {
@@ -780,7 +857,8 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
         }
     }
 
-    fn ty_layout(&self, ty: Ty) -> Result<Layout, Error> {
+    /// Get the layout of a type.
+    pub fn ty_layout(&self, ty: Ty) -> Result<Layout, Error> {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         let ty = ty.internal(&mut *tables, tcx);
@@ -788,19 +866,22 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
         Ok(layout.stable(&mut *tables))
     }
 
-    fn layout_shape(&self, id: Layout) -> LayoutShape {
+    /// Get the layout shape.
+    pub fn layout_shape(&self, id: Layout) -> LayoutShape {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         id.internal(&mut *tables, tcx).0.stable(&mut *tables)
     }
 
-    fn place_pretty(&self, place: &Place) -> String {
+    /// Get a debug string representation of a place.
+    pub fn place_pretty(&self, place: &Place) -> String {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         format!("{:?}", place.internal(&mut *tables, tcx))
     }
 
-    fn binop_ty(&self, bin_op: BinOp, rhs: Ty, lhs: Ty) -> Ty {
+    /// Get the resulting type of binary operation.
+    pub fn binop_ty(&self, bin_op: BinOp, rhs: Ty, lhs: Ty) -> Ty {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         let rhs_internal = rhs.internal(&mut *tables, tcx);
@@ -809,16 +890,31 @@ impl<'tcx> Context for TablesWrapper<'tcx> {
         ty.stable(&mut *tables)
     }
 
-    fn unop_ty(&self, un_op: UnOp, arg: Ty) -> Ty {
+    /// Get the resulting type of unary operation.
+    pub fn unop_ty(&self, un_op: UnOp, arg: Ty) -> Ty {
         let mut tables = self.0.borrow_mut();
         let tcx = tables.tcx;
         let arg_internal = arg.internal(&mut *tables, tcx);
         let ty = un_op.internal(&mut *tables, tcx).ty(tcx, arg_internal);
         ty.stable(&mut *tables)
     }
-}
 
-pub(crate) struct TablesWrapper<'tcx>(pub RefCell<Tables<'tcx>>);
+    /// Get all associated items of a definition.
+    pub fn associated_items(&self, def_id: stable_mir::DefId) -> stable_mir::AssocItems {
+        let mut tables = self.0.borrow_mut();
+        let tcx = tables.tcx;
+        let def_id = tables[def_id];
+        let assoc_items = if tcx.is_trait_alias(def_id) {
+            Vec::new()
+        } else {
+            tcx.associated_item_def_ids(def_id)
+                .iter()
+                .map(|did| tcx.associated_item(*did).stable(&mut *tables))
+                .collect()
+        };
+        assoc_items
+    }
+}
 
 /// Implement error handling for extracting function ABI information.
 impl<'tcx> FnAbiOfHelpers<'tcx> for Tables<'tcx> {
