@@ -7,17 +7,12 @@
 //!
 //! [intel64_ref]: http://www.intel.de/content/dam/www/public/us/en/documents/manuals/64-ia-32-architectures-software-developer-instruction-set-reference-manual-325383.pdf
 
+use crate::core_arch::simd::*;
 use crate::core_arch::x86::__m128i;
 use crate::core_arch::x86::__m256i;
 use crate::core_arch::x86::__m512i;
-use crate::core_arch::x86::__mmask16;
 use crate::core_arch::x86::__mmask8;
-use crate::core_arch::x86::_mm256_setzero_si256;
-use crate::core_arch::x86::_mm512_setzero_si512;
-use crate::core_arch::x86::_mm_setzero_si128;
-use crate::core_arch::x86::m128iExt;
-use crate::core_arch::x86::m256iExt;
-use crate::core_arch::x86::m512iExt;
+use crate::core_arch::x86::__mmask16;
 use crate::intrinsics::simd::{simd_ctpop, simd_select_bitmask};
 use crate::mem::transmute;
 
@@ -31,8 +26,8 @@ use stdarch_test::assert_instr;
 #[target_feature(enable = "avx512vpopcntdq")]
 #[unstable(feature = "stdarch_x86_avx512", issue = "111137")]
 #[cfg_attr(test, assert_instr(vpopcntd))]
-pub unsafe fn _mm512_popcnt_epi32(a: __m512i) -> __m512i {
-    transmute(simd_ctpop(a.as_i32x16()))
+pub fn _mm512_popcnt_epi32(a: __m512i) -> __m512i {
+    unsafe { transmute(simd_ctpop(a.as_i32x16())) }
 }
 
 /// For each packed 32-bit integer maps the value to the number of logical 1 bits.
@@ -45,9 +40,14 @@ pub unsafe fn _mm512_popcnt_epi32(a: __m512i) -> __m512i {
 #[target_feature(enable = "avx512vpopcntdq")]
 #[unstable(feature = "stdarch_x86_avx512", issue = "111137")]
 #[cfg_attr(test, assert_instr(vpopcntd))]
-pub unsafe fn _mm512_maskz_popcnt_epi32(k: __mmask16, a: __m512i) -> __m512i {
-    let zero = _mm512_setzero_si512().as_i32x16();
-    transmute(simd_select_bitmask(k, simd_ctpop(a.as_i32x16()), zero))
+pub fn _mm512_maskz_popcnt_epi32(k: __mmask16, a: __m512i) -> __m512i {
+    unsafe {
+        transmute(simd_select_bitmask(
+            k,
+            simd_ctpop(a.as_i32x16()),
+            i32x16::ZERO,
+        ))
+    }
 }
 
 /// For each packed 32-bit integer maps the value to the number of logical 1 bits.
@@ -60,12 +60,14 @@ pub unsafe fn _mm512_maskz_popcnt_epi32(k: __mmask16, a: __m512i) -> __m512i {
 #[target_feature(enable = "avx512vpopcntdq")]
 #[unstable(feature = "stdarch_x86_avx512", issue = "111137")]
 #[cfg_attr(test, assert_instr(vpopcntd))]
-pub unsafe fn _mm512_mask_popcnt_epi32(src: __m512i, k: __mmask16, a: __m512i) -> __m512i {
-    transmute(simd_select_bitmask(
-        k,
-        simd_ctpop(a.as_i32x16()),
-        src.as_i32x16(),
-    ))
+pub fn _mm512_mask_popcnt_epi32(src: __m512i, k: __mmask16, a: __m512i) -> __m512i {
+    unsafe {
+        transmute(simd_select_bitmask(
+            k,
+            simd_ctpop(a.as_i32x16()),
+            src.as_i32x16(),
+        ))
+    }
 }
 
 /// For each packed 32-bit integer maps the value to the number of logical 1 bits.
@@ -75,8 +77,8 @@ pub unsafe fn _mm512_mask_popcnt_epi32(src: __m512i, k: __mmask16, a: __m512i) -
 #[target_feature(enable = "avx512vpopcntdq,avx512vl")]
 #[unstable(feature = "stdarch_x86_avx512", issue = "111137")]
 #[cfg_attr(test, assert_instr(vpopcntd))]
-pub unsafe fn _mm256_popcnt_epi32(a: __m256i) -> __m256i {
-    transmute(simd_ctpop(a.as_i32x8()))
+pub fn _mm256_popcnt_epi32(a: __m256i) -> __m256i {
+    unsafe { transmute(simd_ctpop(a.as_i32x8())) }
 }
 
 /// For each packed 32-bit integer maps the value to the number of logical 1 bits.
@@ -89,9 +91,14 @@ pub unsafe fn _mm256_popcnt_epi32(a: __m256i) -> __m256i {
 #[target_feature(enable = "avx512vpopcntdq,avx512vl")]
 #[unstable(feature = "stdarch_x86_avx512", issue = "111137")]
 #[cfg_attr(test, assert_instr(vpopcntd))]
-pub unsafe fn _mm256_maskz_popcnt_epi32(k: __mmask8, a: __m256i) -> __m256i {
-    let zero = _mm256_setzero_si256().as_i32x8();
-    transmute(simd_select_bitmask(k, simd_ctpop(a.as_i32x8()), zero))
+pub fn _mm256_maskz_popcnt_epi32(k: __mmask8, a: __m256i) -> __m256i {
+    unsafe {
+        transmute(simd_select_bitmask(
+            k,
+            simd_ctpop(a.as_i32x8()),
+            i32x8::ZERO,
+        ))
+    }
 }
 
 /// For each packed 32-bit integer maps the value to the number of logical 1 bits.
@@ -104,12 +111,14 @@ pub unsafe fn _mm256_maskz_popcnt_epi32(k: __mmask8, a: __m256i) -> __m256i {
 #[target_feature(enable = "avx512vpopcntdq,avx512vl")]
 #[unstable(feature = "stdarch_x86_avx512", issue = "111137")]
 #[cfg_attr(test, assert_instr(vpopcntd))]
-pub unsafe fn _mm256_mask_popcnt_epi32(src: __m256i, k: __mmask8, a: __m256i) -> __m256i {
-    transmute(simd_select_bitmask(
-        k,
-        simd_ctpop(a.as_i32x8()),
-        src.as_i32x8(),
-    ))
+pub fn _mm256_mask_popcnt_epi32(src: __m256i, k: __mmask8, a: __m256i) -> __m256i {
+    unsafe {
+        transmute(simd_select_bitmask(
+            k,
+            simd_ctpop(a.as_i32x8()),
+            src.as_i32x8(),
+        ))
+    }
 }
 
 /// For each packed 32-bit integer maps the value to the number of logical 1 bits.
@@ -119,8 +128,8 @@ pub unsafe fn _mm256_mask_popcnt_epi32(src: __m256i, k: __mmask8, a: __m256i) ->
 #[target_feature(enable = "avx512vpopcntdq,avx512vl")]
 #[unstable(feature = "stdarch_x86_avx512", issue = "111137")]
 #[cfg_attr(test, assert_instr(vpopcntd))]
-pub unsafe fn _mm_popcnt_epi32(a: __m128i) -> __m128i {
-    transmute(simd_ctpop(a.as_i32x4()))
+pub fn _mm_popcnt_epi32(a: __m128i) -> __m128i {
+    unsafe { transmute(simd_ctpop(a.as_i32x4())) }
 }
 
 /// For each packed 32-bit integer maps the value to the number of logical 1 bits.
@@ -133,9 +142,14 @@ pub unsafe fn _mm_popcnt_epi32(a: __m128i) -> __m128i {
 #[target_feature(enable = "avx512vpopcntdq,avx512vl")]
 #[unstable(feature = "stdarch_x86_avx512", issue = "111137")]
 #[cfg_attr(test, assert_instr(vpopcntd))]
-pub unsafe fn _mm_maskz_popcnt_epi32(k: __mmask8, a: __m128i) -> __m128i {
-    let zero = _mm_setzero_si128().as_i32x4();
-    transmute(simd_select_bitmask(k, simd_ctpop(a.as_i32x4()), zero))
+pub fn _mm_maskz_popcnt_epi32(k: __mmask8, a: __m128i) -> __m128i {
+    unsafe {
+        transmute(simd_select_bitmask(
+            k,
+            simd_ctpop(a.as_i32x4()),
+            i32x4::ZERO,
+        ))
+    }
 }
 
 /// For each packed 32-bit integer maps the value to the number of logical 1 bits.
@@ -148,12 +162,14 @@ pub unsafe fn _mm_maskz_popcnt_epi32(k: __mmask8, a: __m128i) -> __m128i {
 #[target_feature(enable = "avx512vpopcntdq,avx512vl")]
 #[unstable(feature = "stdarch_x86_avx512", issue = "111137")]
 #[cfg_attr(test, assert_instr(vpopcntd))]
-pub unsafe fn _mm_mask_popcnt_epi32(src: __m128i, k: __mmask8, a: __m128i) -> __m128i {
-    transmute(simd_select_bitmask(
-        k,
-        simd_ctpop(a.as_i32x4()),
-        src.as_i32x4(),
-    ))
+pub fn _mm_mask_popcnt_epi32(src: __m128i, k: __mmask8, a: __m128i) -> __m128i {
+    unsafe {
+        transmute(simd_select_bitmask(
+            k,
+            simd_ctpop(a.as_i32x4()),
+            src.as_i32x4(),
+        ))
+    }
 }
 
 /// For each packed 64-bit integer maps the value to the number of logical 1 bits.
@@ -163,8 +179,8 @@ pub unsafe fn _mm_mask_popcnt_epi32(src: __m128i, k: __mmask8, a: __m128i) -> __
 #[target_feature(enable = "avx512vpopcntdq")]
 #[unstable(feature = "stdarch_x86_avx512", issue = "111137")]
 #[cfg_attr(test, assert_instr(vpopcntq))]
-pub unsafe fn _mm512_popcnt_epi64(a: __m512i) -> __m512i {
-    transmute(simd_ctpop(a.as_i64x8()))
+pub fn _mm512_popcnt_epi64(a: __m512i) -> __m512i {
+    unsafe { transmute(simd_ctpop(a.as_i64x8())) }
 }
 
 /// For each packed 64-bit integer maps the value to the number of logical 1 bits.
@@ -177,9 +193,14 @@ pub unsafe fn _mm512_popcnt_epi64(a: __m512i) -> __m512i {
 #[target_feature(enable = "avx512vpopcntdq")]
 #[unstable(feature = "stdarch_x86_avx512", issue = "111137")]
 #[cfg_attr(test, assert_instr(vpopcntq))]
-pub unsafe fn _mm512_maskz_popcnt_epi64(k: __mmask8, a: __m512i) -> __m512i {
-    let zero = _mm512_setzero_si512().as_i64x8();
-    transmute(simd_select_bitmask(k, simd_ctpop(a.as_i64x8()), zero))
+pub fn _mm512_maskz_popcnt_epi64(k: __mmask8, a: __m512i) -> __m512i {
+    unsafe {
+        transmute(simd_select_bitmask(
+            k,
+            simd_ctpop(a.as_i64x8()),
+            i64x8::ZERO,
+        ))
+    }
 }
 
 /// For each packed 64-bit integer maps the value to the number of logical 1 bits.
@@ -192,12 +213,14 @@ pub unsafe fn _mm512_maskz_popcnt_epi64(k: __mmask8, a: __m512i) -> __m512i {
 #[target_feature(enable = "avx512vpopcntdq")]
 #[unstable(feature = "stdarch_x86_avx512", issue = "111137")]
 #[cfg_attr(test, assert_instr(vpopcntq))]
-pub unsafe fn _mm512_mask_popcnt_epi64(src: __m512i, k: __mmask8, a: __m512i) -> __m512i {
-    transmute(simd_select_bitmask(
-        k,
-        simd_ctpop(a.as_i64x8()),
-        src.as_i64x8(),
-    ))
+pub fn _mm512_mask_popcnt_epi64(src: __m512i, k: __mmask8, a: __m512i) -> __m512i {
+    unsafe {
+        transmute(simd_select_bitmask(
+            k,
+            simd_ctpop(a.as_i64x8()),
+            src.as_i64x8(),
+        ))
+    }
 }
 
 /// For each packed 64-bit integer maps the value to the number of logical 1 bits.
@@ -207,8 +230,8 @@ pub unsafe fn _mm512_mask_popcnt_epi64(src: __m512i, k: __mmask8, a: __m512i) ->
 #[target_feature(enable = "avx512vpopcntdq,avx512vl")]
 #[unstable(feature = "stdarch_x86_avx512", issue = "111137")]
 #[cfg_attr(test, assert_instr(vpopcntq))]
-pub unsafe fn _mm256_popcnt_epi64(a: __m256i) -> __m256i {
-    transmute(simd_ctpop(a.as_i64x4()))
+pub fn _mm256_popcnt_epi64(a: __m256i) -> __m256i {
+    unsafe { transmute(simd_ctpop(a.as_i64x4())) }
 }
 
 /// For each packed 64-bit integer maps the value to the number of logical 1 bits.
@@ -221,9 +244,14 @@ pub unsafe fn _mm256_popcnt_epi64(a: __m256i) -> __m256i {
 #[target_feature(enable = "avx512vpopcntdq,avx512vl")]
 #[unstable(feature = "stdarch_x86_avx512", issue = "111137")]
 #[cfg_attr(test, assert_instr(vpopcntq))]
-pub unsafe fn _mm256_maskz_popcnt_epi64(k: __mmask8, a: __m256i) -> __m256i {
-    let zero = _mm256_setzero_si256().as_i64x4();
-    transmute(simd_select_bitmask(k, simd_ctpop(a.as_i64x4()), zero))
+pub fn _mm256_maskz_popcnt_epi64(k: __mmask8, a: __m256i) -> __m256i {
+    unsafe {
+        transmute(simd_select_bitmask(
+            k,
+            simd_ctpop(a.as_i64x4()),
+            i64x4::ZERO,
+        ))
+    }
 }
 
 /// For each packed 64-bit integer maps the value to the number of logical 1 bits.
@@ -236,12 +264,14 @@ pub unsafe fn _mm256_maskz_popcnt_epi64(k: __mmask8, a: __m256i) -> __m256i {
 #[target_feature(enable = "avx512vpopcntdq,avx512vl")]
 #[unstable(feature = "stdarch_x86_avx512", issue = "111137")]
 #[cfg_attr(test, assert_instr(vpopcntq))]
-pub unsafe fn _mm256_mask_popcnt_epi64(src: __m256i, k: __mmask8, a: __m256i) -> __m256i {
-    transmute(simd_select_bitmask(
-        k,
-        simd_ctpop(a.as_i64x4()),
-        src.as_i64x4(),
-    ))
+pub fn _mm256_mask_popcnt_epi64(src: __m256i, k: __mmask8, a: __m256i) -> __m256i {
+    unsafe {
+        transmute(simd_select_bitmask(
+            k,
+            simd_ctpop(a.as_i64x4()),
+            src.as_i64x4(),
+        ))
+    }
 }
 
 /// For each packed 64-bit integer maps the value to the number of logical 1 bits.
@@ -251,8 +281,8 @@ pub unsafe fn _mm256_mask_popcnt_epi64(src: __m256i, k: __mmask8, a: __m256i) ->
 #[target_feature(enable = "avx512vpopcntdq,avx512vl")]
 #[unstable(feature = "stdarch_x86_avx512", issue = "111137")]
 #[cfg_attr(test, assert_instr(vpopcntq))]
-pub unsafe fn _mm_popcnt_epi64(a: __m128i) -> __m128i {
-    transmute(simd_ctpop(a.as_i64x2()))
+pub fn _mm_popcnt_epi64(a: __m128i) -> __m128i {
+    unsafe { transmute(simd_ctpop(a.as_i64x2())) }
 }
 
 /// For each packed 64-bit integer maps the value to the number of logical 1 bits.
@@ -265,9 +295,14 @@ pub unsafe fn _mm_popcnt_epi64(a: __m128i) -> __m128i {
 #[target_feature(enable = "avx512vpopcntdq,avx512vl")]
 #[unstable(feature = "stdarch_x86_avx512", issue = "111137")]
 #[cfg_attr(test, assert_instr(vpopcntq))]
-pub unsafe fn _mm_maskz_popcnt_epi64(k: __mmask8, a: __m128i) -> __m128i {
-    let zero = _mm_setzero_si128().as_i64x2();
-    transmute(simd_select_bitmask(k, simd_ctpop(a.as_i64x2()), zero))
+pub fn _mm_maskz_popcnt_epi64(k: __mmask8, a: __m128i) -> __m128i {
+    unsafe {
+        transmute(simd_select_bitmask(
+            k,
+            simd_ctpop(a.as_i64x2()),
+            i64x2::ZERO,
+        ))
+    }
 }
 
 /// For each packed 64-bit integer maps the value to the number of logical 1 bits.
@@ -280,12 +315,14 @@ pub unsafe fn _mm_maskz_popcnt_epi64(k: __mmask8, a: __m128i) -> __m128i {
 #[target_feature(enable = "avx512vpopcntdq,avx512vl")]
 #[unstable(feature = "stdarch_x86_avx512", issue = "111137")]
 #[cfg_attr(test, assert_instr(vpopcntq))]
-pub unsafe fn _mm_mask_popcnt_epi64(src: __m128i, k: __mmask8, a: __m128i) -> __m128i {
-    transmute(simd_select_bitmask(
-        k,
-        simd_ctpop(a.as_i64x2()),
-        src.as_i64x2(),
-    ))
+pub fn _mm_mask_popcnt_epi64(src: __m128i, k: __mmask8, a: __m128i) -> __m128i {
+    unsafe {
+        transmute(simd_select_bitmask(
+            k,
+            simd_ctpop(a.as_i64x2()),
+            src.as_i64x2(),
+        ))
+    }
 }
 
 #[cfg(test)]
