@@ -1,10 +1,9 @@
 //! Run-time feature detection for Aarch64 on Linux.
 
 use super::auxvec;
-use crate::detect::{bit, cache, Feature};
+use crate::detect::{Feature, bit, cache};
 
-/// Try to read the features from the auxiliary vector, and if that fails, try
-/// to read them from /proc/cpuinfo.
+/// Try to read the features from the auxiliary vector.
 pub(crate) fn detect_features() -> cache::Initializer {
     #[cfg(target_os = "android")]
     let is_exynos9810 = {
@@ -14,10 +13,7 @@ pub(crate) fn detect_features() -> cache::Initializer {
         // https://reviews.llvm.org/D114523
         let mut arch = [0_u8; libc::PROP_VALUE_MAX as usize];
         let len = unsafe {
-            libc::__system_property_get(
-                b"ro.arch\0".as_ptr() as *const libc::c_char,
-                arch.as_mut_ptr() as *mut libc::c_char,
-            )
+            libc::__system_property_get(c"ro.arch".as_ptr(), arch.as_mut_ptr() as *mut libc::c_char)
         };
         // On Exynos, ro.arch is not available on Android 12+, but it is fine
         // because Android 9+ includes the fix.
@@ -30,11 +26,6 @@ pub(crate) fn detect_features() -> cache::Initializer {
         let hwcap: AtHwcap = auxv.into();
         return hwcap.cache(is_exynos9810);
     }
-    #[cfg(feature = "std_detect_file_io")]
-    if let Ok(c) = super::cpuinfo::CpuInfo::new() {
-        let hwcap: AtHwcap = c.into();
-        return hwcap.cache(is_exynos9810);
-    }
     cache::Initializer::default()
 }
 
@@ -83,11 +74,11 @@ struct AtHwcap {
     dcpodp: bool,
     sve2: bool,
     sveaes: bool,
-    // svepmull: No LLVM support.
+    svepmull: bool,
     svebitperm: bool,
     svesha3: bool,
     svesm4: bool,
-    // flagm2: No LLVM support.
+    flagm2: bool,
     frint: bool,
     // svei8mm: See i8mm feature.
     svef32mm: bool,
@@ -99,6 +90,51 @@ struct AtHwcap {
     rng: bool,
     bti: bool,
     mte: bool,
+    ecv: bool,
+    // afp: bool,
+    // rpres: bool,
+    // mte3: bool,
+    sme: bool,
+    smei16i64: bool,
+    smef64f64: bool,
+    // smei8i32: bool,
+    // smef16f32: bool,
+    // smeb16f32: bool,
+    // smef32f32: bool,
+    smefa64: bool,
+    wfxt: bool,
+    // ebf16: bool,
+    // sveebf16: bool,
+    cssc: bool,
+    // rprfm: bool,
+    sve2p1: bool,
+    sme2: bool,
+    sme2p1: bool,
+    // smei16i32: bool,
+    // smebi32i32: bool,
+    smeb16b16: bool,
+    smef16f16: bool,
+    mops: bool,
+    hbc: bool,
+    sveb16b16: bool,
+    lrcpc3: bool,
+    lse128: bool,
+    fpmr: bool,
+    lut: bool,
+    faminmax: bool,
+    f8cvt: bool,
+    f8fma: bool,
+    f8dp4: bool,
+    f8dp2: bool,
+    f8e4m3: bool,
+    f8e5m2: bool,
+    smelutv2: bool,
+    smef8f16: bool,
+    smef8f32: bool,
+    smesf8fma: bool,
+    smesf8dp4: bool,
+    smesf8dp2: bool,
+    // pauthlr: bool,
 }
 
 impl From<auxvec::AuxVec> for AtHwcap {
@@ -137,14 +173,16 @@ impl From<auxvec::AuxVec> for AtHwcap {
             sb: bit::test(auxv.hwcap, 29),
             paca: bit::test(auxv.hwcap, 30),
             pacg: bit::test(auxv.hwcap, 31),
+
+            // AT_HWCAP2
             dcpodp: bit::test(auxv.hwcap2, 0),
             sve2: bit::test(auxv.hwcap2, 1),
             sveaes: bit::test(auxv.hwcap2, 2),
-            // svepmull: bit::test(auxv.hwcap2, 3),
+            svepmull: bit::test(auxv.hwcap2, 3),
             svebitperm: bit::test(auxv.hwcap2, 4),
             svesha3: bit::test(auxv.hwcap2, 5),
             svesm4: bit::test(auxv.hwcap2, 6),
-            // flagm2: bit::test(auxv.hwcap2, 7),
+            flagm2: bit::test(auxv.hwcap2, 7),
             frint: bit::test(auxv.hwcap2, 8),
             // svei8mm: bit::test(auxv.hwcap2, 9),
             svef32mm: bit::test(auxv.hwcap2, 10),
@@ -156,70 +194,51 @@ impl From<auxvec::AuxVec> for AtHwcap {
             rng: bit::test(auxv.hwcap2, 16),
             bti: bit::test(auxv.hwcap2, 17),
             mte: bit::test(auxv.hwcap2, 18),
-        }
-    }
-}
-
-#[cfg(feature = "std_detect_file_io")]
-impl From<super::cpuinfo::CpuInfo> for AtHwcap {
-    /// Reads AtHwcap from /proc/cpuinfo .
-    fn from(c: super::cpuinfo::CpuInfo) -> Self {
-        let f = &c.field("Features");
-        AtHwcap {
-            // 64-bit names. FIXME: In 32-bit compatibility mode /proc/cpuinfo will
-            // map some of the 64-bit names to some 32-bit feature names. This does not
-            // cover that yet.
-            fp: f.has("fp"),
-            asimd: f.has("asimd"),
-            // evtstrm: f.has("evtstrm"),
-            aes: f.has("aes"),
-            pmull: f.has("pmull"),
-            sha1: f.has("sha1"),
-            sha2: f.has("sha2"),
-            crc32: f.has("crc32"),
-            atomics: f.has("atomics"),
-            fphp: f.has("fphp"),
-            asimdhp: f.has("asimdhp"),
-            // cpuid: f.has("cpuid"),
-            asimdrdm: f.has("asimdrdm"),
-            jscvt: f.has("jscvt"),
-            fcma: f.has("fcma"),
-            lrcpc: f.has("lrcpc"),
-            dcpop: f.has("dcpop"),
-            sha3: f.has("sha3"),
-            sm3: f.has("sm3"),
-            sm4: f.has("sm4"),
-            asimddp: f.has("asimddp"),
-            sha512: f.has("sha512"),
-            sve: f.has("sve"),
-            fhm: f.has("asimdfhm"),
-            dit: f.has("dit"),
-            uscat: f.has("uscat"),
-            ilrcpc: f.has("ilrcpc"),
-            flagm: f.has("flagm"),
-            ssbs: f.has("ssbs"),
-            sb: f.has("sb"),
-            paca: f.has("paca"),
-            pacg: f.has("pacg"),
-            dcpodp: f.has("dcpodp"),
-            sve2: f.has("sve2"),
-            sveaes: f.has("sveaes"),
-            // svepmull: f.has("svepmull"),
-            svebitperm: f.has("svebitperm"),
-            svesha3: f.has("svesha3"),
-            svesm4: f.has("svesm4"),
-            // flagm2: f.has("flagm2"),
-            frint: f.has("frint"),
-            // svei8mm: f.has("svei8mm"),
-            svef32mm: f.has("svef32mm"),
-            svef64mm: f.has("svef64mm"),
-            // svebf16: f.has("svebf16"),
-            i8mm: f.has("i8mm"),
-            bf16: f.has("bf16"),
-            // dgh: f.has("dgh"),
-            rng: f.has("rng"),
-            bti: f.has("bti"),
-            mte: f.has("mte"),
+            ecv: bit::test(auxv.hwcap2, 19),
+            // afp: bit::test(auxv.hwcap2, 20),
+            // rpres: bit::test(auxv.hwcap2, 21),
+            // mte3: bit::test(auxv.hwcap2, 22),
+            sme: bit::test(auxv.hwcap2, 23),
+            smei16i64: bit::test(auxv.hwcap2, 24),
+            smef64f64: bit::test(auxv.hwcap2, 25),
+            // smei8i32: bit::test(auxv.hwcap2, 26),
+            // smef16f32: bit::test(auxv.hwcap2, 27),
+            // smeb16f32: bit::test(auxv.hwcap2, 28),
+            // smef32f32: bit::test(auxv.hwcap2, 29),
+            smefa64: bit::test(auxv.hwcap2, 30),
+            wfxt: bit::test(auxv.hwcap2, 31),
+            // ebf16: bit::test(auxv.hwcap2, 32),
+            // sveebf16: bit::test(auxv.hwcap2, 33),
+            cssc: bit::test(auxv.hwcap2, 34),
+            // rprfm: bit::test(auxv.hwcap2, 35),
+            sve2p1: bit::test(auxv.hwcap2, 36),
+            sme2: bit::test(auxv.hwcap2, 37),
+            sme2p1: bit::test(auxv.hwcap2, 38),
+            // smei16i32: bit::test(auxv.hwcap2, 39),
+            // smebi32i32: bit::test(auxv.hwcap2, 40),
+            smeb16b16: bit::test(auxv.hwcap2, 41),
+            smef16f16: bit::test(auxv.hwcap2, 42),
+            mops: bit::test(auxv.hwcap2, 43),
+            hbc: bit::test(auxv.hwcap2, 44),
+            sveb16b16: bit::test(auxv.hwcap2, 45),
+            lrcpc3: bit::test(auxv.hwcap2, 46),
+            lse128: bit::test(auxv.hwcap2, 47),
+            fpmr: bit::test(auxv.hwcap2, 48),
+            lut: bit::test(auxv.hwcap2, 49),
+            faminmax: bit::test(auxv.hwcap2, 50),
+            f8cvt: bit::test(auxv.hwcap2, 51),
+            f8fma: bit::test(auxv.hwcap2, 52),
+            f8dp4: bit::test(auxv.hwcap2, 53),
+            f8dp2: bit::test(auxv.hwcap2, 54),
+            f8e4m3: bit::test(auxv.hwcap2, 55),
+            f8e5m2: bit::test(auxv.hwcap2, 56),
+            smelutv2: bit::test(auxv.hwcap2, 57),
+            smef8f16: bit::test(auxv.hwcap2, 58),
+            smef8f32: bit::test(auxv.hwcap2, 59),
+            smesf8fma: bit::test(auxv.hwcap2, 60),
+            smesf8dp4: bit::test(auxv.hwcap2, 61),
+            smesf8dp2: bit::test(auxv.hwcap2, 62),
+            // pauthlr: bit::test(auxv.hwcap2, ??),
         }
     }
 }
@@ -267,15 +286,20 @@ impl AtHwcap {
             enable_feature(Feature::crc, self.crc32);
             enable_feature(Feature::lse, self.atomics);
             enable_feature(Feature::lse2, self.uscat);
+            enable_feature(Feature::lse128, self.lse128 && self.atomics);
             enable_feature(Feature::rcpc, self.lrcpc);
             // RCPC2 (rcpc-immo in LLVM) requires RCPC support
-            enable_feature(Feature::rcpc2, self.ilrcpc && self.lrcpc);
+            let rcpc2 = self.ilrcpc && self.lrcpc;
+            enable_feature(Feature::rcpc2, rcpc2);
+            enable_feature(Feature::rcpc3, self.lrcpc3 && rcpc2);
             enable_feature(Feature::dit, self.dit);
             enable_feature(Feature::flagm, self.flagm);
+            enable_feature(Feature::flagm2, self.flagm2);
             enable_feature(Feature::ssbs, self.ssbs);
             enable_feature(Feature::sb, self.sb);
             enable_feature(Feature::paca, self.paca);
             enable_feature(Feature::pacg, self.pacg);
+            // enable_feature(Feature::pauth_lr, self.pauthlr);
             enable_feature(Feature::dpb, self.dcpop);
             enable_feature(Feature::dpb2, self.dcpodp);
             enable_feature(Feature::rand, self.rng);
@@ -317,8 +341,12 @@ impl AtHwcap {
             // SVE2 requires SVE
             let sve2 = self.sve2 && self.sve && asimd;
             enable_feature(Feature::sve2, sve2);
+            enable_feature(Feature::sve2p1, self.sve2p1 && sve2);
             // SVE2 extensions require SVE2 and crypto features
-            enable_feature(Feature::sve2_aes, self.sveaes && sve2 && self.aes);
+            enable_feature(
+                Feature::sve2_aes,
+                self.sveaes && self.svepmull && sve2 && self.aes,
+            );
             enable_feature(
                 Feature::sve2_sm4,
                 self.svesm4 && sve2 && self.sm3 && self.sm4,
@@ -328,6 +356,44 @@ impl AtHwcap {
                 self.svesha3 && sve2 && self.sha512 && self.sha3 && self.sha1 && self.sha2,
             );
             enable_feature(Feature::sve2_bitperm, self.svebitperm && self.sve2);
+            enable_feature(Feature::sve_b16b16, self.bf16 && self.sveb16b16);
+            enable_feature(Feature::hbc, self.hbc);
+            enable_feature(Feature::mops, self.mops);
+            enable_feature(Feature::ecv, self.ecv);
+            enable_feature(Feature::lut, self.lut);
+            enable_feature(Feature::cssc, self.cssc);
+            enable_feature(Feature::fpmr, self.fpmr);
+            enable_feature(Feature::faminmax, self.faminmax);
+            let fp8 = self.f8cvt && self.faminmax && self.lut && self.bf16;
+            enable_feature(Feature::fp8, fp8);
+            let fp8fma = self.f8fma && fp8;
+            enable_feature(Feature::fp8fma, fp8fma);
+            let fp8dot4 = self.f8dp4 && fp8fma;
+            enable_feature(Feature::fp8dot4, fp8dot4);
+            enable_feature(Feature::fp8dot2, self.f8dp2 && fp8dot4);
+            enable_feature(Feature::wfxt, self.wfxt);
+            let sme = self.sme && self.bf16;
+            enable_feature(Feature::sme, sme);
+            enable_feature(Feature::sme_i16i64, self.smei16i64 && sme);
+            enable_feature(Feature::sme_f64f64, self.smef64f64 && sme);
+            enable_feature(Feature::sme_fa64, self.smefa64 && sme && sve2);
+            let sme2 = self.sme2 && sme;
+            enable_feature(Feature::sme2, sme2);
+            enable_feature(Feature::sme2p1, self.sme2p1 && sme2);
+            enable_feature(
+                Feature::sme_b16b16,
+                sme2 && self.bf16 && self.sveb16b16 && self.smeb16b16,
+            );
+            enable_feature(Feature::sme_f16f16, self.smef16f16 && sme2);
+            enable_feature(Feature::sme_lutv2, self.smelutv2);
+            let sme_f8f32 = self.smef8f32 && sme2 && fp8;
+            enable_feature(Feature::sme_f8f32, sme_f8f32);
+            enable_feature(Feature::sme_f8f16, self.smef8f16 && sme_f8f32);
+            let ssve_fp8fma = self.smesf8fma && sme2 && fp8;
+            enable_feature(Feature::ssve_fp8fma, ssve_fp8fma);
+            let ssve_fp8dot4 = self.smesf8dp4 && ssve_fp8fma;
+            enable_feature(Feature::ssve_fp8dot4, ssve_fp8dot4);
+            enable_feature(Feature::ssve_fp8dot2, self.smesf8dp2 && ssve_fp8dot4);
         }
         value
     }
