@@ -1,10 +1,9 @@
 //! Run-time feature detection for PowerPC on Linux.
 
 use super::auxvec;
-use crate::detect::{cache, Feature};
+use crate::detect::{Feature, cache};
 
-/// Try to read the features from the auxiliary vector, and if that fails, try
-/// to read them from /proc/cpuinfo.
+/// Try to read the features from the auxiliary vector.
 pub(crate) fn detect_features() -> cache::Initializer {
     let mut value = cache::Initializer::default();
     let enable_feature = |value: &mut cache::Initializer, f, enable| {
@@ -21,15 +20,15 @@ pub(crate) fn detect_features() -> cache::Initializer {
         // index of the bit to test like in ARM and Aarch64)
         enable_feature(&mut value, Feature::altivec, auxv.hwcap & 0x10000000 != 0);
         enable_feature(&mut value, Feature::vsx, auxv.hwcap & 0x00000080 != 0);
-        enable_feature(&mut value, Feature::power8, auxv.hwcap2 & 0x80000000 != 0);
-        return value;
-    }
-
-    // PowerPC's /proc/cpuinfo lacks a proper Feature field,
-    // but `altivec` support is indicated in the `cpu` field.
-    #[cfg(feature = "std_detect_file_io")]
-    if let Ok(c) = super::cpuinfo::CpuInfo::new() {
-        enable_feature(&mut value, Feature::altivec, c.field("cpu").has("altivec"));
+        let power8_features = auxv.hwcap2 & 0x80000000 != 0;
+        enable_feature(&mut value, Feature::power8, power8_features);
+        enable_feature(&mut value, Feature::power8_altivec, power8_features);
+        enable_feature(&mut value, Feature::power8_crypto, power8_features);
+        enable_feature(&mut value, Feature::power8_vector, power8_features);
+        let power9_features = auxv.hwcap2 & 0x00800000 != 0;
+        enable_feature(&mut value, Feature::power9, power9_features);
+        enable_feature(&mut value, Feature::power9_altivec, power9_features);
+        enable_feature(&mut value, Feature::power9_vector, power9_features);
         return value;
     }
     value
