@@ -29,7 +29,7 @@ fn normalize(mut symbol: &str) -> String {
 
     // Normalize to no leading underscore to handle platforms that may
     // inject extra ones in symbol names.
-    while symbol.starts_with('_') {
+    while symbol.starts_with('_') || symbol.starts_with('.') {
         symbol.remove(0);
     }
     // Windows/x86 has a suffix such as @@4.
@@ -140,7 +140,7 @@ fn parse(output: &str) -> HashSet<Function> {
                     .filter(|&x| !x.is_empty())
                     .skip(1)
                     .map(str::to_lowercase)
-                    .skip_while(|s| *s == "lock") // skip x86-specific prefix
+                    .skip_while(|s| matches!(&**s, "lock" | "vex")) // skip x86-specific prefix
                     .collect::<Vec<String>>()
             } else {
                 // objdump with --no-show-raw-insn
@@ -150,8 +150,8 @@ fn parse(output: &str) -> HashSet<Function> {
                 instruction
                     .split_whitespace()
                     .skip(1)
-                    .skip_while(|s| *s == "lock" || *s == "{evex}") // skip x86-specific prefix
-                    .map(std::string::ToString::to_string)
+                    .skip_while(|s| matches!(*s, "lock" | "{evex}" | "{vex}")) // skip x86-specific prefix
+                    .map(ToString::to_string)
                     .collect::<Vec<String>>()
             };
 
@@ -169,7 +169,7 @@ fn parse(output: &str) -> HashSet<Function> {
                     }
                 }
                 match (parts.first(), parts.last()) {
-                    (Some(instr), Some(last_arg)) if is_shll(&instr) && last_arg == "#0" => {
+                    (Some(instr), Some(last_arg)) if is_shll(instr) && last_arg == "#0" => {
                         assert_eq!(parts.len(), 4);
                         let mut new_parts = Vec::with_capacity(3);
                         let new_instr = format!("{}{}{}", &instr[..1], "xtl", &instr[5..]);
@@ -185,7 +185,12 @@ fn parse(output: &str) -> HashSet<Function> {
                     _ => {}
                 };
             }
+
             instructions.push(parts.join(" "));
+            if matches!(&**instructions.last().unwrap(), "ret" | "retq") {
+                cached_header = None;
+                break;
+            }
         }
         let function = Function {
             name: symbol,
