@@ -15,9 +15,7 @@ df -h
 echo
 
 echo "biggest files in the working dir:"
-set +o pipefail
-du . | sort -nr | head -n100
-set -o pipefail
+du . | sort -n | tail -n100 | sort -nr # because piping sort to head gives a broken pipe
 echo
 
 if isMacOS
