@@ -3,6 +3,6 @@ fn main() {
     let s1 = String::from("hello");
     let s2 = s1;
 
-    println!("{}, world!", s1);
+    println!("{s1}, world!");
     // ANCHOR_END: here
 }
