@@ -11,8 +11,8 @@ fn main() {
                 Ok(fc) => fc,
                 Err(e) => panic!("Problem creating the file: {e:?}"),
             },
-            other_error => {
-                panic!("Problem opening the file: {other_error:?}");
+            _ => {
+                panic!("Problem opening the file: {error:?}");
             }
         },
     };
