@@ -1,16 +1,16 @@
-use aggregator::{self, Summary, Tweet};
+use aggregator::{self, SocialPost, Summary};
 
 fn main() {
     // ANCHOR: here
-    let tweet = Tweet {
+    let post = SocialPost {
         username: String::from("horse_ebooks"),
         content: String::from(
             "of course, as you probably already know, people",
         ),
         reply: false,
-        retweet: false,
+        repost: false,
     };
 
-    println!("1 new tweet: {}", tweet.summarize());
+    println!("1 new social post: {}", post.summarize());
     // ANCHOR_END: here
 }
