@@ -1,4 +1,3 @@
-// ANCHOR: here
 use std::error::Error;
 use std::fs;
 
@@ -9,8 +8,6 @@ pub struct Config {
 
 impl Config {
     pub fn build(args: &[String]) -> Result<Config, &'static str> {
-        // --snip--
-        // ANCHOR_END: here
         if args.len() < 3 {
             return Err("not enough arguments");
         }
@@ -19,18 +16,29 @@ impl Config {
         let file_path = args[2].clone();
 
         Ok(Config { query, file_path })
-        // ANCHOR: here
     }
 }
 
 pub fn run(config: Config) -> Result<(), Box<dyn Error>> {
-    // --snip--
-    // ANCHOR_END: here
     let contents = fs::read_to_string(config.file_path)?;
 
-    println!("With text:\n{contents}");
-
     Ok(())
-    // ANCHOR: here
+}
+
+// ANCHOR: here
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn one_result() {
+        let query = "duct";
+        let contents = "\
+Rust:
+safe, fast, productive.
+Pick three.";
+
+        assert_eq!(vec!["safe, fast, productive."], search(query, contents));
+    }
 }
 // ANCHOR_END: here
