@@ -26,9 +26,9 @@ pub fn cli() -> Command {
             "Build only the specified example",
             "Build all examples",
             "Build only the specified test target",
-            "Build all test targets",
+            "Build all targets that have `test = true` set",
             "Build only the specified bench target",
-            "Build all bench targets",
+            "Build all targets that have `bench = true` set",
             "Build all targets",
         )
         .arg_features()
