@@ -22,8 +22,8 @@
 use crate::core::global_cache_tracker::{self, GlobalCacheTracker};
 use crate::ops::CleanContext;
 use crate::util::cache_lock::{CacheLock, CacheLockMode};
-use crate::{CargoResult, Config};
-use anyhow::{format_err, Context};
+use crate::{CargoResult, GlobalContext};
+use anyhow::{format_err, Context as _};
 use serde::Deserialize;
 use std::time::Duration;
 
@@ -48,11 +48,8 @@ const DEFAULT_AUTO_FREQUENCY: &str = "1 day";
 ///
 /// It should be cheap to call this multiple times (subsequent calls are
 /// ignored), but try not to abuse that.
-pub fn auto_gc(config: &Config) {
-    if !config.cli_unstable().gc {
-        return;
-    }
-    if !config.network_allowed() {
+pub fn auto_gc(gctx: &GlobalContext) {
+    if !gctx.network_allowed() {
         // As a conservative choice, auto-gc is disabled when offline. If the
         // user is indefinitely offline, we don't want to delete things they
         // may later depend on.
@@ -60,21 +57,21 @@ pub fn auto_gc(config: &Config) {
         return;
     }
 
-    if let Err(e) = auto_gc_inner(config) {
-        if global_cache_tracker::is_silent_error(&e) && !config.extra_verbose() {
+    if let Err(e) = auto_gc_inner(gctx) {
+        if global_cache_tracker::is_silent_error(&e) && !gctx.extra_verbose() {
             tracing::warn!(target: "gc", "failed to auto-clean cache data: {e:?}");
         } else {
             crate::display_warning_with_error(
                 "failed to auto-clean cache data",
                 &e,
-                &mut config.shell(),
+                &mut gctx.shell(),
             );
         }
     }
 }
 
-fn auto_gc_inner(config: &Config) -> CargoResult<()> {
-    let _lock = match config.try_acquire_package_cache_lock(CacheLockMode::MutateExclusive)? {
+fn auto_gc_inner(gctx: &GlobalContext) -> CargoResult<()> {
+    let _lock = match gctx.try_acquire_package_cache_lock(CacheLockMode::MutateExclusive)? {
         Some(lock) => lock,
         None => {
             tracing::debug!(target: "gc", "unable to acquire mutate lock, auto gc disabled");
@@ -82,16 +79,16 @@ fn auto_gc_inner(config: &Config) -> CargoResult<()> {
         }
     };
     // This should not be called when there are pending deferred entries, so check that.
-    let deferred = config.deferred_global_last_use()?;
+    let deferred = gctx.deferred_global_last_use()?;
     debug_assert!(deferred.is_empty());
-    let mut global_cache_tracker = config.global_cache_tracker()?;
-    let mut gc = Gc::new(config, &mut global_cache_tracker)?;
-    let mut clean_ctx = CleanContext::new(config);
+    let mut global_cache_tracker = gctx.global_cache_tracker()?;
+    let mut gc = Gc::new(gctx, &mut global_cache_tracker)?;
+    let mut clean_ctx = CleanContext::new(gctx);
     gc.auto(&mut clean_ctx)?;
     Ok(())
 }
 
-/// Automatic garbage collection settings from the `gc.auto` config table.
+/// Cache cleaning settings from the `cache.global-clean` config table.
 ///
 /// NOTE: Not all of these options may get stabilized. Some of them are very
 /// low-level details, and may not be something typical users need.
@@ -99,9 +96,7 @@ fn auto_gc_inner(config: &Config) -> CargoResult<()> {
 /// If any of these options are `None`, the built-in default is used.
 #[derive(Deserialize, Default)]
 #[serde(rename_all = "kebab-case")]
-struct AutoConfig {
-    /// The maximum frequency that automatic garbage collection happens.
-    frequency: Option<String>,
+struct GlobalCleanConfig {
     /// Anything older than this duration will be deleted in the source cache.
     max_src_age: Option<String>,
     /// Anything older than this duration will be deleted in the compressed crate cache.
@@ -172,53 +167,78 @@ impl GcOpts {
 
     /// Updates the configuration of this [`GcOpts`] to incorporate the
     /// settings from config.
-    pub fn update_for_auto_gc(&mut self, config: &Config) -> CargoResult<()> {
-        let auto_config = config
-            .get::<Option<AutoConfig>>("gc.auto")?
+    pub fn update_for_auto_gc(&mut self, gctx: &GlobalContext) -> CargoResult<()> {
+        let config = gctx
+            .get::<Option<GlobalCleanConfig>>("cache.global-clean")?
             .unwrap_or_default();
-        self.update_for_auto_gc_config(&auto_config)
+        self.update_for_auto_gc_config(&config, gctx.cli_unstable().gc)
     }
 
-    fn update_for_auto_gc_config(&mut self, auto_config: &AutoConfig) -> CargoResult<()> {
+    fn update_for_auto_gc_config(
+        &mut self,
+        config: &GlobalCleanConfig,
+        unstable_allowed: bool,
+    ) -> CargoResult<()> {
+        macro_rules! config_default {
+            ($config:expr, $field:ident, $default:expr, $unstable_allowed:expr) => {
+                if !unstable_allowed {
+                    // These config options require -Zgc
+                    $default
+                } else {
+                    $config.$field.as_deref().unwrap_or($default)
+                }
+            };
+        }
+
         self.max_src_age = newer_time_span_for_config(
             self.max_src_age,
             "gc.auto.max-src-age",
-            auto_config
-                .max_src_age
-                .as_deref()
-                .unwrap_or(DEFAULT_MAX_AGE_EXTRACTED),
+            config_default!(
+                config,
+                max_src_age,
+                DEFAULT_MAX_AGE_EXTRACTED,
+                unstable_allowed
+            ),
         )?;
         self.max_crate_age = newer_time_span_for_config(
             self.max_crate_age,
             "gc.auto.max-crate-age",
-            auto_config
-                .max_crate_age
-                .as_deref()
-                .unwrap_or(DEFAULT_MAX_AGE_DOWNLOADED),
+            config_default!(
+                config,
+                max_crate_age,
+                DEFAULT_MAX_AGE_DOWNLOADED,
+                unstable_allowed
+            ),
         )?;
         self.max_index_age = newer_time_span_for_config(
             self.max_index_age,
             "gc.auto.max-index-age",
-            auto_config
-                .max_index_age
-                .as_deref()
-                .unwrap_or(DEFAULT_MAX_AGE_DOWNLOADED),
+            config_default!(
+                config,
+                max_index_age,
+                DEFAULT_MAX_AGE_DOWNLOADED,
+                unstable_allowed
+            ),
         )?;
         self.max_git_co_age = newer_time_span_for_config(
             self.max_git_co_age,
             "gc.auto.max-git-co-age",
-            auto_config
-                .max_git_co_age
-                .as_deref()
-                .unwrap_or(DEFAULT_MAX_AGE_EXTRACTED),
+            config_default!(
+                config,
+                max_git_co_age,
+                DEFAULT_MAX_AGE_EXTRACTED,
+                unstable_allowed
+            ),
         )?;
         self.max_git_db_age = newer_time_span_for_config(
             self.max_git_db_age,
             "gc.auto.max-git-db-age",
-            auto_config
-                .max_git_db_age
-                .as_deref()
-                .unwrap_or(DEFAULT_MAX_AGE_DOWNLOADED),
+            config_default!(
+                config,
+                max_git_db_age,
+                DEFAULT_MAX_AGE_DOWNLOADED,
+                unstable_allowed
+            ),
         )?;
         Ok(())
     }
@@ -227,8 +247,8 @@ impl GcOpts {
 /// Garbage collector.
 ///
 /// See the module docs at [`crate::core::gc`] for more information on GC.
-pub struct Gc<'a, 'config> {
-    config: &'config Config,
+pub struct Gc<'a, 'gctx> {
+    gctx: &'gctx GlobalContext,
     global_cache_tracker: &'a mut GlobalCacheTracker,
     /// A lock on the package cache.
     ///
@@ -236,17 +256,17 @@ pub struct Gc<'a, 'config> {
     /// be allowed to write to the cache at the same time, or for others to
     /// read while we are modifying the cache.
     #[allow(dead_code)] // Held for drop.
-    lock: CacheLock<'config>,
+    lock: CacheLock<'gctx>,
 }
 
-impl<'a, 'config> Gc<'a, 'config> {
+impl<'a, 'gctx> Gc<'a, 'gctx> {
     pub fn new(
-        config: &'config Config,
+        gctx: &'gctx GlobalContext,
         global_cache_tracker: &'a mut GlobalCacheTracker,
-    ) -> CargoResult<Gc<'a, 'config>> {
-        let lock = config.acquire_package_cache_lock(CacheLockMode::MutateExclusive)?;
+    ) -> CargoResult<Gc<'a, 'gctx>> {
+        let lock = gctx.acquire_package_cache_lock(CacheLockMode::MutateExclusive)?;
         Ok(Gc {
-            config,
+            gctx,
             global_cache_tracker,
             lock,
         })
@@ -255,30 +275,25 @@ impl<'a, 'config> Gc<'a, 'config> {
     /// Performs automatic garbage cleaning.
     ///
     /// This returns immediately without doing work if garbage collection has
-    /// been performed recently (since `gc.auto.frequency`).
-    fn auto(&mut self, clean_ctx: &mut CleanContext<'config>) -> CargoResult<()> {
-        if !self.config.cli_unstable().gc {
-            return Ok(());
-        }
-        let auto_config = self
-            .config
-            .get::<Option<AutoConfig>>("gc.auto")?
-            .unwrap_or_default();
-        let Some(freq) = parse_frequency(
-            auto_config
-                .frequency
-                .as_deref()
-                .unwrap_or(DEFAULT_AUTO_FREQUENCY),
-        )?
-        else {
+    /// been performed recently (since `cache.auto-clean-frequency`).
+    fn auto(&mut self, clean_ctx: &mut CleanContext<'gctx>) -> CargoResult<()> {
+        let freq = self
+            .gctx
+            .get::<Option<String>>("cache.auto-clean-frequency")?;
+        let Some(freq) = parse_frequency(freq.as_deref().unwrap_or(DEFAULT_AUTO_FREQUENCY))? else {
             tracing::trace!(target: "gc", "auto gc disabled");
             return Ok(());
         };
         if !self.global_cache_tracker.should_run_auto_gc(freq)? {
             return Ok(());
         }
+        let config = self
+            .gctx
+            .get::<Option<GlobalCleanConfig>>("cache.global-clean")?
+            .unwrap_or_default();
+
         let mut gc_opts = GcOpts::default();
-        gc_opts.update_for_auto_gc_config(&auto_config)?;
+        gc_opts.update_for_auto_gc_config(&config, self.gctx.cli_unstable().gc)?;
         self.gc(clean_ctx, &gc_opts)?;
         if !clean_ctx.dry_run {
             self.global_cache_tracker.set_last_auto_gc()?;
@@ -287,11 +302,7 @@ impl<'a, 'config> Gc<'a, 'config> {
     }
 
     /// Performs garbage collection based on the given options.
-    pub fn gc(
-        &mut self,
-        clean_ctx: &mut CleanContext<'config>,
-        gc_opts: &GcOpts,
-    ) -> CargoResult<()> {
+    pub fn gc(&mut self, clean_ctx: &mut CleanContext<'gctx>, gc_opts: &GcOpts) -> CargoResult<()> {
         self.global_cache_tracker.clean(clean_ctx, gc_opts)?;
         // In the future, other gc operations go here, such as target cleaning.
         Ok(())
@@ -341,7 +352,7 @@ fn parse_frequency(frequency: &str) -> CargoResult<Option<Duration>> {
     }
     let duration = maybe_parse_time_span(frequency).ok_or_else(|| {
         format_err!(
-            "config option `gc.auto.frequency` expected a value of \"always\", \"never\", \
+            "config option `cache.auto-clean-frequency` expected a value of \"always\", \"never\", \
              or \"N seconds/minutes/days/weeks/months\", got: {frequency:?}"
         )
     })?;
@@ -464,17 +475,18 @@ mod tests {
         assert_eq!(maybe_parse_time_span(" 1 day"), None);
         assert_eq!(maybe_parse_time_span("1  second"), None);
 
-        let e = parse_time_span_for_config("gc.auto.max-src-age", "-1 days").unwrap_err();
+        let e =
+            parse_time_span_for_config("cache.global-clean.max-src-age", "-1 days").unwrap_err();
         assert_eq!(
             e.to_string(),
-            "config option `gc.auto.max-src-age` \
+            "config option `cache.global-clean.max-src-age` \
              expected a value of the form \"N seconds/minutes/days/weeks/months\", \
              got: \"-1 days\""
         );
         let e = parse_frequency("abc").unwrap_err();
         assert_eq!(
             e.to_string(),
-            "config option `gc.auto.frequency` \
+            "config option `cache.auto-clean-frequency` \
              expected a value of \"always\", \"never\", or \"N seconds/minutes/days/weeks/months\", \
              got: \"abc\""
         );
