@@ -9,7 +9,6 @@ use std::rc::Rc;
 use std::time::{Duration, Instant};
 
 use anyhow::Context as _;
-use bytesize::ByteSize;
 use cargo_util_schemas::manifest::RustVersion;
 use curl::easy::Easy;
 use curl::multi::{EasyHandle, Multi};
@@ -35,6 +34,7 @@ use crate::util::network::http::http_handle_and_timeout;
 use crate::util::network::http::HttpTimeout;
 use crate::util::network::retry::{Retry, RetryResult};
 use crate::util::network::sleep::SleepTracker;
+use crate::util::HumanBytes;
 use crate::util::{self, internal, GlobalContext, Progress, ProgressStyle};
 
 /// Information about a package that is available somewhere in the file system.
@@ -186,11 +186,6 @@ impl Package {
         }
     }
 
-    /// Returns if package should include `Cargo.lock`.
-    pub fn include_lockfile(&self) -> bool {
-        self.targets().iter().any(|t| t.is_example() || t.is_bin())
-    }
-
     pub fn serialized(
         &self,
         unstable_flags: &CliUnstable,
@@ -295,7 +290,7 @@ pub struct PackageSet<'gctx> {
     sources: RefCell<SourceMap<'gctx>>,
     gctx: &'gctx GlobalContext,
     multi: Multi,
-    /// Used to prevent reusing the PackageSet to download twice.
+    /// Used to prevent reusing the `PackageSet` to download twice.
     downloading: Cell<bool>,
     /// Whether or not to use curl HTTP/2 multiplexing.
     multiplexing: bool,
@@ -499,17 +494,18 @@ impl<'gctx> PackageSet<'gctx> {
         force_all_targets: ForceAllTargets,
     ) -> CargoResult<()> {
         fn collect_used_deps(
-            used: &mut BTreeSet<PackageId>,
+            used: &mut BTreeSet<(PackageId, CompileKind)>,
             resolve: &Resolve,
             pkg_id: PackageId,
             has_dev_units: HasDevUnits,
-            requested_kinds: &[CompileKind],
+            requested_kind: CompileKind,
             target_data: &RustcTargetData<'_>,
             force_all_targets: ForceAllTargets,
         ) -> CargoResult<()> {
-            if !used.insert(pkg_id) {
+            if !used.insert((pkg_id, requested_kind)) {
                 return Ok(());
             }
+            let requested_kinds = &[requested_kind];
             let filtered_deps = PackageSet::filter_deps(
                 pkg_id,
                 resolve,
@@ -518,16 +514,34 @@ impl<'gctx> PackageSet<'gctx> {
                 target_data,
                 force_all_targets,
             );
-            for (pkg_id, _dep) in filtered_deps {
+            for (pkg_id, deps) in filtered_deps {
                 collect_used_deps(
                     used,
                     resolve,
                     pkg_id,
                     has_dev_units,
-                    requested_kinds,
+                    requested_kind,
                     target_data,
                     force_all_targets,
                 )?;
+                let artifact_kinds = deps.iter().filter_map(|dep| {
+                    Some(
+                        dep.artifact()?
+                            .target()?
+                            .to_resolved_compile_kind(*requested_kinds.iter().next().unwrap()),
+                    )
+                });
+                for artifact_kind in artifact_kinds {
+                    collect_used_deps(
+                        used,
+                        resolve,
+                        pkg_id,
+                        has_dev_units,
+                        artifact_kind,
+                        target_data,
+                        force_all_targets,
+                    )?;
+                }
             }
             Ok(())
         }
@@ -538,16 +552,22 @@ impl<'gctx> PackageSet<'gctx> {
         let mut to_download = BTreeSet::new();
 
         for id in root_ids {
-            collect_used_deps(
-                &mut to_download,
-                resolve,
-                *id,
-                has_dev_units,
-                requested_kinds,
-                target_data,
-                force_all_targets,
-            )?;
+            for requested_kind in requested_kinds {
+                collect_used_deps(
+                    &mut to_download,
+                    resolve,
+                    *id,
+                    has_dev_units,
+                    *requested_kind,
+                    target_data,
+                    force_all_targets,
+                )?;
+            }
         }
+        let to_download = to_download
+            .into_iter()
+            .map(|(p, _)| p)
+            .collect::<BTreeSet<_>>();
         self.get_many(to_download.into_iter())?;
         Ok(())
     }
@@ -894,7 +914,8 @@ impl<'a, 'gctx> Downloads<'a, 'gctx> {
         // have a great view into the progress of the extraction. Let's prepare
         // the user for this CPU-heavy step if it looks like it'll take some
         // time to do so.
-        if dl.total.get() < ByteSize::kb(400).0 {
+        let kib_400 = 1024 * 400;
+        if dl.total.get() < kib_400 {
             self.tick(WhyTick::DownloadFinished)?;
         } else {
             self.tick(WhyTick::Extracting(&dl.id.name()))?;
@@ -1093,7 +1114,7 @@ impl<'a, 'gctx> Downloads<'a, 'gctx> {
                     }
                 }
                 if remaining > 0 && dur > Duration::from_millis(500) {
-                    msg.push_str(&format!(", remaining bytes: {}", ByteSize(remaining)));
+                    msg.push_str(&format!(", remaining bytes: {:.1}", HumanBytes(remaining)));
                 }
             }
         }
@@ -1133,20 +1154,21 @@ impl<'a, 'gctx> Drop for Downloads<'a, 'gctx> {
             "crates"
         };
         let mut status = format!(
-            "{} {} ({}) in {}",
+            "{} {} ({:.1}) in {}",
             self.downloads_finished,
             crate_string,
-            ByteSize(self.downloaded_bytes),
+            HumanBytes(self.downloaded_bytes),
             util::elapsed(self.start.elapsed())
         );
         // print the size of largest crate if it was >1mb
         // however don't print if only a single crate was downloaded
         // because it is obvious that it will be the largest then
-        if self.largest.0 > ByteSize::mb(1).0 && self.downloads_finished > 1 {
+        let mib_1 = 1024 * 1024;
+        if self.largest.0 > mib_1 && self.downloads_finished > 1 {
             status.push_str(&format!(
-                " (largest was `{}` at {})",
+                " (largest was `{}` at {:.1})",
                 self.largest.1,
-                ByteSize(self.largest.0),
+                HumanBytes(self.largest.0),
             ));
         }
         // Clear progress before displaying final summary.
