@@ -1,13 +1,12 @@
 use super::dep_cache::RegistryQueryer;
 use super::errors::ActivateResult;
-use super::types::{ConflictMap, ConflictReason, FeaturesSet, ResolveOpts};
+use super::types::{ActivationsKey, ConflictMap, ConflictReason, FeaturesSet, ResolveOpts};
 use super::RequestedFeatures;
-use crate::core::{Dependency, PackageId, SourceId, Summary};
-use crate::util::interning::InternedString;
+use crate::core::{Dependency, PackageId, Summary};
+use crate::util::interning::{InternedString, INTERNED_DEFAULT};
 use crate::util::Graph;
 use anyhow::format_err;
-use std::collections::HashMap;
-use std::num::NonZeroU64;
+use std::collections::{BTreeSet, HashMap};
 use tracing::debug;
 
 // A `Context` is basically a bunch of local resolution information which is
@@ -39,39 +38,9 @@ pub type ContextAge = usize;
 /// By storing this in a hash map we ensure that there is only one
 /// semver compatible version of each crate.
 /// This all so stores the `ContextAge`.
-pub type ActivationsKey = (InternedString, SourceId, SemverCompatibility);
-
 pub type Activations =
     im_rc::HashMap<ActivationsKey, (Summary, ContextAge), rustc_hash::FxBuildHasher>;
 
-/// A type that represents when cargo treats two Versions as compatible.
-/// Versions `a` and `b` are compatible if their left-most nonzero digit is the
-/// same.
-#[derive(Clone, Copy, Eq, PartialEq, Hash, Debug, PartialOrd, Ord)]
-pub enum SemverCompatibility {
-    Major(NonZeroU64),
-    Minor(NonZeroU64),
-    Patch(u64),
-}
-
-impl From<&semver::Version> for SemverCompatibility {
-    fn from(ver: &semver::Version) -> Self {
-        if let Some(m) = NonZeroU64::new(ver.major) {
-            return SemverCompatibility::Major(m);
-        }
-        if let Some(m) = NonZeroU64::new(ver.minor) {
-            return SemverCompatibility::Minor(m);
-        }
-        SemverCompatibility::Patch(ver.patch)
-    }
-}
-
-impl PackageId {
-    pub fn as_activations_key(self) -> ActivationsKey {
-        (self.name(), self.source_id(), self.version().into())
-    }
-}
-
 impl ResolverContext {
     pub fn new() -> ResolverContext {
         ResolverContext {
@@ -137,7 +106,8 @@ impl ResolverContext {
                 // versions came from a `[patch]` source.
                 if let Some((_, dep)) = parent {
                     if dep.source_id() != id.source_id() {
-                        let key = (id.name(), dep.source_id(), id.version().into());
+                        let key =
+                            ActivationsKey::new(id.name(), id.version().into(), dep.source_id());
                         let prev = self.activations.insert(key, (summary.clone(), age));
                         if let Some((previous_summary, _)) = prev {
                             return Err(
@@ -151,6 +121,7 @@ impl ResolverContext {
             }
         }
         debug!("checking if {} is already activated", summary.package_id());
+        let empty_features = BTreeSet::new();
         match &opts.features {
             // This returns `false` for CliFeatures just for simplicity. It
             // would take a bit of work to compare since they are not in the
@@ -165,16 +136,16 @@ impl ResolverContext {
                 features,
                 uses_default_features,
             } => {
-                let has_default_feature = summary.features().contains_key("default");
-                Ok(match self.resolve_features.get(&id) {
-                    Some(prev) => {
-                        features.is_subset(prev)
-                            && (!uses_default_features
-                                || prev.contains("default")
-                                || !has_default_feature)
-                    }
-                    None => features.is_empty() && (!uses_default_features || !has_default_feature),
-                })
+                let has_default_feature = summary.features().contains_key(&INTERNED_DEFAULT);
+                let prev = self
+                    .resolve_features
+                    .get(&id)
+                    .map(|f| &**f)
+                    .unwrap_or(&empty_features);
+                Ok(features.is_subset(prev)
+                    && (!uses_default_features
+                        || prev.contains(&INTERNED_DEFAULT)
+                        || !has_default_feature))
             }
         }
     }
