@@ -56,6 +56,7 @@ impl Shell {
                 stderr_tty: std::io::stderr().is_terminal(),
                 stdout_unicode: supports_unicode(&std::io::stdout()),
                 stderr_unicode: supports_unicode(&std::io::stderr()),
+                stderr_term_integration: supports_term_integration(&std::io::stderr()),
             },
             verbosity: Verbosity::Verbose,
             needs_clear: false,
@@ -122,6 +123,18 @@ impl Shell {
         }
     }
 
+    pub fn is_err_term_integration_available(&self) -> bool {
+        if let ShellOut::Stream {
+            stderr_term_integration,
+            ..
+        } = self.output
+        {
+            stderr_term_integration
+        } else {
+            false
+        }
+    }
+
     /// Gets a reference to the underlying stdout writer.
     pub fn out(&mut self) -> &mut dyn Write {
         if self.needs_clear {
@@ -426,6 +439,7 @@ enum ShellOut {
         hyperlinks: bool,
         stdout_unicode: bool,
         stderr_unicode: bool,
+        stderr_term_integration: bool,
     },
 }
 
@@ -575,6 +589,16 @@ fn supports_hyperlinks() -> bool {
     supports_hyperlinks::supports_hyperlinks()
 }
 
+/// Determines whether the terminal supports ANSI OSC 9;4.
+#[allow(clippy::disallowed_methods)] // Read environment variables to detect terminal
+fn supports_term_integration(stream: &dyn IsTerminal) -> bool {
+    let windows_terminal = std::env::var("WT_SESSION").is_ok();
+    let conemu = std::env::var("ConEmuANSI").ok() == Some("ON".into());
+    let wezterm = std::env::var("TERM_PROGRAM").ok() == Some("WezTerm".into());
+
+    (windows_terminal || conemu || wezterm) && stream.is_terminal()
+}
+
 pub struct Hyperlink<D: fmt::Display> {
     url: Option<D>,
 }
@@ -662,7 +686,7 @@ mod imp {
                 ptr::null_mut(),
                 OPEN_EXISTING,
                 0,
-                0,
+                std::ptr::null_mut(),
             );
             if h == INVALID_HANDLE_VALUE {
                 return TtyWidth::NoTty;
