@@ -6,24 +6,14 @@ use crate::util::restricted_names;
 use crate::CargoResult;
 use crate::GlobalContext;
 
-const DEFAULT_EDITION: crate::core::features::Edition =
-    crate::core::features::Edition::LATEST_STABLE;
-const AUTO_FIELDS: &[&str] = &[
-    "autolib",
-    "autobins",
-    "autoexamples",
-    "autotests",
-    "autobenches",
-];
-
 pub(super) fn expand_manifest(
     content: &str,
     path: &std::path::Path,
     gctx: &GlobalContext,
 ) -> CargoResult<String> {
-    let source = split_source(content)?;
-    if let Some(frontmatter) = source.frontmatter {
-        match source.info {
+    let source = ScriptSource::parse(content)?;
+    if let Some(frontmatter) = source.frontmatter() {
+        match source.info() {
             Some("cargo") | None => {}
             Some(other) => {
                 if let Some(remainder) = other.strip_prefix("cargo,") {
@@ -50,7 +40,7 @@ pub(super) fn expand_manifest(
             )
             .into_path_unlocked();
         let mut hacked_source = String::new();
-        if let Some(shebang) = source.shebang {
+        if let Some(shebang) = source.shebang() {
             writeln!(hacked_source, "{shebang}")?;
         }
         writeln!(hacked_source)?; // open
@@ -58,55 +48,29 @@ pub(super) fn expand_manifest(
             writeln!(hacked_source)?;
         }
         writeln!(hacked_source)?; // close
-        writeln!(hacked_source, "{}", source.content)?;
+        writeln!(hacked_source, "{}", source.content())?;
         if let Some(parent) = hacked_path.parent() {
             cargo_util::paths::create_dir_all(parent)?;
         }
         cargo_util::paths::write_if_changed(&hacked_path, hacked_source)?;
 
-        let manifest = expand_manifest_(&frontmatter, &hacked_path, gctx)
-            .with_context(|| format!("failed to parse manifest at {}", path.display()))?;
+        let manifest = inject_bin_path(&frontmatter, &hacked_path)
+            .with_context(|| format!("failed to parse manifest at `{}`", path.display()))?;
         let manifest = toml::to_string_pretty(&manifest)?;
         Ok(manifest)
     } else {
         let frontmatter = "";
-        let manifest = expand_manifest_(frontmatter, path, gctx)
-            .with_context(|| format!("failed to parse manifest at {}", path.display()))?;
+        let manifest = inject_bin_path(frontmatter, path)
+            .with_context(|| format!("failed to parse manifest at `{}`", path.display()))?;
         let manifest = toml::to_string_pretty(&manifest)?;
         Ok(manifest)
     }
 }
 
-fn expand_manifest_(
-    manifest: &str,
-    path: &std::path::Path,
-    gctx: &GlobalContext,
-) -> CargoResult<toml::Table> {
+/// HACK: Add a `[[bin]]` table to the `original_toml`
+fn inject_bin_path(manifest: &str, path: &std::path::Path) -> CargoResult<toml::Table> {
     let mut manifest: toml::Table = toml::from_str(&manifest)?;
 
-    for key in ["workspace", "lib", "bin", "example", "test", "bench"] {
-        if manifest.contains_key(key) {
-            anyhow::bail!("`{key}` is not allowed in embedded manifests")
-        }
-    }
-
-    // Prevent looking for a workspace by `read_manifest_from_str`
-    manifest.insert("workspace".to_owned(), toml::Table::new().into());
-
-    let package = manifest
-        .entry("package".to_owned())
-        .or_insert_with(|| toml::Table::new().into())
-        .as_table_mut()
-        .ok_or_else(|| anyhow::format_err!("`package` must be a table"))?;
-    for key in ["workspace", "build", "links"]
-        .iter()
-        .chain(AUTO_FIELDS.iter())
-    {
-        if package.contains_key(*key) {
-            anyhow::bail!("`package.{key}` is not allowed in embedded manifests")
-        }
-    }
-    // HACK: Using an absolute path while `hacked_path` is in use
     let bin_path = path.to_string_lossy().into_owned();
     let file_stem = path
         .file_stem()
@@ -114,51 +78,22 @@ fn expand_manifest_(
         .to_string_lossy();
     let name = sanitize_name(file_stem.as_ref());
     let bin_name = name.clone();
-    package
-        .entry("name".to_owned())
-        .or_insert(toml::Value::String(name));
-    package.entry("edition".to_owned()).or_insert_with(|| {
-        let _ = gctx.shell().warn(format_args!(
-            "`package.edition` is unspecified, defaulting to `{}`",
-            DEFAULT_EDITION
-        ));
-        toml::Value::String(DEFAULT_EDITION.to_string())
-    });
-    package
-        .entry("build".to_owned())
-        .or_insert_with(|| toml::Value::Boolean(false));
-    for field in AUTO_FIELDS {
-        package
-            .entry(field.to_owned())
-            .or_insert_with(|| toml::Value::Boolean(false));
-    }
 
     let mut bin = toml::Table::new();
     bin.insert("name".to_owned(), toml::Value::String(bin_name));
     bin.insert("path".to_owned(), toml::Value::String(bin_path));
-    manifest.insert(
-        "bin".to_owned(),
-        toml::Value::Array(vec![toml::Value::Table(bin)]),
-    );
-
-    let release = manifest
-        .entry("profile".to_owned())
-        .or_insert_with(|| toml::Value::Table(Default::default()))
-        .as_table_mut()
-        .ok_or_else(|| anyhow::format_err!("`profile` must be a table"))?
-        .entry("release".to_owned())
-        .or_insert_with(|| toml::Value::Table(Default::default()))
-        .as_table_mut()
-        .ok_or_else(|| anyhow::format_err!("`profile.release` must be a table"))?;
-    release
-        .entry("strip".to_owned())
-        .or_insert_with(|| toml::Value::Boolean(true));
+    manifest
+        .entry("bin")
+        .or_insert_with(|| Vec::<toml::Value>::new().into())
+        .as_array_mut()
+        .ok_or_else(|| anyhow::format_err!("`bin` must be an array"))?
+        .push(toml::Value::Table(bin));
 
     Ok(manifest)
 }
 
 /// Ensure the package name matches the validation from `ops::cargo_new::check_name`
-fn sanitize_name(name: &str) -> String {
+pub fn sanitize_name(name: &str) -> String {
     let placeholder = if name.contains('_') {
         '_'
     } else {
@@ -189,94 +124,120 @@ fn sanitize_name(name: &str) -> String {
 }
 
 #[derive(Debug)]
-struct Source<'s> {
+pub struct ScriptSource<'s> {
     shebang: Option<&'s str>,
     info: Option<&'s str>,
     frontmatter: Option<&'s str>,
     content: &'s str,
 }
 
-fn split_source(input: &str) -> CargoResult<Source<'_>> {
-    let mut source = Source {
-        shebang: None,
-        info: None,
-        frontmatter: None,
-        content: input,
-    };
+impl<'s> ScriptSource<'s> {
+    pub fn parse(input: &'s str) -> CargoResult<Self> {
+        let mut source = Self {
+            shebang: None,
+            info: None,
+            frontmatter: None,
+            content: input,
+        };
 
+        if let Some(shebang_end) = strip_shebang(source.content) {
+            let (shebang, content) = source.content.split_at(shebang_end);
+            source.shebang = Some(shebang);
+            source.content = content;
+        }
+
+        const FENCE_CHAR: char = '-';
+
+        let mut rest = source.content;
+        while !rest.is_empty() {
+            let without_spaces = rest.trim_start_matches([' ', '\t']);
+            let without_nl = without_spaces.trim_start_matches(['\r', '\n']);
+            if without_nl == rest {
+                // nothing trimmed
+                break;
+            } else if without_nl == without_spaces {
+                // frontmatter must come after a newline
+                return Ok(source);
+            }
+            rest = without_nl;
+        }
+        let fence_end = rest
+            .char_indices()
+            .find_map(|(i, c)| (c != FENCE_CHAR).then_some(i))
+            .unwrap_or(source.content.len());
+        let (fence_pattern, rest) = match fence_end {
+            0 => {
+                return Ok(source);
+            }
+            1 | 2 => {
+                anyhow::bail!(
+                    "found {fence_end} `{FENCE_CHAR}` in rust frontmatter, expected at least 3"
+                )
+            }
+            _ => rest.split_at(fence_end),
+        };
+        let nl_fence_pattern = format!("\n{fence_pattern}");
+        let (info, content) = rest.split_once("\n").unwrap_or((rest, ""));
+        let info = info.trim();
+        if !info.is_empty() {
+            source.info = Some(info);
+        }
+        source.content = content;
+
+        let Some(frontmatter_nl) = source.content.find(&nl_fence_pattern) else {
+            anyhow::bail!("no closing `{fence_pattern}` found for frontmatter");
+        };
+        source.frontmatter = Some(&source.content[..frontmatter_nl + 1]);
+        source.content = &source.content[frontmatter_nl + nl_fence_pattern.len()..];
+
+        let (line, content) = source
+            .content
+            .split_once("\n")
+            .unwrap_or((source.content, ""));
+        let line = line.trim();
+        if !line.is_empty() {
+            anyhow::bail!("unexpected trailing content on closing fence: `{line}`");
+        }
+        source.content = content;
+
+        Ok(source)
+    }
+
+    pub fn shebang(&self) -> Option<&'s str> {
+        self.shebang
+    }
+
+    pub fn info(&self) -> Option<&'s str> {
+        self.info
+    }
+
+    pub fn frontmatter(&self) -> Option<&'s str> {
+        self.frontmatter
+    }
+
+    pub fn content(&self) -> &'s str {
+        self.content
+    }
+}
+
+fn strip_shebang(input: &str) -> Option<usize> {
     // See rust-lang/rust's compiler/rustc_lexer/src/lib.rs's `strip_shebang`
     // Shebang must start with `#!` literally, without any preceding whitespace.
     // For simplicity we consider any line starting with `#!` a shebang,
     // regardless of restrictions put on shebangs by specific platforms.
-    if let Some(rest) = source.content.strip_prefix("#!") {
+    if let Some(rest) = input.strip_prefix("#!") {
         // Ok, this is a shebang but if the next non-whitespace token is `[`,
         // then it may be valid Rust code, so consider it Rust code.
-        if rest.trim_start().starts_with('[') {
-            return Ok(source);
+        //
+        // NOTE: rustc considers line and block comments to be whitespace but to avoid
+        // any more awareness of Rust grammar, we are excluding it.
+        if !rest.trim_start().starts_with('[') {
+            // No other choice than to consider this a shebang.
+            let newline_end = input.find('\n').map(|pos| pos + 1).unwrap_or(input.len());
+            return Some(newline_end);
         }
-
-        // No other choice than to consider this a shebang.
-        let newline_end = source
-            .content
-            .find('\n')
-            .map(|pos| pos + 1)
-            .unwrap_or(source.content.len());
-        let (shebang, content) = source.content.split_at(newline_end);
-        source.shebang = Some(shebang);
-        source.content = content;
     }
-
-    const FENCE_CHAR: char = '-';
-
-    let mut trimmed_content = source.content;
-    while !trimmed_content.is_empty() {
-        let c = trimmed_content;
-        let c = c.trim_start_matches([' ', '\t']);
-        let c = c.trim_start_matches(['\r', '\n']);
-        if c == trimmed_content {
-            break;
-        }
-        trimmed_content = c;
-    }
-    let fence_end = trimmed_content
-        .char_indices()
-        .find_map(|(i, c)| (c != FENCE_CHAR).then_some(i))
-        .unwrap_or(source.content.len());
-    let (fence_pattern, rest) = match fence_end {
-        0 => {
-            return Ok(source);
-        }
-        1 | 2 => {
-            anyhow::bail!(
-                "found {fence_end} `{FENCE_CHAR}` in rust frontmatter, expected at least 3"
-            )
-        }
-        _ => trimmed_content.split_at(fence_end),
-    };
-    let (info, content) = rest.split_once("\n").unwrap_or((rest, ""));
-    let info = info.trim();
-    if !info.is_empty() {
-        source.info = Some(info);
-    }
-    source.content = content;
-
-    let Some((frontmatter, content)) = source.content.split_once(fence_pattern) else {
-        anyhow::bail!("no closing `{fence_pattern}` found for frontmatter");
-    };
-    source.frontmatter = Some(frontmatter);
-    source.content = content;
-
-    let (line, content) = source
-        .content
-        .split_once("\n")
-        .unwrap_or((source.content, ""));
-    let line = line.trim();
-    if !line.is_empty() {
-        anyhow::bail!("unexpected trailing content on closing fence: `{line}`");
-    }
-    source.content = content;
-
-    Ok(source)
+    None
 }
 
 #[cfg(test)]
@@ -291,16 +252,16 @@ mod test_expand {
     fn assert_source(source: &str, expected: impl IntoData) {
         use std::fmt::Write as _;
 
-        let actual = match split_source(source) {
+        let actual = match ScriptSource::parse(source) {
             Ok(actual) => actual,
             Err(err) => panic!("unexpected err: {err}"),
         };
 
         let mut rendered = String::new();
-        write_optional_field(&mut rendered, "shebang", actual.shebang);
-        write_optional_field(&mut rendered, "info", actual.info);
-        write_optional_field(&mut rendered, "frontmatter", actual.frontmatter);
-        writeln!(&mut rendered, "content: {:?}", actual.content).unwrap();
+        write_optional_field(&mut rendered, "shebang", actual.shebang());
+        write_optional_field(&mut rendered, "info", actual.info());
+        write_optional_field(&mut rendered, "frontmatter", actual.frontmatter());
+        writeln!(&mut rendered, "content: {:?}", actual.content()).unwrap();
         assert_data_eq!(rendered, expected.raw());
     }
 
@@ -497,7 +458,7 @@ content: "\nfn main() {}"
     #[test]
     fn split_too_few_dashes() {
         assert_err(
-            split_source(
+            ScriptSource::parse(
                 r#"#!/usr/bin/env cargo
 --
 [dependencies]
@@ -510,10 +471,90 @@ fn main() {}
         );
     }
 
+    #[test]
+    fn split_indent() {
+        assert_source(
+            r#"#!/usr/bin/env cargo
+    ---
+    [dependencies]
+    time="0.1.25"
+    ----
+
+fn main() {}
+"#,
+            str![[r##"
+shebang: "#!/usr/bin/env cargo\n"
+info: None
+frontmatter: None
+content: "    ---\n    [dependencies]\n    time=\"0.1.25\"\n    ----\n\nfn main() {}\n"
+
+"##]],
+        );
+    }
+
+    #[test]
+    fn split_escaped() {
+        assert_source(
+            r#"#!/usr/bin/env cargo
+-----
+---
+---
+-----
+
+fn main() {}
+"#,
+            str![[r##"
+shebang: "#!/usr/bin/env cargo\n"
+info: None
+frontmatter: "---\n---\n"
+content: "\nfn main() {}\n"
+
+"##]],
+        );
+    }
+
+    #[test]
+    fn split_invalid_escaped() {
+        assert_err(
+            ScriptSource::parse(
+                r#"#!/usr/bin/env cargo
+---
+-----
+-----
+---
+
+fn main() {}
+"#,
+            ),
+            str!["unexpected trailing content on closing fence: `--`"],
+        );
+    }
+
+    #[test]
+    fn split_dashes_in_body() {
+        assert_source(
+            r#"#!/usr/bin/env cargo
+---
+Hello---
+World
+---
+
+fn main() {}
+"#,
+            str![[r##"
+shebang: "#!/usr/bin/env cargo\n"
+info: None
+frontmatter: "Hello---\nWorld\n"
+content: "\nfn main() {}\n"
+
+"##]],
+        );
+    }
+
     #[test]
     fn split_mismatched_dashes() {
         assert_err(
-            split_source(
+            ScriptSource::parse(
                 r#"#!/usr/bin/env cargo
 ---
 [dependencies]
@@ -529,7 +570,7 @@ fn main() {}
     #[test]
     fn split_missing_close() {
         assert_err(
-            split_source(
+            ScriptSource::parse(
                 r#"#!/usr/bin/env cargo
 ---
 [dependencies]
@@ -560,21 +601,6 @@ fn main() {}
 name = "test-"
 path = "/home/me/test.rs"
 
-[package]
-autobenches = false
-autobins = false
-autoexamples = false
-autolib = false
-autotests = false
-build = false
-edition = "2021"
-name = "test-"
-
-[profile.release]
-strip = true
-
-[workspace]
-
 "#]]
         );
     }
@@ -598,21 +624,6 @@ path = [..]
 [dependencies]
 time = "0.1.25"
 
-[package]
-autobenches = false
-autobins = false
-autoexamples = false
-autolib = false
-autotests = false
-build = false
-edition = "2021"
-name = "test-"
-
-[profile.release]
-strip = true
-
-[workspace]
-
 "#]]
         );
     }
