@@ -1,6 +1,7 @@
 mod add_basic;
 mod add_multiple;
-mod add_normalized_name_external;
+mod add_no_vendored_package_with_alter_registry;
+mod add_no_vendored_package_with_vendor;
 mod add_toolchain;
 mod build;
 mod build_prefer_existing_version;
@@ -12,17 +13,20 @@ mod deprecated_section;
 mod detect_workspace_inherit;
 mod detect_workspace_inherit_features;
 mod detect_workspace_inherit_optional;
+mod detect_workspace_inherit_path_base;
 mod detect_workspace_inherit_public;
 mod dev;
 mod dev_build_conflict;
+mod dev_existing_path_base;
 mod dev_prefer_existing_version;
 mod dry_run;
 mod empty_dep_name;
-mod empty_dep_table;
 mod features;
 mod features_activated_over_limit;
 mod features_deactivated_over_limit;
 mod features_empty;
+mod features_error_activated_over_limit;
+mod features_error_deactivated_over_limit;
 mod features_multiple_occurrences;
 mod features_preserve;
 mod features_spaced_values;
@@ -35,7 +39,7 @@ mod git_dev;
 mod git_inferred_name;
 mod git_inferred_name_multiple;
 mod git_multiple_names;
-mod git_normalized_name;
+mod git_multiple_packages_features;
 mod git_registry;
 mod git_rev;
 mod git_tag;
@@ -68,6 +72,13 @@ mod no_args;
 mod no_default_features;
 mod no_optional;
 mod no_public;
+mod normalize_name_git;
+mod normalize_name_path;
+mod normalize_name_path_existing;
+mod normalize_name_registry;
+mod normalize_name_registry_existing;
+mod normalize_name_registry_yanked;
+mod normalize_name_workspace_dep;
 mod offline_empty_cache;
 mod optional;
 mod overwrite_default_features;
@@ -89,6 +100,7 @@ mod overwrite_no_public_with_public;
 mod overwrite_optional;
 mod overwrite_optional_with_no_optional;
 mod overwrite_optional_with_optional;
+mod overwrite_path_base_with_version;
 mod overwrite_path_noop;
 mod overwrite_path_with_version;
 mod overwrite_preserves_inline_table;
@@ -103,12 +115,17 @@ mod overwrite_with_rename;
 mod overwrite_workspace_dep;
 mod overwrite_workspace_dep_features;
 mod path;
+mod path_base;
+mod path_base_inferred_name;
+mod path_base_missing_base_path;
+mod path_base_unstable;
 mod path_dev;
 mod path_inferred_name;
 mod path_inferred_name_conflicts_full_feature;
-mod path_normalized_name;
 mod preserve_dep_std_table;
+mod preserve_features_sorted;
 mod preserve_features_table;
+mod preserve_features_unsorted;
 mod preserve_sorted;
 mod preserve_unsorted;
 mod public;
@@ -120,6 +137,13 @@ mod rust_version_ignore;
 mod rust_version_incompatible;
 mod rust_version_latest;
 mod rust_version_older;
+mod rustc_ignore;
+mod rustc_incompatible;
+mod rustc_latest;
+mod rustc_older;
+mod script_bare;
+mod script_frontmatter;
+mod script_shebang;
 mod sorted_table_with_dotted_item;
 mod target;
 mod target_cfg;
@@ -128,3 +152,4 @@ mod vers;
 mod workspace_name;
 mod workspace_path;
 mod workspace_path_dev;
+mod yanked;
