@@ -7,6 +7,7 @@ use cargo_test_support::Project;
 
 #[cargo_test]
 fn case() {
+    cargo_test_support::registry::init();
     let project = Project::from_template(current_dir!().join("in"));
     let project_root = project.root();
     let cwd = &project_root;
@@ -17,8 +18,8 @@ fn case() {
         .current_dir(cwd)
         .assert()
         .failure()
-        .stdout_matches(str![""])
-        .stderr_matches(file!["stderr.term.svg"]);
+        .stdout_eq(str![""])
+        .stderr_eq(file!["stderr.term.svg"]);
 
     assert_ui().subset_matches(current_dir!().join("out"), &project_root);
 }
