@@ -1,6 +1,8 @@
 //! Tests for `[env]` config.
 
 use cargo_test_support::basic_manifest;
+use cargo_test_support::prelude::*;
+use cargo_test_support::str;
 use cargo_test_support::{basic_bin_manifest, project};
 
 #[cargo_test]
@@ -27,8 +29,11 @@ fn env_basic() {
         .build();
 
     p.cargo("run")
-        .with_stdout_contains("compile-time:Hello")
-        .with_stdout_contains("run-time:Hello")
+        .with_stdout_data(str![[r#"
+compile-time:Hello
+run-time:Hello
+
+"#]])
         .run();
 }
 
@@ -54,7 +59,16 @@ fn env_invalid() {
 
     p.cargo("check")
         .with_status(101)
-        .with_stderr_contains("[..]could not load config key `env.ENV_TEST_BOOL`")
+        .with_stderr_data(str![[r#"
+[ERROR] error in [ROOT]/foo/.cargo/config.toml: could not load config key `env.ENV_TEST_BOOL`
+
+Caused by:
+  error in [ROOT]/foo/.cargo/config.toml: could not load config key `env.ENV_TEST_BOOL`
+
+Caused by:
+  invalid type: boolean `false`, expected a string or map
+
+"#]])
         .run();
 }
 
@@ -78,9 +92,11 @@ fn env_no_disallowed() {
         );
         p.cargo("check")
             .with_status(101)
-            .with_stderr(&format!(
-                "[ERROR] setting the `{disallowed}` environment variable \
-                is not supported in the `[env]` configuration table"
+            .with_stderr_data(format!(
+                "\
+[ERROR] setting the `{disallowed}` environment variable \
+is not supported in the `[env]` configuration table
+"
             ))
             .run();
     }
@@ -116,9 +132,12 @@ fn env_force() {
         .env("ENV_TEST_FORCED", "from-env")
         .env("ENV_TEST_UNFORCED", "from-env")
         .env("ENV_TEST_UNFORCED_DEFAULT", "from-env")
-        .with_stdout_contains("ENV_TEST_FORCED:from-config")
-        .with_stdout_contains("ENV_TEST_UNFORCED:from-env")
-        .with_stdout_contains("ENV_TEST_UNFORCED_DEFAULT:from-env")
+        .with_stdout_data(str![[r#"
+ENV_TEST_FORCED:from-config
+ENV_TEST_UNFORCED:from-env
+ENV_TEST_UNFORCED_DEFAULT:from-env
+
+"#]])
         .run();
 }
 
@@ -179,7 +198,10 @@ fn env_no_override() {
         .build();
 
     p.cargo("run")
-        .with_stdout_contains("CARGO_PKG_NAME:unchanged")
+        .with_stdout_data(str![[r#"
+CARGO_PKG_NAME:unchanged
+
+"#]])
         .run();
 }
 
@@ -229,8 +251,14 @@ fn env_applied_to_target_info_discovery_rustc() {
 
     p.cargo("run")
         .env("RUSTC_WORKSPACE_WRAPPER", wrapper)
-        .with_stderr_contains("WRAPPER ENV_TEST:from-config")
-        .with_stderr_contains("MAIN ENV_TEST:from-config")
+        .with_stderr_data(str![[r#"
+[COMPILING] foo v0.5.0 ([ROOT]/foo)
+WRAPPER ENV_TEST:from-config
+[FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
+[RUNNING] `target/debug/foo[EXE]`
+MAIN ENV_TEST:from-config
+
+"#]])
         .run();
 
     // Ensure wrapper also maintains the same overridden priority for envs.
@@ -238,7 +266,178 @@ fn env_applied_to_target_info_discovery_rustc() {
     p.cargo("run")
         .env("ENV_TEST", "from-env")
         .env("RUSTC_WORKSPACE_WRAPPER", wrapper)
-        .with_stderr_contains("WRAPPER ENV_TEST:from-env")
-        .with_stderr_contains("MAIN ENV_TEST:from-env")
+        .with_stderr_data(str![[r#"
+[COMPILING] foo v0.5.0 ([ROOT]/foo)
+WRAPPER ENV_TEST:from-env
+[FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
+[RUNNING] `target/debug/foo[EXE]`
+MAIN ENV_TEST:from-env
+
+"#]])
+        .run();
+}
+
+#[cargo_test]
+fn env_changed_defined_in_config_toml() {
+    let p = project()
+        .file("Cargo.toml", &basic_bin_manifest("foo"))
+        .file(
+            "src/main.rs",
+            r#"
+        use std::env;
+        fn main() {
+            println!( "{}", env!("ENV_TEST") );
+        }
+        "#,
+        )
+        .file(
+            ".cargo/config.toml",
+            r#"
+                [env]
+                ENV_TEST = "from-config"
+            "#,
+        )
+        .build();
+
+    p.cargo("run")
+        .with_stdout_data(str![[r#"
+from-config
+
+"#]])
+        .with_stderr_data(str![[r#"
+[COMPILING] foo v0.5.0 ([ROOT]/foo)
+[FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
+[RUNNING] `target/debug/foo[EXE]`
+
+"#]])
+        .run();
+
+    p.cargo("run")
+        .env("ENV_TEST", "from-env")
+        .with_stdout_data(str![[r#"
+from-env
+
+"#]])
+        .with_stderr_data(str![[r#"
+[COMPILING] foo v0.5.0 ([ROOT]/foo)
+[FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
+[RUNNING] `target/debug/foo[EXE]`
+
+"#]])
+        .run();
+    // This identical cargo invocation is to ensure no rebuild happen.
+    p.cargo("run")
+        .env("ENV_TEST", "from-env")
+        .with_stdout_data(str![[r#"
+from-env
+
+"#]])
+        .with_stderr_data(str![[r#"
+[FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
+[RUNNING] `target/debug/foo[EXE]`
+
+"#]])
+        .run();
+}
+
+#[cargo_test]
+fn forced_env_changed_defined_in_config_toml() {
+    let p = project()
+        .file("Cargo.toml", &basic_bin_manifest("foo"))
+        .file(
+            "src/main.rs",
+            r#"
+        use std::env;
+        fn main() {
+            println!( "{}", env!("ENV_TEST") );
+        }
+        "#,
+        )
+        .file(
+            ".cargo/config.toml",
+            r#"
+                [env]
+                ENV_TEST = {value = "from-config", force = true}
+            "#,
+        )
+        .build();
+
+    p.cargo("run")
+        .with_stdout_data(str![[r#"
+from-config
+
+"#]])
+        .with_stderr_data(str![[r#"
+[COMPILING] foo v0.5.0 ([ROOT]/foo)
+[FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
+[RUNNING] `target/debug/foo[EXE]`
+
+"#]])
+        .run();
+
+    p.cargo("run")
+        .env("ENV_TEST", "from-env")
+        .with_stdout_data(str![[r#"
+from-config
+
+"#]])
+        .with_stderr_data(str![[r#"
+[FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
+[RUNNING] `target/debug/foo[EXE]`
+
+"#]])
+        .run();
+}
+
+#[cargo_test]
+fn env_changed_defined_in_config_args() {
+    let p = project()
+        .file("Cargo.toml", &basic_bin_manifest("foo"))
+        .file(
+            "src/main.rs",
+            r#"
+        use std::env;
+        fn main() {
+            println!( "{}", env!("ENV_TEST") );
+        }
+        "#,
+        )
+        .build();
+    p.cargo(r#"run --config 'env.ENV_TEST="one"'"#)
+        .with_stdout_data(str![[r#"
+one
+
+"#]])
+        .with_stderr_data(str![[r#"
+[COMPILING] foo v0.5.0 ([ROOT]/foo)
+[FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
+[RUNNING] `target/debug/foo[EXE]`
+
+"#]])
+        .run();
+
+    p.cargo(r#"run --config 'env.ENV_TEST="two"'"#)
+        .with_stdout_data(str![[r#"
+two
+
+"#]])
+        .with_stderr_data(str![[r#"
+[COMPILING] foo v0.5.0 ([ROOT]/foo)
+[FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
+[RUNNING] `target/debug/foo[EXE]`
+
+"#]])
+        .run();
+    // This identical cargo invocation is to ensure no rebuild happen.
+    p.cargo(r#"run --config 'env.ENV_TEST="two"'"#)
+        .with_stdout_data(str![[r#"
+two
+
+"#]])
+        .with_stderr_data(str![[r#"
+[FINISHED] `dev` profile [unoptimized + debuginfo] target(s) in [ELAPSED]s
+[RUNNING] `target/debug/foo[EXE]`
+
+"#]])
         .run();
 }
