@@ -75,6 +75,8 @@ fn namespaced_invalid_feature() {
 Caused by:
   feature `bar` includes `baz` which is neither a dependency nor another feature
 
+  [HELP] a feature with a similar name exists: `bar`
+
 "#]])
         .run();
 }
@@ -417,7 +419,11 @@ regex
 
     p.cargo("run --features lazy_static")
         .with_stderr_data(str![[r#"
-[ERROR] Package `foo v0.1.0 ([ROOT]/foo)` does not have feature `lazy_static`. It has an optional dependency with that name, but that dependency uses the "dep:" syntax in the features table, so it does not have an implicit feature with that name.
+[ERROR] package `foo v0.1.0 ([ROOT]/foo)` does not have feature `lazy_static`
+
+[HELP] an optional dependency with that name exists, but the `features` table includes it with the "dep:" syntax so it does not have an implicit feature with that name
+Dependency `lazy_static` would be enabled by these features:
+	- `regex`
 
 "#]])
         .with_status(101)
@@ -1006,6 +1012,9 @@ homepage = "https://example.com/"
 readme = false
 license = "MIT"
 
+[features]
+feat = ["opt-dep1"]
+
 [lib]
 name = "foo"
 path = "src/lib.rs"
@@ -1018,9 +1027,6 @@ optional = true
 version = "1.0"
 optional = true
 
-[features]
-feat = ["opt-dep1"]
-
 "##]],
         )],
     );
@@ -1143,6 +1149,11 @@ homepage = "https://example.com/"
 readme = false
 license = "MIT"
 
+[features]
+feat1 = []
+feat2 = ["dep:bar"]
+feat3 = ["feat2"]
+
 [lib]
 name = "foo"
 path = "src/lib.rs"
@@ -1151,11 +1162,6 @@ path = "src/lib.rs"
 version = "1.0"
 optional = true
 
-[features]
-feat1 = []
-feat2 = ["dep:bar"]
-feat3 = ["feat2"]
-
 "##]],
         )],
     );
