@@ -11,6 +11,7 @@ mod bad_manifest_path;
 mod bench;
 mod binary_name;
 mod build;
+mod build_dir;
 mod build_plan;
 mod build_script;
 mod build_script_env;
@@ -85,6 +86,7 @@ mod doc;
 mod docscrape;
 mod edition;
 mod error;
+mod feature_unification;
 mod features;
 mod features2;
 mod features_namespaced;
@@ -133,9 +135,11 @@ mod open_namespaces;
 mod owner;
 mod package;
 mod package_features;
+mod package_message_format;
 mod patch;
 mod path;
 mod paths;
+mod pgo;
 mod pkgid;
 mod precise_pre_release;
 mod proc_macro;
@@ -165,6 +169,7 @@ mod rustdoc_extern_html;
 mod rustdocflags;
 mod rustflags;
 mod rustup;
+mod sbom;
 mod script;
 mod search;
 mod shell_quoting;
@@ -174,14 +179,13 @@ mod standard_lib;
 mod test;
 mod timings;
 mod tool_paths;
-mod tree;
-mod tree_graph_features;
 mod unit_graph;
 mod update;
 mod vendor;
 mod verify_project;
 mod version;
 mod warn_on_failure;
+mod warning_override;
 mod weak_dep_features;
 mod workspaces;
 mod yank;
