@@ -130,11 +130,7 @@ fn vendor_path_specified() {
         "deps/.vendor"
     };
 
-    let output = p
-        .cargo("vendor --respect-source-config")
-        .arg(path)
-        .exec_with_output()
-        .unwrap();
+    let output = p.cargo("vendor --respect-source-config").arg(path).run();
     // Assert against original output to ensure that
     // path is normalized by `ops::vendor` on Windows.
     assert_eq!(
@@ -1425,10 +1421,7 @@ fn git_complex() {
         .file("src/lib.rs", "")
         .build();
 
-    let output = p
-        .cargo("vendor --respect-source-config")
-        .exec_with_output()
-        .unwrap();
+    let output = p.cargo("vendor --respect-source-config").run();
     let output = String::from_utf8(output.stdout).unwrap();
     p.change_file(".cargo/config.toml", &output);
 
@@ -1505,10 +1498,7 @@ fn git_deterministic() {
         .file("src/lib.rs", "")
         .build();
 
-    let output = p
-        .cargo("vendor --respect-source-config")
-        .exec_with_output()
-        .unwrap();
+    let output = p.cargo("vendor --respect-source-config").run();
     let output = String::from_utf8(output.stdout).unwrap();
     p.change_file(".cargo/config.toml", &output);
 
@@ -1744,10 +1734,7 @@ fn config_instructions_works() {
         )
         .file("src/lib.rs", "")
         .build();
-    let output = p
-        .cargo("vendor --respect-source-config")
-        .exec_with_output()
-        .unwrap();
+    let output = p.cargo("vendor --respect-source-config").run();
     let output = String::from_utf8(output.stdout).unwrap();
     p.change_file(".cargo/config.toml", &output);
 
@@ -1952,3 +1939,131 @@ fn vendor_crate_with_ws_inherit() {
 "#]])
         .run();
 }
+
+#[cargo_test]
+fn dont_delete_non_registry_sources_with_respect_source_config() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                [package]
+                name = "foo"
+                version = "0.1.0"
+
+                [dependencies]
+                log = "0.3.5"
+            "#,
+        )
+        .file("src/lib.rs", "")
+        .build();
+
+    Package::new("log", "0.3.5").publish();
+
+    p.cargo("vendor --respect-source-config").run();
+    let lock = p.read_file("vendor/log/Cargo.toml");
+    assert!(lock.contains("version = \"0.3.5\""));
+
+    add_crates_io_vendor_config(&p);
+    p.cargo("vendor --respect-source-config new-vendor-dir")
+        .with_stderr_data(str![[r#"
+   Vendoring log v0.3.5 ([ROOT]/foo/vendor/log) to new-vendor-dir/log
+To use vendored sources, add this to your .cargo/config.toml for this project:
+
+
+"#]])
+        .with_stdout_data(str![[r#"
+[source.crates-io]
+replace-with = "vendored-sources"
+
+[source.vendored-sources]
+directory = "new-vendor-dir"
+
+"#]])
+        .run();
+}
+
+#[cargo_test]
+fn error_loading_which_lock() {
+    // Tests an error message to make sure it is clear which
+    // manifest/workspace caused the problem. In this particular case, it was
+    // because the 2024 edition wants to know which rust version is in use.
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                [package]
+                name = "a"
+                version = "0.1.0"
+            "#,
+        )
+        .file("src/lib.rs", "")
+        .file(
+            "b/Cargo.toml",
+            r#"
+                [package]
+                name = "b"
+                version = "0.1.0"
+                edition = "2024"
+            "#,
+        )
+        .file("b/src/lib.rs", "")
+        .build();
+
+    p.cargo("vendor --respect-source-config -s b/Cargo.toml")
+        .env("RUSTC", "does-not-exist")
+        .with_status(101)
+        .with_stderr_data(str![[r#"
+[ERROR] failed to sync
+
+Caused by:
+  failed to load lockfile for [ROOT]/foo/b
+
+Caused by:
+  could not execute process `does-not-exist -vV` (never executed)
+
+Caused by:
+  [NOT_FOUND]
+
+"#]])
+        .run();
+}
+
+#[cargo_test]
+fn error_downloading() {
+    // Tests the error message when downloading packages.
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                [package]
+                name = "foo"
+                version = "0.1.0"
+
+                [dependencies]
+                bar = "1.0"
+            "#,
+        )
+        .file("src/lib.rs", "")
+        .build();
+
+    Package::new("bar", "1.0.0").publish();
+    p.cargo("generate-lockfile").run();
+    std::fs::remove_file(cargo_test_support::paths::root().join("dl/bar/1.0.0/download")).unwrap();
+    p.cargo("vendor --respect-source-config")
+        .with_status(101)
+        .with_stderr_data(str![[r#"
+[DOWNLOADING] crates ...
+[ERROR] failed to sync
+
+Caused by:
+  failed to download packages for [ROOT]/foo
+
+Caused by:
+  failed to download from `[ROOTURL]/dl/bar/1.0.0/download`
+
+Caused by:
+  [37] Could[..]t read a file:// file (Couldn't open file [ROOT]/dl/bar/1.0.0/download)
+
+"#]])
+        .run();
+}
