@@ -164,12 +164,16 @@ pub fn association_with_structures_should_not_trigger_the_lint() {
 
 pub fn hard_coded_allowed() {
     let _ = 1f16 + 1f16;
+    //~^ arithmetic_side_effects
     let _ = 1f32 + 1f32;
     let _ = 1f64 + 1f64;
     let _ = 1f128 + 1f128;
+    //~^ arithmetic_side_effects
 
     let _ = Saturating(0u32) + Saturating(0u32);
     let _ = String::new() + "";
+    let _ = String::new() + &String::new();
+    //~^ arithmetic_side_effects
     let _ = Wrapping(0u32) + Wrapping(0u32);
 
     let saturating: Saturating<u32> = Saturating(0u32);
@@ -305,114 +309,221 @@ pub fn unknown_ops_or_runtime_ops_that_can_overflow() {
 
     // Assign
     _n += 1;
+    //~^ arithmetic_side_effects
     _n += &1;
+    //~^ arithmetic_side_effects
     _n -= 1;
+    //~^ arithmetic_side_effects
     _n -= &1;
+    //~^ arithmetic_side_effects
     _n /= 0;
+    //~^ arithmetic_side_effects
     _n /= &0;
+    //~^ arithmetic_side_effects
     _n %= 0;
+    //~^ arithmetic_side_effects
     _n %= &0;
+    //~^ arithmetic_side_effects
     _n *= 2;
+    //~^ arithmetic_side_effects
     _n *= &2;
+    //~^ arithmetic_side_effects
     _n += -1;
+    //~^ arithmetic_side_effects
     _n += &-1;
+    //~^ arithmetic_side_effects
     _n -= -1;
+    //~^ arithmetic_side_effects
     _n -= &-1;
+    //~^ arithmetic_side_effects
     _n /= -0;
+    //~^ arithmetic_side_effects
     _n /= &-0;
+    //~^ arithmetic_side_effects
     _n %= -0;
+    //~^ arithmetic_side_effects
     _n %= &-0;
+    //~^ arithmetic_side_effects
     _n *= -2;
+    //~^ arithmetic_side_effects
     _n *= &-2;
+    //~^ arithmetic_side_effects
     _custom += Custom;
+    //~^ arithmetic_side_effects
     _custom += &Custom;
+    //~^ arithmetic_side_effects
     _custom -= Custom;
+    //~^ arithmetic_side_effects
     _custom -= &Custom;
+    //~^ arithmetic_side_effects
     _custom /= Custom;
+    //~^ arithmetic_side_effects
     _custom /= &Custom;
+    //~^ arithmetic_side_effects
     _custom %= Custom;
+    //~^ arithmetic_side_effects
     _custom %= &Custom;
+    //~^ arithmetic_side_effects
     _custom *= Custom;
+    //~^ arithmetic_side_effects
     _custom *= &Custom;
+    //~^ arithmetic_side_effects
     _custom >>= Custom;
+    //~^ arithmetic_side_effects
     _custom >>= &Custom;
+    //~^ arithmetic_side_effects
     _custom <<= Custom;
+    //~^ arithmetic_side_effects
     _custom <<= &Custom;
+    //~^ arithmetic_side_effects
     _custom += -Custom;
+    //~^ arithmetic_side_effects
     _custom += &-Custom;
+    //~^ arithmetic_side_effects
     _custom -= -Custom;
+    //~^ arithmetic_side_effects
     _custom -= &-Custom;
+    //~^ arithmetic_side_effects
     _custom /= -Custom;
+    //~^ arithmetic_side_effects
     _custom /= &-Custom;
+    //~^ arithmetic_side_effects
     _custom %= -Custom;
+    //~^ arithmetic_side_effects
     _custom %= &-Custom;
+    //~^ arithmetic_side_effects
     _custom *= -Custom;
+    //~^ arithmetic_side_effects
     _custom *= &-Custom;
+    //~^ arithmetic_side_effects
     _custom >>= -Custom;
+    //~^ arithmetic_side_effects
     _custom >>= &-Custom;
+    //~^ arithmetic_side_effects
     _custom <<= -Custom;
+    //~^ arithmetic_side_effects
     _custom <<= &-Custom;
+    //~^ arithmetic_side_effects
 
     // Binary
     _n = _n + 1;
+    //~^ arithmetic_side_effects
     _n = _n + &1;
+    //~^ arithmetic_side_effects
     _n = 1 + _n;
+    //~^ arithmetic_side_effects
     _n = &1 + _n;
+    //~^ arithmetic_side_effects
     _n = _n - 1;
+    //~^ arithmetic_side_effects
     _n = _n - &1;
+    //~^ arithmetic_side_effects
     _n = 1 - _n;
+    //~^ arithmetic_side_effects
     _n = &1 - _n;
+    //~^ arithmetic_side_effects
     _n = _n / 0;
+    //~^ arithmetic_side_effects
     _n = _n / &0;
+    //~^ arithmetic_side_effects
     _n = _n % 0;
+    //~^ arithmetic_side_effects
     _n = _n % &0;
+    //~^ arithmetic_side_effects
     _n = _n * 2;
+    //~^ arithmetic_side_effects
     _n = _n * &2;
+    //~^ arithmetic_side_effects
     _n = 2 * _n;
+    //~^ arithmetic_side_effects
     _n = &2 * _n;
+    //~^ arithmetic_side_effects
     _n = 23 + &85;
+    //~^ arithmetic_side_effects
     _n = &23 + 85;
+    //~^ arithmetic_side_effects
     _n = &23 + &85;
+    //~^ arithmetic_side_effects
     _custom = _custom + _custom;
+    //~^ arithmetic_side_effects
     _custom = _custom + &_custom;
+    //~^ arithmetic_side_effects
     _custom = Custom + _custom;
+    //~^ arithmetic_side_effects
     _custom = &Custom + _custom;
+    //~^ arithmetic_side_effects
     _custom = _custom - Custom;
+    //~^ arithmetic_side_effects
     _custom = _custom - &Custom;
+    //~^ arithmetic_side_effects
     _custom = Custom - _custom;
+    //~^ arithmetic_side_effects
     _custom = &Custom - _custom;
+    //~^ arithmetic_side_effects
     _custom = _custom / Custom;
+    //~^ arithmetic_side_effects
     _custom = _custom / &Custom;
+    //~^ arithmetic_side_effects
     _custom = _custom % Custom;
+    //~^ arithmetic_side_effects
     _custom = _custom % &Custom;
+    //~^ arithmetic_side_effects
     _custom = _custom * Custom;
+    //~^ arithmetic_side_effects
     _custom = _custom * &Custom;
+    //~^ arithmetic_side_effects
     _custom = Custom * _custom;
+    //~^ arithmetic_side_effects
     _custom = &Custom * _custom;
+    //~^ arithmetic_side_effects
     _custom = Custom + &Custom;
+    //~^ arithmetic_side_effects
     _custom = &Custom + Custom;
+    //~^ arithmetic_side_effects
     _custom = &Custom + &Custom;
+    //~^ arithmetic_side_effects
     _custom = _custom >> _custom;
+    //~^ arithmetic_side_effects
     _custom = _custom >> &_custom;
+    //~^ arithmetic_side_effects
     _custom = Custom << _custom;
+    //~^ arithmetic_side_effects
     _custom = &Custom << _custom;
+    //~^ arithmetic_side_effects
 
     // Method
     _n.saturating_div(0);
+    //~^ arithmetic_side_effects
     _n.wrapping_div(0);
+    //~^ arithmetic_side_effects
     _n.wrapping_rem(0);
+    //~^ arithmetic_side_effects
     _n.wrapping_rem_euclid(0);
+    //~^ arithmetic_side_effects
 
     _n.saturating_div(_n);
+    //~^ arithmetic_side_effects
     _n.wrapping_div(_n);
+    //~^ arithmetic_side_effects
     _n.wrapping_rem(_n);
+    //~^ arithmetic_side_effects
     _n.wrapping_rem_euclid(_n);
+    //~^ arithmetic_side_effects
+
+    _n.saturating_div(*Box::new(_n));
+    //~^ arithmetic_side_effects
 
     // Unary
     _n = -_n;
+    //~^ arithmetic_side_effects
     _n = -&_n;
+    //~^ arithmetic_side_effects
     _custom = -_custom;
+    //~^ arithmetic_side_effects
     _custom = -&_custom;
+    //~^ arithmetic_side_effects
+    _ = -*Box::new(_n);
+    //~^ arithmetic_side_effects
 }
 
 // Copied and pasted from the `integer_arithmetic` lint for comparison.
@@ -422,10 +533,15 @@ pub fn integer_arithmetic() {
     let mut var2 = -1i32;
 
     1 + i;
+    //~^ arithmetic_side_effects
     i * 2;
+    //~^ arithmetic_side_effects
     1 % i / 2;
+    //~^ arithmetic_side_effects
     i - 2 + 2 - i;
+    //~^ arithmetic_side_effects
     -i;
+    //~^ arithmetic_side_effects
     i >> 1;
     i << 1;
 
@@ -437,18 +553,27 @@ pub fn integer_arithmetic() {
     i ^ 1;
 
     i += 1;
+    //~^ arithmetic_side_effects
     i -= 1;
+    //~^ arithmetic_side_effects
     i *= 2;
+    //~^ arithmetic_side_effects
     i /= 2;
     i /= 0;
+    //~^ arithmetic_side_effects
     i /= -1;
     i /= var1;
+    //~^ arithmetic_side_effects
     i /= var2;
+    //~^ arithmetic_side_effects
     i %= 2;
     i %= 0;
+    //~^ arithmetic_side_effects
     i %= -1;
     i %= var1;
+    //~^ arithmetic_side_effects
     i %= var2;
+    //~^ arithmetic_side_effects
     i <<= 3;
     i >>= 2;
 
@@ -459,6 +584,7 @@ pub fn integer_arithmetic() {
 
 pub fn issue_10583(a: u16) -> u16 {
     10 / a
+    //~^ arithmetic_side_effects
 }
 
 pub fn issue_10767() {
@@ -513,10 +639,12 @@ pub fn issue_11392() {
 pub fn issue_11393() {
     fn example_div(x: Wrapping<i32>, maybe_zero: Wrapping<i32>) -> Wrapping<i32> {
         x / maybe_zero
+        //~^ arithmetic_side_effects
     }
 
     fn example_rem(x: Wrapping<i32>, maybe_zero: Wrapping<i32>) -> Wrapping<i32> {
         x % maybe_zero
+        //~^ arithmetic_side_effects
     }
 
     let [x, maybe_zero] = [1, 0].map(Wrapping);
@@ -528,10 +656,21 @@ pub fn issue_12318() {
     use core::ops::{AddAssign, DivAssign, MulAssign, RemAssign, SubAssign};
     let mut one: i32 = 1;
     one.add_assign(1);
+    //~^ arithmetic_side_effects
     one.div_assign(1);
     one.mul_assign(1);
     one.rem_assign(1);
     one.sub_assign(1);
+    //~^ arithmetic_side_effects
+}
+
+pub fn explicit_methods() {
+    use core::ops::Add;
+    let one: i32 = 1;
+    one.add(&one);
+    //~^ arithmetic_side_effects
+    Box::new(one).add(one);
+    //~^ arithmetic_side_effects
 }
 
 fn main() {}
