@@ -7,9 +7,11 @@ extern crate proc_macros;
 use proc_macros::{external, with_span};
 
 fn main() {
-    let i = 0u32 as u64;
+    let i = 0u32 as u64; //~ as_conversions
 
     let j = &i as *const u64 as *mut u64;
+    //~^ as_conversions
+    //~| as_conversions
 
     external!(0u32 as u64);
 }
@@ -17,7 +19,7 @@ fn main() {
 with_span!(
     span
 
-    fn coverting() {
+    fn converting() {
         let x = 0u32 as u64;
     }
 );
