@@ -12,31 +12,37 @@ fn main() {
     // Should lint
     // precedence
     if a {
+        //~^ bool_to_int_with_if
         1
     } else {
         0
     };
     if a {
+        //~^ bool_to_int_with_if
         0
     } else {
         1
     };
     if !a {
+        //~^ bool_to_int_with_if
         1
     } else {
         0
     };
     if a || b {
+        //~^ bool_to_int_with_if
         1
     } else {
         0
     };
     if cond(a, b) {
+        //~^ bool_to_int_with_if
         1
     } else {
         0
     };
     if x + y < 4 {
+        //~^ bool_to_int_with_if
         1
     } else {
         0
@@ -46,6 +52,7 @@ fn main() {
     if a {
         123
     } else if b {
+        //~^ bool_to_int_with_if
         1
     } else {
         0
@@ -55,6 +62,7 @@ fn main() {
     if a {
         123
     } else if b {
+        //~^ bool_to_int_with_if
         0
     } else {
         1
@@ -120,6 +128,7 @@ fn main() {
 // Lint returns and type inference
 fn some_fn(a: bool) -> u8 {
     if a { 1 } else { 0 }
+    //~^ bool_to_int_with_if
 }
 
 fn side_effect() {}
@@ -148,3 +157,27 @@ fn if_let(a: Enum, b: Enum) {
         0
     };
 }
+
+fn issue14628() {
+    macro_rules! mac {
+        (if $cond:expr, $then:expr, $else:expr) => {
+            if $cond { $then } else { $else }
+        };
+        (zero) => {
+            0
+        };
+        (one) => {
+            1
+        };
+    }
+
+    let _ = if dbg!(4 > 0) { 1 } else { 0 };
+    //~^ bool_to_int_with_if
+
+    let _ = dbg!(if 4 > 0 { 1 } else { 0 });
+    //~^ bool_to_int_with_if
+
+    let _ = mac!(if 4 > 0, 1, 0);
+    let _ = if 4 > 0 { mac!(one) } else { 0 };
+    let _ = if 4 > 0 { 1 } else { mac!(zero) };
+}
