@@ -1,9 +1,12 @@
 #![warn(clippy::copy_iterator)]
+#![allow(clippy::manual_inspect)]
 
 #[derive(Copy, Clone)]
 struct Countdown(u8);
 
 impl Iterator for Countdown {
+    //~^ copy_iterator
+
     type Item = u8;
 
     fn next(&mut self) -> Option<u8> {
