@@ -7,15 +7,17 @@ use std::ptr;
 use std::sync::Once;
 use std::sync::atomic::AtomicUsize;
 
-const ATOMIC: AtomicUsize = AtomicUsize::new(5); //~ ERROR: interior mutable
-const CELL: Cell<usize> = Cell::new(6); //~ ERROR: interior mutable
+const ATOMIC: AtomicUsize = AtomicUsize::new(5);
+//~^ declare_interior_mutable_const
+const CELL: Cell<usize> = Cell::new(6);
+//~^ declare_interior_mutable_const
 const ATOMIC_TUPLE: ([AtomicUsize; 1], Vec<AtomicUsize>, u8) = ([ATOMIC], Vec::new(), 7);
-//~^ ERROR: interior mutable
+//~^ declare_interior_mutable_const
 
 macro_rules! declare_const {
     ($name:ident: $ty:ty = $e:expr) => {
         const $name: $ty = $e;
-        //~^ ERROR: interior mutable
+        //~^ declare_interior_mutable_const
     };
 }
 declare_const!(_ONCE: Once = Once::new());
@@ -43,6 +45,7 @@ mod issue_8493 {
     macro_rules! issue_8493 {
         () => {
             const _BAZ: Cell<usize> = Cell::new(0);
+            //~^ declare_interior_mutable_const
             static _FOOBAR: () = {
                 thread_local! {
                     static _VAR: Cell<i32> = const { Cell::new(0) };
