@@ -18,6 +18,7 @@ struct FooDefault<'a> {
 }
 
 impl std::default::Default for FooDefault<'_> {
+    //~^ derivable_impls
     fn default() -> Self {
         Self {
             a: false,
@@ -39,6 +40,7 @@ impl std::default::Default for FooDefault<'_> {
 struct TupleDefault(bool, i32, u64);
 
 impl std::default::Default for TupleDefault {
+    //~^ derivable_impls
     fn default() -> Self {
         Self(false, 0, 0u64)
     }
@@ -91,6 +93,7 @@ impl Default for FooNDVec {
 struct StrDefault<'a>(&'a str);
 
 impl Default for StrDefault<'_> {
+    //~^ derivable_impls
     fn default() -> Self {
         Self("")
     }
@@ -117,6 +120,7 @@ mac!(0);
 
 struct Y(u32);
 impl Default for Y {
+    //~^ derivable_impls
     fn default() -> Self {
         Self(mac!())
     }
@@ -156,6 +160,7 @@ struct WithoutSelfCurly {
 }
 
 impl Default for WithoutSelfCurly {
+    //~^ derivable_impls
     fn default() -> Self {
         WithoutSelfCurly { a: false }
     }
@@ -164,6 +169,7 @@ impl Default for WithoutSelfCurly {
 struct WithoutSelfParan(bool);
 
 impl Default for WithoutSelfParan {
+    //~^ derivable_impls
     fn default() -> Self {
         WithoutSelfParan(false)
     }
@@ -181,6 +187,58 @@ impl Default for SpecializedImpl2<String> {
     }
 }
 
+pub struct DirectDefaultDefaultCall {
+    v: Vec<i32>,
+}
+
+impl Default for DirectDefaultDefaultCall {
+    //~^ derivable_impls
+    fn default() -> Self {
+        // When calling `Default::default()` in all fields, we know it is the same as deriving.
+        Self { v: Default::default() }
+    }
+}
+
+pub struct EquivalentToDefaultDefaultCallVec {
+    v: Vec<i32>,
+}
+
+impl Default for EquivalentToDefaultDefaultCallVec {
+    //~^ derivable_impls
+    fn default() -> Self {
+        // The body of `<Vec as Default>::default()` is `Vec::new()`, so they are equivalent.
+        Self { v: Vec::new() }
+    }
+}
+
+pub struct S {
+    x: i32,
+}
+
+impl S {
+    fn new() -> S {
+        S { x: 42 }
+    }
+}
+
+impl Default for S {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
+pub struct EquivalentToDefaultDefaultCallLocal {
+    v: S,
+}
+
+impl Default for EquivalentToDefaultDefaultCallLocal {
+    //~^ derivable_impls
+    fn default() -> Self {
+        // The body of `<S as Default>::default()` is `S::new()`, so they are equivalent.
+        Self { v: S::new() }
+    }
+}
+
 // https://github.com/rust-lang/rust-clippy/issues/7654
 
 pub struct Color {
@@ -214,6 +272,7 @@ pub struct RepeatDefault1 {
 }
 
 impl Default for RepeatDefault1 {
+    //~^ derivable_impls
     fn default() -> Self {
         RepeatDefault1 { a: [0; 32] }
     }
@@ -248,6 +307,7 @@ pub enum SimpleEnum {
 }
 
 impl Default for SimpleEnum {
+    //~^ derivable_impls
     fn default() -> Self {
         SimpleEnum::Bar
     }
