@@ -10,16 +10,18 @@
 #![warn(clippy::disallowed_names)]
 
 fn test(foo: ()) {}
-//~^ ERROR: use of a disallowed/placeholder name `foo`
-//~| NOTE: `-D clippy::disallowed-names` implied by `-D warnings`
+//~^ disallowed_names
 
 fn main() {
     let foo = 42;
-    //~^ ERROR: use of a disallowed/placeholder name `foo`
+    //~^ disallowed_names
+
     let baz = 42;
-    //~^ ERROR: use of a disallowed/placeholder name `baz`
+    //~^ disallowed_names
+
     let quux = 42;
-    //~^ ERROR: use of a disallowed/placeholder name `quux`
+    //~^ disallowed_names
+
     // Unlike these others, `bar` is actually considered an acceptable name.
     // Among many other legitimate uses, bar commonly refers to a period of time in music.
     // See https://github.com/rust-lang/rust-clippy/issues/5225.
@@ -31,35 +33,40 @@ fn main() {
 
     match (42, Some(1337), Some(0)) {
         (foo, Some(baz), quux @ Some(_)) => (),
-        //~^ ERROR: use of a disallowed/placeholder name `foo`
-        //~| ERROR: use of a disallowed/placeholder name `baz`
-        //~| ERROR: use of a disallowed/placeholder name `quux`
+        //~^ disallowed_names
+        //~| disallowed_names
+        //~| disallowed_names
         _ => (),
     }
 }
 
 fn issue_1647(mut foo: u8) {
-    //~^ ERROR: use of a disallowed/placeholder name `foo`
+    //~^ disallowed_names
+
     let mut baz = 0;
-    //~^ ERROR: use of a disallowed/placeholder name `baz`
+    //~^ disallowed_names
+
     if let Some(mut quux) = Some(42) {}
-    //~^ ERROR: use of a disallowed/placeholder name `quux`
+    //~^ disallowed_names
 }
 
 fn issue_1647_ref() {
     let ref baz = 0;
-    //~^ ERROR: use of a disallowed/placeholder name `baz`
+    //~^ disallowed_names
+
     if let Some(ref quux) = Some(42) {}
-    //~^ ERROR: use of a disallowed/placeholder name `quux`
+    //~^ disallowed_names
 }
 
 fn issue_1647_ref_mut() {
     let ref mut baz = 0;
-    //~^ ERROR: use of a disallowed/placeholder name `baz`
+    //~^ disallowed_names
+
     if let Some(ref mut quux) = Some(42) {}
-    //~^ ERROR: use of a disallowed/placeholder name `quux`
+    //~^ disallowed_names
 }
 
+#[cfg(test)]
 mod tests {
     fn issue_7305() {
         // `disallowed_names` lint should not be triggered inside of the test code.
