@@ -7,6 +7,7 @@
 
 mod outer {
     //!
+    //~^ empty_docs
 
     /// this is a struct
     struct Bananas {
@@ -15,8 +16,10 @@ mod outer {
     }
 
     ///
+    //~^ empty_docs
     enum Warn {
         ///
+        //~^ empty_docs
         A,
         B,
     }
@@ -28,16 +31,19 @@ mod outer {
     }
 
     #[doc = ""]
+    //~^ empty_docs
     fn warn_about_this() {}
 
     #[doc = ""]
     #[doc = ""]
+    //~^^ empty_docs
     fn this_doesn_warn() {}
 
     #[doc = "a fine function"]
     fn this_is_fine() {}
 
     ///
+    //~^ empty_docs
     mod inner {
         ///
         fn dont_warn_inner_outer() {
@@ -51,6 +57,7 @@ mod outer {
 
         fn warn() {
             /*! */
+            //~^ empty_docs
         }
 
         fn dont_warn() {
@@ -59,6 +66,7 @@ mod outer {
 
         trait NoDoc {
             ///
+            //~^ empty_docs
             fn some() {}
         }
     }
@@ -67,6 +75,7 @@ mod outer {
         /// lint y
         x: i32,
         ///
+        //~^ empty_docs
         y: i32,
     }
 }
@@ -75,7 +84,7 @@ mod issue_12377 {
     use proc_macro_attr::with_empty_docs;
 
     #[with_empty_docs]
-    extern "C" {
+    unsafe extern "C" {
         type Test;
     }
 
