@@ -16,10 +16,13 @@ fn main() {
     let mut vec = vec![1, 2, 3, 4];
 
     for _ in vec.iter() {}
+    //~^ explicit_iter_loop
     for _ in vec.iter_mut() {}
+    //~^ explicit_iter_loop
 
     let rvec = &vec;
     for _ in rvec.iter() {}
+    //~^ explicit_iter_loop
 
     let rmvec = &mut vec;
     for _ in rmvec.iter() {}
@@ -29,36 +32,48 @@ fn main() {
     for _ in &mut vec {} // these are fine
 
     for _ in [1, 2, 3].iter() {}
+    //~^ explicit_iter_loop
 
     for _ in (&mut [1, 2, 3]).iter() {}
 
     for _ in [0; 32].iter() {}
+    //~^ explicit_iter_loop
     for _ in [0; 33].iter() {}
+    //~^ explicit_iter_loop
 
     let ll: LinkedList<()> = LinkedList::new();
     for _ in ll.iter() {}
+    //~^ explicit_iter_loop
     let rll = &ll;
     for _ in rll.iter() {}
+    //~^ explicit_iter_loop
 
     let vd: VecDeque<()> = VecDeque::new();
     for _ in vd.iter() {}
+    //~^ explicit_iter_loop
     let rvd = &vd;
     for _ in rvd.iter() {}
+    //~^ explicit_iter_loop
 
     let bh: BinaryHeap<()> = BinaryHeap::new();
     for _ in bh.iter() {}
+    //~^ explicit_iter_loop
 
     let hm: HashMap<(), ()> = HashMap::new();
     for _ in hm.iter() {}
+    //~^ explicit_iter_loop
 
     let bt: BTreeMap<(), ()> = BTreeMap::new();
     for _ in bt.iter() {}
+    //~^ explicit_iter_loop
 
     let hs: HashSet<()> = HashSet::new();
     for _ in hs.iter() {}
+    //~^ explicit_iter_loop
 
     let bs: BTreeSet<()> = BTreeSet::new();
     for _ in bs.iter() {}
+    //~^ explicit_iter_loop
 
     struct NoIntoIter();
     impl NoIntoIter {
@@ -148,8 +163,23 @@ fn main() {
     }
     let mut x = CustomType;
     for _ in x.iter() {}
+    //~^ explicit_iter_loop
     for _ in x.iter_mut() {}
+    //~^ explicit_iter_loop
 
     let r = &x;
     for _ in r.iter() {}
+    //~^ explicit_iter_loop
+}
+
+#[clippy::msrv = "1.79"]
+pub fn issue_13184() {
+    // https://github.com/rust-lang/rust-clippy/issues/13184
+    // No need to fix, as IntoIterator for Box is valid starting from 1.80
+    let mut values: Box<[u32]> = Box::new([1, 2]);
+    for _ in values.iter() {}
+    for _ in values.iter_mut() {}
+
+    let rvalues = &values;
+    for _ in rvalues.iter() {}
 }
