@@ -1,4 +1,3 @@
-#![feature(const_fn_floating_point_arithmetic)]
 #![warn(clippy::suboptimal_flops)]
 
 /// Allow suboptimal_flops in constant context
@@ -9,17 +8,25 @@ pub const fn const_context() {
 
 pub fn issue9391(degrees: i64) {
     let _ = degrees as f64 * std::f64::consts::PI / 180.0;
+    //~^ suboptimal_flops
     let _ = degrees as f64 * 180.0 / std::f64::consts::PI;
+    //~^ suboptimal_flops
 }
 
 fn main() {
     let x = 3f32;
     let _ = x * 180f32 / std::f32::consts::PI;
+    //~^ suboptimal_flops
     let _ = 90. * 180f64 / std::f64::consts::PI;
+    //~^ suboptimal_flops
     let _ = 90.5 * 180f64 / std::f64::consts::PI;
+    //~^ suboptimal_flops
     let _ = x * std::f32::consts::PI / 180f32;
+    //~^ suboptimal_flops
     let _ = 90. * std::f32::consts::PI / 180f32;
+    //~^ suboptimal_flops
     let _ = 90.5 * std::f32::consts::PI / 180f32;
+    //~^ suboptimal_flops
     // let _ = 90.5 * 80. * std::f32::consts::PI / 180f32;
     // Cases where the lint shouldn't be applied
     let _ = x * 90f32 / std::f32::consts::PI;
