@@ -6,8 +6,9 @@
 fn long_span() {
     let _: Option<i32> = Some(1)
         .map(|x| {
-        //~^ ERROR: called `map(..).flatten()` on `Option`
-        //~| NOTE: `-D clippy::map-flatten` implied by `-D warnings`
+        //~^ map_flatten
+
+
             if x <= 5 {
                 Some(x)
             } else {
@@ -18,7 +19,8 @@ fn long_span() {
 
     let _: Result<i32, i32> = Ok(1)
         .map(|x| {
-        //~^ ERROR: called `map(..).flatten()` on `Result`
+        //~^ map_flatten
+
             if x == 1 {
                 Ok(x)
             } else {
@@ -31,7 +33,8 @@ fn long_span() {
     fn do_something() { }
     let _: Result<i32, i32> = result
         .map(|res| {
-        //~^ ERROR: called `map(..).flatten()` on `Result`
+        //~^ map_flatten
+
             if res > 0 {
                 do_something();
                 Ok(res)
@@ -44,7 +47,8 @@ fn long_span() {
     let _: Vec<_> = vec![5_i8; 6]
         .into_iter()
         .map(|some_value| {
-        //~^ ERROR: called `map(..).flatten()` on `Iterator`
+        //~^ map_flatten
+
             if some_value > 3 {
                 Some(some_value)
             } else {
@@ -55,6 +59,18 @@ fn long_span() {
         .collect();
 }
 
+#[allow(clippy::useless_vec)]
+fn no_suggestion_if_comments_present() {
+    let vec = vec![vec![1, 2, 3]];
+    let _ = vec
+        .iter()
+        // a lovely comment explaining the code in very detail
+        .map(|x| x.iter())
+        //~^ map_flatten
+        // the answer to life, the universe and everything could be here
+        .flatten();
+}
+
 fn main() {
     long_span();
 }
