@@ -1,4 +1,3 @@
-//@run-rustfix
 #![warn(clippy::match_single_binding)]
 #![allow(
     unused,
@@ -32,12 +31,14 @@ fn main() {
     let c = 3;
     // Lint
     match (a, b, c) {
+        //~^ match_single_binding
         (x, y, z) => {
             println!("{} {} {}", x, y, z);
         },
     }
     // Lint
     match (a, b, c) {
+        //~^ match_single_binding
         (x, y, z) => println!("{} {} {}", x, y, z),
     }
     // Ok
@@ -55,10 +56,12 @@ fn main() {
     }
     // Lint
     match a {
+        //~^ match_single_binding
         _ => println!("whatever"),
     }
     // Lint
     match a {
+        //~^ match_single_binding
         _ => {
             let x = 29;
             println!("x has a value of {}", x);
@@ -66,6 +69,7 @@ fn main() {
     }
     // Lint
     match a {
+        //~^ match_single_binding
         _ => {
             let e = 5 * a;
             if e >= 5 {
@@ -76,24 +80,29 @@ fn main() {
     // Lint
     let p = Point { x: 0, y: 7 };
     match p {
+        //~^ match_single_binding
         Point { x, y } => println!("Coords: ({}, {})", x, y),
     }
     // Lint
     match p {
+        //~^ match_single_binding
         Point { x: x1, y: y1 } => println!("Coords: ({}, {})", x1, y1),
     }
     // Lint
     let x = 5;
     match x {
+        //~^ match_single_binding
         ref r => println!("Got a reference to {}", r),
     }
     // Lint
     let mut x = 5;
     match x {
+        //~^ match_single_binding
         ref mut mr => println!("Got a mutable reference to {}", mr),
     }
     // Lint
     let product = match coords() {
+        //~^ match_single_binding
         Point { x, y } => x * y,
     };
     // Lint
@@ -102,6 +111,7 @@ fn main() {
     let _ = v
         .iter()
         .map(|i| match i.unwrap() {
+            //~^ match_single_binding
             unwrapped => unwrapped,
         })
         .collect::<Vec<u8>>();
@@ -128,6 +138,7 @@ fn main() {
     // Lint
     let x = 1;
     match x {
+        //~^ match_single_binding
         // =>
         _ => println!("Not an array index start"),
     }
@@ -137,6 +148,7 @@ fn issue_8723() {
     let (mut val, idx) = ("a b", 1);
 
     val = match val.split_at(idx) {
+        //~^ match_single_binding
         (pre, suf) => {
             println!("{}", pre);
             suf
@@ -150,12 +162,14 @@ fn side_effects() {}
 
 fn issue_9575() {
     let _ = || match side_effects() {
+        //~^ match_single_binding
         _ => println!("Needs curlies"),
     };
 }
 
 fn issue_9725(r: Option<u32>) {
     match r {
+        //~^ match_single_binding
         x => match x {
             Some(_) => {
                 println!("Some");
@@ -169,40 +183,69 @@ fn issue_9725(r: Option<u32>) {
 
 fn issue_10447() -> usize {
     match 1 {
+        //~^ match_single_binding
         _ => (),
     }
 
     let a = match 1 {
+        //~^ match_single_binding
         _ => (),
     };
 
     match 1 {
+        //~^ match_single_binding
         _ => side_effects(),
     }
 
     let b = match 1 {
+        //~^ match_single_binding
         _ => side_effects(),
     };
 
     match 1 {
+        //~^ match_single_binding
         _ => println!("1"),
     }
 
     let c = match 1 {
+        //~^ match_single_binding
         _ => println!("1"),
     };
 
     let in_expr = [
         match 1 {
+            //~^ match_single_binding
             _ => (),
         },
         match 1 {
+            //~^ match_single_binding
             _ => side_effects(),
         },
         match 1 {
+            //~^ match_single_binding
             _ => println!("1"),
         },
     ];
 
     2
 }
+
+fn issue14634() {
+    macro_rules! id {
+        ($i:ident) => {
+            $i
+        };
+    }
+    match dbg!(3) {
+        _ => println!("here"),
+    }
+    //~^^^ match_single_binding
+    match dbg!(3) {
+        id!(a) => println!("found {a}"),
+    }
+    //~^^^ match_single_binding
+    let id!(_a) = match dbg!(3) {
+        id!(b) => dbg!(b + 1),
+    };
+    //~^^^ match_single_binding
+}
