@@ -13,27 +13,34 @@ fn should_lint() {
     let v: Vec<i32> = Vec::new();
     let mut acc = 0;
     v.iter().for_each(|elem| {
+        //~^ needless_for_each
         acc += elem;
     });
     v.into_iter().for_each(|elem| {
+        //~^ needless_for_each
         acc += elem;
     });
 
     [1, 2, 3].iter().for_each(|elem| {
+        //~^ needless_for_each
         acc += elem;
     });
 
     let mut hash_map: HashMap<i32, i32> = HashMap::new();
     hash_map.iter().for_each(|(k, v)| {
+        //~^ needless_for_each
         acc += k + v;
     });
     hash_map.iter_mut().for_each(|(k, v)| {
+        //~^ needless_for_each
         acc += *k + *v;
     });
     hash_map.keys().for_each(|k| {
+        //~^ needless_for_each
         acc += k;
     });
     hash_map.values().for_each(|v| {
+        //~^ needless_for_each
         acc += v;
     });
 
@@ -41,6 +48,7 @@ fn should_lint() {
         Vec::new()
     }
     my_vec().iter().for_each(|elem| {
+        //~^ needless_for_each
         acc += elem;
     });
 }
@@ -113,6 +121,10 @@ fn should_not_lint() {
     let _ = v.iter().for_each(|elem| {
         acc += elem;
     });
+    // `for_each` has a closure with an unsafe block.
+    v.iter().for_each(|elem| unsafe {
+        acc += elem;
+    });
 }
 
 fn main() {}
