@@ -3,6 +3,7 @@
 #![allow(
     clippy::drop_non_drop,
     clippy::implicit_clone,
+    clippy::pathbuf_init_then_push,
     clippy::uninlined_format_args,
     clippy::unnecessary_literal_unwrap
 )]
@@ -12,23 +13,31 @@ use std::path::Path;
 
 fn main() {
     let _s = ["lorem", "ipsum"].join(" ").to_string();
+    //~^ redundant_clone
 
     let s = String::from("foo");
     let _s = s.clone();
+    //~^ redundant_clone
 
     let s = String::from("foo");
     let _s = s.to_string();
+    //~^ redundant_clone
 
     let s = String::from("foo");
     let _s = s.to_owned();
+    //~^ redundant_clone
 
     let _s = Path::new("/a/b/").join("c").to_owned();
+    //~^ redundant_clone
 
     let _s = Path::new("/a/b/").join("c").to_path_buf();
+    //~^ redundant_clone
 
     let _s = OsString::new().to_owned();
+    //~^ redundant_clone
 
     let _s = OsString::new().to_os_string();
+    //~^ redundant_clone
 
     // Check that lint level works
     #[allow(clippy::redundant_clone)]
@@ -40,6 +49,7 @@ fn main() {
 
     let tup = (String::from("foo"),);
     let _t = tup.0.clone();
+    //~^ redundant_clone
 
     let tup_ref = &(String::from("foo"),);
     let _s = tup_ref.0.clone(); // this `.clone()` cannot be removed
@@ -72,6 +82,7 @@ fn main() {
 struct Alpha;
 fn with_branch(a: Alpha, b: bool) -> (Alpha, Alpha) {
     if b { (a.clone(), a.clone()) } else { (Alpha, a) }
+    //~^ redundant_clone
 }
 
 fn cannot_double_move(a: Alpha) -> (Alpha, Alpha) {
@@ -129,7 +140,9 @@ fn borrower_propagation() {
     }
 
     let _s = s.clone();
+    //~^ redundant_clone
     let _t = t.clone();
+    //~^ redundant_clone
 
     #[derive(Clone)]
     struct Foo {
@@ -140,6 +153,7 @@ fn borrower_propagation() {
         let f = Foo { x: 123 };
         let _x = Some(f.x);
         let _f = f.clone();
+        //~^ redundant_clone
     }
 
     {
@@ -152,6 +166,7 @@ fn borrower_propagation() {
 fn not_consumed() {
     let x = std::path::PathBuf::from("home");
     let y = x.clone().join("matthias");
+    //~^ redundant_clone
     // join() creates a new owned PathBuf, does not take a &mut to x variable, thus the .clone() is
     // redundant. (It also does not consume the PathBuf)
 
@@ -206,6 +221,7 @@ fn clone_then_move_cloned() {
     let x = Alpha;
     // ok, data is moved while the clone is in use.
     foo(&x.clone(), move || {
+        //~^ redundant_clone
         let _ = x;
     });
 
@@ -243,3 +259,35 @@ fn false_negative_5707() {
     let _z = x.clone(); // pr 7346 can't lint on `x`
     drop(y);
 }
+
+mod issue10074 {
+    #[derive(Debug, Clone)]
+    enum MyEnum {
+        A = 1,
+    }
+
+    fn false_positive_on_as() {
+        let e = MyEnum::A;
+        let v = e.clone() as u16;
+
+        println!("{e:?}");
+        println!("{v}");
+    }
+}
+
+mod issue13900 {
+    use std::fmt::Display;
+
+    fn do_something(f: impl Display + Clone) -> String {
+        let g = f.clone();
+        format!("{} + {}", f, g)
+    }
+
+    fn regression() {
+        let mut a = String::new();
+        let mut b = String::new();
+        for _ in 1..10 {
+            b = a.clone();
+        }
+    }
+}
