@@ -1,4 +1,4 @@
-#![allow(dead_code, clippy::explicit_auto_deref, clippy::useless_vec)]
+#![allow(dead_code, clippy::explicit_auto_deref, clippy::useless_vec, clippy::manual_contains)]
 #![warn(clippy::search_is_some)]
 
 fn main() {
@@ -7,37 +7,58 @@ fn main() {
 
     // Check `find().is_some()`, single-line case.
     let _ = v.iter().find(|&x| *x < 0).is_some();
+    //~^ search_is_some
     let _ = (0..1).find(|x| **y == *x).is_some(); // one dereference less
+    //
+    //~^^ search_is_some
     let _ = (0..1).find(|x| *x == 0).is_some();
+    //~^ search_is_some
     let _ = v.iter().find(|x| **x == 0).is_some();
+    //~^ search_is_some
     let _ = (4..5).find(|x| *x == 1 || *x == 3 || *x == 5).is_some();
+    //~^ search_is_some
     let _ = (1..3).find(|x| [1, 2, 3].contains(x)).is_some();
+    //~^ search_is_some
     let _ = (1..3).find(|x| *x == 0 || [1, 2, 3].contains(x)).is_some();
+    //~^ search_is_some
     let _ = (1..3).find(|x| [1, 2, 3].contains(x) || *x == 0).is_some();
+    //~^ search_is_some
     let _ = (1..3)
         .find(|x| [1, 2, 3].contains(x) || *x == 0 || [4, 5, 6].contains(x) || *x == -1)
+        //~^ search_is_some
         .is_some();
 
     // Check `position().is_some()`, single-line case.
     let _ = v.iter().position(|&x| x < 0).is_some();
+    //~^ search_is_some
 
     // Check `rposition().is_some()`, single-line case.
     let _ = v.iter().rposition(|&x| x < 0).is_some();
+    //~^ search_is_some
 
     let s1 = String::from("hello world");
     let s2 = String::from("world");
     // caller of `find()` is a `&`static str`
     let _ = "hello world".find("world").is_some();
+    //~^ search_is_some
     let _ = "hello world".find(&s2).is_some();
+    //~^ search_is_some
     let _ = "hello world".find(&s2[2..]).is_some();
+    //~^ search_is_some
     // caller of `find()` is a `String`
     let _ = s1.find("world").is_some();
+    //~^ search_is_some
     let _ = s1.find(&s2).is_some();
+    //~^ search_is_some
     let _ = s1.find(&s2[2..]).is_some();
+    //~^ search_is_some
     // caller of `find()` is slice of `String`
     let _ = s1[2..].find("world").is_some();
+    //~^ search_is_some
     let _ = s1[2..].find(&s2).is_some();
+    //~^ search_is_some
     let _ = s1[2..].find(&s2[2..]).is_some();
+    //~^ search_is_some
 }
 
 #[allow(clippy::clone_on_copy, clippy::map_clone)]
@@ -54,6 +75,7 @@ mod issue7392 {
             .hand
             .iter()
             .filter(|c| filter_hand.iter().find(|cc| c == cc).is_some())
+            //~^ search_is_some
             .map(|c| c.clone())
             .collect::<Vec<_>>();
     }
@@ -70,6 +92,7 @@ mod issue7392 {
             .hand
             .iter()
             .filter(|(c, _)| filter_hand.iter().find(|cc| c == *cc).is_some())
+            //~^ search_is_some
             .map(|c| c.clone())
             .collect::<Vec<_>>();
     }
@@ -81,23 +104,27 @@ mod issue7392 {
         }
         let vfoo = vec![Foo { foo: 1, bar: 2 }];
         let _ = vfoo.iter().find(|v| v.foo == 1 && v.bar == 2).is_some();
+        //~^ search_is_some
 
         let vfoo = vec![(42, Foo { foo: 1, bar: 2 })];
         let _ = vfoo
             .iter()
             .find(|(i, v)| *i == 42 && v.foo == 1 && v.bar == 2)
+            //~^ search_is_some
             .is_some();
     }
 
     fn index_projection() {
         let vfoo = vec![[0, 1, 2, 3]];
         let _ = vfoo.iter().find(|a| a[0] == 42).is_some();
+        //~^ search_is_some
     }
 
     #[allow(clippy::match_like_matches_macro)]
     fn slice_projection() {
         let vfoo = vec![[0, 1, 2, 3, 0, 1, 2, 3]];
         let _ = vfoo.iter().find(|sub| sub[1..4].len() == 3).is_some();
+        //~^ search_is_some
     }
 
     fn please(x: &u32) -> bool {
@@ -116,16 +143,22 @@ mod issue7392 {
         let x = 19;
         let ppx: &u32 = &x;
         let _ = [ppx].iter().find(|ppp_x: &&&u32| please(**ppp_x)).is_some();
+        //~^ search_is_some
         let _ = [String::from("Hey hey")].iter().find(|s| s.len() == 2).is_some();
+        //~^ search_is_some
 
         let v = vec![3, 2, 1, 0];
         let _ = v.iter().find(|x| deref_enough(**x)).is_some();
+        //~^ search_is_some
         let _ = v.iter().find(|x: &&u32| deref_enough(**x)).is_some();
+        //~^ search_is_some
 
         #[allow(clippy::redundant_closure)]
         let _ = v.iter().find(|x| arg_no_deref(x)).is_some();
+        //~^ search_is_some
         #[allow(clippy::redundant_closure)]
         let _ = v.iter().find(|x: &&u32| arg_no_deref(x)).is_some();
+        //~^ search_is_some
     }
 
     fn field_index_projection() {
@@ -148,6 +181,7 @@ mod issue7392 {
         let _ = vfoo
             .iter()
             .find(|v| v.inner_double.bar[0][0] == 2 && v.inner.bar[0] == 2)
+            //~^ search_is_some
             .is_some();
     }
 
@@ -162,11 +196,13 @@ mod issue7392 {
             inner: vec![Foo { bar: 0 }],
         }];
         let _ = vfoo.iter().find(|v| v.inner[0].bar == 2).is_some();
+        //~^ search_is_some
     }
 
     fn double_deref_index_projection() {
         let vfoo = vec![&&[0, 1, 2, 3]];
         let _ = vfoo.iter().find(|x| (**x)[0] == 9).is_some();
+        //~^ search_is_some
     }
 
     fn method_call_by_ref() {
@@ -180,11 +216,15 @@ mod issue7392 {
         }
         let vfoo = vec![Foo { bar: 1 }];
         let _ = vfoo.iter().find(|v| v.by_ref(&v.bar)).is_some();
+        //~^ search_is_some
     }
 
     fn ref_bindings() {
-        let _ = [&(&1, 2), &(&3, 4), &(&5, 4)].iter().find(|(&x, y)| x == *y).is_some();
-        let _ = [&(&1, 2), &(&3, 4), &(&5, 4)].iter().find(|&(&x, y)| x == *y).is_some();
+        let _ = [&(&1, 2), &(&3, 4), &(&5, 4)]
+            .iter()
+            .find(|&&&(&x, ref y)| x == *y)
+            //~^ search_is_some
+            .is_some();
     }
 
     fn test_string_1(s: &str) -> bool {
@@ -204,7 +244,9 @@ mod issue7392 {
         let lst = &[String::from("Hello"), String::from("world")];
         let v: Vec<&[String]> = vec![lst];
         let _ = v.iter().find(|s| s[0].is_empty()).is_some();
+        //~^ search_is_some
         let _ = v.iter().find(|s| test_string_1(&s[0])).is_some();
+        //~^ search_is_some
 
         // Field projections
         struct FieldProjection<'a> {
@@ -214,8 +256,11 @@ mod issue7392 {
         let instance = FieldProjection { field: &field };
         let v = vec![instance];
         let _ = v.iter().find(|fp| fp.field.is_power_of_two()).is_some();
+        //~^ search_is_some
         let _ = v.iter().find(|fp| test_u32_1(fp.field)).is_some();
+        //~^ search_is_some
         let _ = v.iter().find(|fp| test_u32_2(*fp.field)).is_some();
+        //~^ search_is_some
     }
 }
 
@@ -231,6 +276,7 @@ mod issue9120 {
     fn wrapper<T: Fn(&&u32) -> bool>(v: Vec<u32>, func: T) -> bool {
         #[allow(clippy::redundant_closure)]
         v.iter().find(|x: &&u32| func(x)).is_some()
+        //~^ search_is_some
     }
 
     fn do_tests() {
@@ -240,11 +286,14 @@ mod issue9120 {
 
         #[allow(clippy::redundant_closure)]
         let _ = v.iter().find(|x: &&u32| arg_no_deref_impl(x)).is_some();
+        //~^ search_is_some
 
         #[allow(clippy::redundant_closure)]
         let _ = v.iter().find(|x: &&u32| arg_no_deref_dyn(x)).is_some();
+        //~^ search_is_some
 
         #[allow(clippy::redundant_closure)]
         let _ = v.iter().find(|x: &&u32| (*arg_no_deref_dyn)(x)).is_some();
+        //~^ search_is_some
     }
 }
