@@ -58,8 +58,7 @@ fn should_trigger_lint_with_mutex_guard_in_match_scrutinee() {
     // is preserved until the end of the match, but there is no clear indication that this is the
     // case.
     match mutex.lock().unwrap().foo() {
-        //~^ ERROR: temporary with significant `Drop` in `match` scrutinee will live until the
-        //~| NOTE: this might lead to deadlocks or other unexpected behavior
+        //~^ significant_drop_in_scrutinee
         true => {
             mutex.lock().unwrap().bar();
         },
@@ -146,8 +145,7 @@ fn should_trigger_lint_with_wrapped_mutex() {
     // lifetime is not obvious. Additionally, it is not obvious from looking at the scrutinee that
     // the temporary contains such a type, making it potentially even more surprising.
     match s.lock_m().get_the_value() {
-        //~^ ERROR: temporary with significant `Drop` in `match` scrutinee will live until the
-        //~| NOTE: this might lead to deadlocks or other unexpected behavior
+        //~^ significant_drop_in_scrutinee
         1 => {
             println!("Got 1. Is it still 1?");
             println!("{}", s.lock_m().get_the_value());
@@ -169,8 +167,7 @@ fn should_trigger_lint_with_double_wrapped_mutex() {
     // looking at the scrutinee that the temporary contains such a type, making it potentially even
     // more surprising.
     match s.lock_m_m().get_the_value() {
-        //~^ ERROR: temporary with significant `Drop` in `match` scrutinee will live until the
-        //~| NOTE: this might lead to deadlocks or other unexpected behavior
+        //~^ significant_drop_in_scrutinee
         1 => {
             println!("Got 1. Is it still 1?");
             println!("{}", s.lock_m().get_the_value());
@@ -219,8 +216,7 @@ fn should_trigger_lint_for_vec() {
     // which have significant drops. The types with significant drops are also non-obvious when
     // reading the expression in the scrutinee.
     match counter.temp_increment().len() {
-        //~^ ERROR: temporary with significant `Drop` in `match` scrutinee will live until the
-        //~| NOTE: this might lead to deadlocks or other unexpected behavior
+        //~^ significant_drop_in_scrutinee
         2 => {
             let current_count = counter.i.load(Ordering::Relaxed);
             println!("Current count {}", current_count);
@@ -244,8 +240,7 @@ fn should_trigger_lint_for_tuple_in_scrutinee() {
 
     {
         match (mutex1.lock().unwrap().s.len(), true) {
-            //~^ ERROR: temporary with significant `Drop` in `match` scrutinee will live until
-            //~| NOTE: this might lead to deadlocks or other unexpected behavior
+            //~^ significant_drop_in_scrutinee
             (3, _) => {
                 println!("started");
                 mutex1.lock().unwrap().s.len();
@@ -255,8 +250,7 @@ fn should_trigger_lint_for_tuple_in_scrutinee() {
         };
 
         match (true, mutex1.lock().unwrap().s.len(), true) {
-            //~^ ERROR: temporary with significant `Drop` in `match` scrutinee will live until
-            //~| NOTE: this might lead to deadlocks or other unexpected behavior
+            //~^ significant_drop_in_scrutinee
             (_, 3, _) => {
                 println!("started");
                 mutex1.lock().unwrap().s.len();
@@ -267,10 +261,8 @@ fn should_trigger_lint_for_tuple_in_scrutinee() {
 
         let mutex2 = Mutex::new(StateWithField { s: "two".to_owned() });
         match (mutex1.lock().unwrap().s.len(), true, mutex2.lock().unwrap().s.len()) {
-            //~^ ERROR: temporary with significant `Drop` in `match` scrutinee will live until
-            //~| NOTE: this might lead to deadlocks or other unexpected behavior
-            //~| ERROR: temporary with significant `Drop` in `match` scrutinee will live until
-            //~| NOTE: this might lead to deadlocks or other unexpected behavior
+            //~^ significant_drop_in_scrutinee
+            //~| significant_drop_in_scrutinee
             (3, _, 3) => {
                 println!("started");
                 mutex1.lock().unwrap().s.len();
@@ -322,8 +314,7 @@ fn should_trigger_lint_for_accessing_field_in_mutex_in_one_side_of_binary_op() {
     let mutex = Mutex::new(StateWithField { s: "state".to_owned() });
 
     match mutex.lock().unwrap().s.len() > 1 {
-        //~^ ERROR: temporary with significant `Drop` in `match` scrutinee will live until the
-        //~| NOTE: this might lead to deadlocks or other unexpected behavior
+        //~^ significant_drop_in_scrutinee
         true => {
             mutex.lock().unwrap().s.len();
         },
@@ -331,8 +322,7 @@ fn should_trigger_lint_for_accessing_field_in_mutex_in_one_side_of_binary_op() {
     };
 
     match 1 < mutex.lock().unwrap().s.len() {
-        //~^ ERROR: temporary with significant `Drop` in `match` scrutinee will live until the
-        //~| NOTE: this might lead to deadlocks or other unexpected behavior
+        //~^ significant_drop_in_scrutinee
         true => {
             mutex.lock().unwrap().s.len();
         },
@@ -351,8 +341,8 @@ fn should_trigger_lint_for_accessing_fields_in_mutex_in_both_sides_of_binary_op(
     });
 
     match mutex1.lock().unwrap().s.len() < mutex2.lock().unwrap().s.len() {
-        //~^ ERROR: temporary with significant `Drop` in `match` scrutinee will live until the
-        //~| NOTE: this might lead to deadlocks or other unexpected behavior
+        //~^ significant_drop_in_scrutinee
+        //~| significant_drop_in_scrutinee
         true => {
             println!(
                 "{} < {}",
@@ -364,8 +354,8 @@ fn should_trigger_lint_for_accessing_fields_in_mutex_in_both_sides_of_binary_op(
     };
 
     match mutex1.lock().unwrap().s.len() >= mutex2.lock().unwrap().s.len() {
-        //~^ ERROR: temporary with significant `Drop` in `match` scrutinee will live until the
-        //~| NOTE: this might lead to deadlocks or other unexpected behavior
+        //~^ significant_drop_in_scrutinee
+        //~| significant_drop_in_scrutinee
         true => {
             println!(
                 "{} >= {}",
@@ -401,8 +391,7 @@ fn should_trigger_lint_for_return_from_closure_in_scrutinee() {
     // Should trigger lint because the temporary with a significant drop is returned from the
     // closure but not used directly in any match arms, so it has a potentially surprising lifetime.
     match get_mutex_guard().s.len() > 1 {
-        //~^ ERROR: temporary with significant `Drop` in `match` scrutinee will live until the
-        //~| NOTE: this might lead to deadlocks or other unexpected behavior
+        //~^ significant_drop_in_scrutinee
         true => {
             mutex1.lock().unwrap().s.len();
         },
@@ -420,8 +409,7 @@ fn should_trigger_lint_for_return_from_match_in_scrutinee() {
     // significant drop is but not used directly in any match arms, so it has a potentially
     // surprising lifetime.
     match match i {
-        //~^ ERROR: temporary with significant `Drop` in `match` scrutinee will live until the
-        //~| NOTE: this might lead to deadlocks or other unexpected behavior
+        //~^ significant_drop_in_scrutinee
         100 => mutex1.lock().unwrap(),
         _ => mutex2.lock().unwrap(),
     }
@@ -448,8 +436,7 @@ fn should_trigger_lint_for_return_from_if_in_scrutinee() {
     // with a significant drop is but not used directly in any match arms, so it has a potentially
     // surprising lifetime.
     match if i > 1 {
-        //~^ ERROR: temporary with significant `Drop` in `match` scrutinee will live until the
-        //~| NOTE: this might lead to deadlocks or other unexpected behavior
+        //~^ significant_drop_in_scrutinee
         mutex1.lock().unwrap()
     } else {
         mutex2.lock().unwrap()
@@ -504,8 +491,7 @@ fn should_trigger_lint_for_boxed_mutex_guard() {
     // Should trigger lint because a temporary Box holding a type with a significant drop in a match
     // scrutinee may have a potentially surprising lifetime.
     match s.lock().deref().deref() {
-        //~^ ERROR: temporary with significant `Drop` in `match` scrutinee will live until the
-        //~| NOTE: this might lead to deadlocks or other unexpected behavior
+        //~^ significant_drop_in_scrutinee
         0 | 1 => println!("Value was less than 2"),
         _ => println!("Value is {}", s.lock().deref()),
     };
@@ -554,32 +540,28 @@ fn should_trigger_lint_in_assign_expr() {
     let mut i = 100;
 
     match mutex.lock().unwrap().i = i {
-        //~^ ERROR: temporary with significant `Drop` in `match` scrutinee will live until the
-        //~| NOTE: this might lead to deadlocks or other unexpected behavior
+        //~^ significant_drop_in_scrutinee
         _ => {
             println!("{}", mutex.lock().unwrap().i);
         },
     };
 
     match i = mutex.lock().unwrap().i {
-        //~^ ERROR: temporary with significant `Drop` in `match` scrutinee will live until the
-        //~| NOTE: this might lead to deadlocks or other unexpected behavior
+        //~^ significant_drop_in_scrutinee
         _ => {
             println!("{}", mutex.lock().unwrap().i);
         },
     };
 
     match mutex.lock().unwrap().i += 1 {
-        //~^ ERROR: temporary with significant `Drop` in `match` scrutinee will live until the
-        //~| NOTE: this might lead to deadlocks or other unexpected behavior
+        //~^ significant_drop_in_scrutinee
         _ => {
             println!("{}", mutex.lock().unwrap().i);
         },
     };
 
     match i += mutex.lock().unwrap().i {
-        //~^ ERROR: temporary with significant `Drop` in `match` scrutinee will live until the
-        //~| NOTE: this might lead to deadlocks or other unexpected behavior
+        //~^ significant_drop_in_scrutinee
         _ => {
             println!("{}", mutex.lock().unwrap().i);
         },
@@ -643,8 +625,7 @@ impl ResultReturner {
 fn should_trigger_lint_for_non_ref_move_and_clone_suggestion() {
     let rwlock = RwLock::<ResultReturner>::new(ResultReturner { s: "1".to_string() });
     match rwlock.read().unwrap().to_number() {
-        //~^ ERROR: temporary with significant `Drop` in `match` scrutinee will live until the
-        //~| NOTE: this might lead to deadlocks or other unexpected behavior
+        //~^ significant_drop_in_scrutinee
         Ok(n) => println!("Converted to number: {}", n),
         Err(e) => println!("Could not convert {} to number", e),
     };
@@ -671,8 +652,7 @@ fn should_trigger_lint_without_significant_drop_in_arm() {
     // is preserved until the end of the match, but there is no clear indication that this is the
     // case.
     match mutex.lock().unwrap().foo() {
-        //~^ ERROR: temporary with significant `Drop` in `match` scrutinee will live until the
-        //~| NOTE: this might lead to deadlocks or other unexpected behavior
+        //~^ significant_drop_in_scrutinee
         true => do_bar(&mutex),
         false => {},
     };
@@ -734,8 +714,7 @@ fn should_not_trigger_for_significant_drop_ref() {
     }
 
     match guard.take().len() {
-        //~^ ERROR: temporary with significant `Drop` in `match` scrutinee will live until the
-        //~| NOTE: this might lead to deadlocks or other unexpected behavior
+        //~^ significant_drop_in_scrutinee
         0 => println!("empty"),
         _ => println!("not empty"),
     };
@@ -760,8 +739,7 @@ fn should_trigger_lint_if_and_only_if_lifetime_is_irrelevant() {
     // Should trigger lint even if `copy_old_lifetime()` has a lifetime, as the lifetime of
     // `&vec` is unrelated to the temporary with significant drop (i.e., the `MutexGuard`).
     for val in mutex.lock().unwrap().copy_old_lifetime() {
-        //~^ ERROR: temporary with significant `Drop` in `for` loop condition will live until the
-        //~| NOTE: this might lead to deadlocks or other unexpected behavior
+        //~^ significant_drop_in_scrutinee
         println!("{}", val);
     }
 
@@ -800,8 +778,7 @@ fn should_not_trigger_lint_with_explicit_drop() {
 
     // Should trigger lint if there is no explicit drop.
     for val in [mutex.lock().unwrap()[0], 2] {
-        //~^ ERROR: temporary with significant `Drop` in `for` loop condition will live until the
-        //~| NOTE: this might lead to deadlocks or other unexpected behavior
+        //~^ significant_drop_in_scrutinee
         println!("{:?}", val);
     }
 }
@@ -810,8 +787,7 @@ fn should_trigger_lint_in_if_let() {
     let mutex = Mutex::new(vec![1]);
 
     if let Some(val) = mutex.lock().unwrap().first().copied() {
-        //~^ ERROR: temporary with significant `Drop` in `if let` scrutinee will live until the
-        //~| NOTE: this might lead to deadlocks or other unexpected behavior
+        //~^ significant_drop_in_scrutinee
         println!("{}", val);
     }
 
@@ -826,10 +802,41 @@ fn should_trigger_lint_in_while_let() {
     let mutex = Mutex::new(vec![1]);
 
     while let Some(val) = mutex.lock().unwrap().pop() {
-        //~^ ERROR: temporary with significant `Drop` in `while let` scrutinee will live until the
-        //~| NOTE: this might lead to deadlocks or other unexpected behavior
+        //~^ significant_drop_in_scrutinee
         println!("{}", val);
     }
 }
 
+async fn foo_async(mutex: &Mutex<i32>) -> Option<MutexGuard<'_, i32>> {
+    Some(mutex.lock().unwrap())
+}
+
+async fn should_trigger_lint_for_async(mutex: Mutex<i32>) -> i32 {
+    match *foo_async(&mutex).await.unwrap() {
+        //~^ significant_drop_in_scrutinee
+        n if n < 10 => n,
+        _ => 10,
+    }
+}
+
+async fn should_not_trigger_lint_in_async_expansion(mutex: Mutex<i32>) -> i32 {
+    match foo_async(&mutex).await {
+        Some(guard) => *guard,
+        _ => 0,
+    }
+}
+
+fn should_trigger_lint_in_match_expr() {
+    let mutex = Mutex::new(State {});
+
+    // Should trigger lint because the lifetime of the temporary MutexGuard is surprising because it
+    // is preserved until the end of the match, but there is no clear indication that this is the
+    // case.
+    let _ = match mutex.lock().unwrap().foo() {
+        //~^ significant_drop_in_scrutinee
+        true => 0,
+        false => 1,
+    };
+}
+
 fn main() {}
