@@ -4,14 +4,23 @@
 mod unsafe_items_invalid_comment {
     // SAFETY:
     const CONST: u32 = 0;
+    //~^ unnecessary_safety_comment
+
     // SAFETY:
     static STATIC: u32 = 0;
+    //~^ unnecessary_safety_comment
+
     // SAFETY:
     struct Struct;
+    //~^ unnecessary_safety_comment
+
     // SAFETY:
     enum Enum {}
+    //~^ unnecessary_safety_comment
+
     // SAFETY:
     mod module {}
+    //~^ unnecessary_safety_comment
 }
 
 mod unnecessary_from_macro {
@@ -31,6 +40,7 @@ mod unnecessary_from_macro {
         ($t:ty) => {
             // Safety: unnecessary
             impl T for $t {}
+            //~^ unnecessary_safety_comment
         };
     }
 
@@ -40,12 +50,15 @@ mod unnecessary_from_macro {
 fn unnecessary_on_stmt_and_expr() -> u32 {
     // SAFETY: unnecessary
     let num = 42;
+    //~^ unnecessary_safety_comment
 
     // SAFETY: unnecessary
     if num > 24 {}
+    //~^ unnecessary_safety_comment
 
     // SAFETY: unnecessary
     24
+    //~^ unnecessary_safety_comment
 }
 
 mod issue_10084 {
@@ -65,4 +78,25 @@ mod issue_10084 {
     }
 }
 
+mod issue_12048 {
+    pub const X: u8 = 0;
+
+    /// Returns a pointer to five.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use foo::point_to_five;
+    ///
+    /// let five_pointer = point_to_five();
+    /// // Safety: this pointer always points to a valid five.
+    /// let five = unsafe { *five_pointer };
+    /// assert_eq!(five, 5);
+    /// ```
+    pub fn point_to_five() -> *const u8 {
+        static FIVE: u8 = 5;
+        &FIVE
+    }
+}
+
 fn main() {}
