@@ -1,14 +1,14 @@
 #![allow(
+    clippy::manual_async_fn,
     clippy::needless_borrow,
     clippy::needless_borrows_for_generic_args,
+    clippy::needless_lifetimes,
+    clippy::owned_cow,
     clippy::ptr_arg,
-    clippy::manual_async_fn,
-    clippy::needless_lifetimes
+    clippy::uninlined_format_args
 )]
 #![warn(clippy::unnecessary_to_owned, clippy::redundant_clone)]
 
-//@no-rustfix: need to change the suggestion to a multipart suggestion
-
 use std::borrow::Cow;
 use std::ffi::{CStr, CString, OsStr, OsString};
 use std::ops::Deref;
@@ -64,82 +64,143 @@ fn main() {
     let x_ref = &x;
 
     require_c_str(&Cow::from(c_str).into_owned());
+    //~^ unnecessary_to_owned
     require_c_str(&c_str.to_owned());
+    //~^ unnecessary_to_owned
 
     require_os_str(&os_str.to_os_string());
+    //~^ unnecessary_to_owned
     require_os_str(&Cow::from(os_str).into_owned());
+    //~^ unnecessary_to_owned
     require_os_str(&os_str.to_owned());
+    //~^ unnecessary_to_owned
 
     require_path(&path.to_path_buf());
+    //~^ unnecessary_to_owned
     require_path(&Cow::from(path).into_owned());
+    //~^ unnecessary_to_owned
     require_path(&path.to_owned());
+    //~^ unnecessary_to_owned
 
     require_str(&s.to_string());
+    //~^ unnecessary_to_owned
     require_str(&Cow::from(s).into_owned());
+    //~^ unnecessary_to_owned
     require_str(&s.to_owned());
+    //~^ unnecessary_to_owned
     require_str(&x_ref.to_string());
+    //~^ unnecessary_to_owned
 
     require_slice(&slice.to_vec());
+    //~^ unnecessary_to_owned
     require_slice(&Cow::from(slice).into_owned());
+    //~^ unnecessary_to_owned
     require_slice(&array.to_owned());
+    //~^ unnecessary_to_owned
     require_slice(&array_ref.to_owned());
+    //~^ unnecessary_to_owned
     require_slice(&slice.to_owned());
+    //~^ unnecessary_to_owned
     require_slice(&x_ref.to_owned()); // No longer flagged because of #8759.
 
     require_x(&Cow::<X>::Owned(x.clone()).into_owned());
+    //~^ unnecessary_to_owned
     require_x(&x_ref.to_owned()); // No longer flagged because of #8759.
 
     require_deref_c_str(c_str.to_owned());
+    //~^ unnecessary_to_owned
     require_deref_os_str(os_str.to_owned());
+    //~^ unnecessary_to_owned
     require_deref_path(path.to_owned());
+    //~^ unnecessary_to_owned
     require_deref_str(s.to_owned());
+    //~^ unnecessary_to_owned
     require_deref_slice(slice.to_owned());
+    //~^ unnecessary_to_owned
 
     require_impl_deref_c_str(c_str.to_owned());
+    //~^ unnecessary_to_owned
     require_impl_deref_os_str(os_str.to_owned());
+    //~^ unnecessary_to_owned
     require_impl_deref_path(path.to_owned());
+    //~^ unnecessary_to_owned
     require_impl_deref_str(s.to_owned());
+    //~^ unnecessary_to_owned
     require_impl_deref_slice(slice.to_owned());
+    //~^ unnecessary_to_owned
 
     require_deref_str_slice(s.to_owned(), slice.to_owned());
+    //~^ unnecessary_to_owned
+    //~| unnecessary_to_owned
     require_deref_slice_str(slice.to_owned(), s.to_owned());
+    //~^ unnecessary_to_owned
+    //~| unnecessary_to_owned
 
     require_as_ref_c_str(c_str.to_owned());
+    //~^ unnecessary_to_owned
     require_as_ref_os_str(os_str.to_owned());
+    //~^ unnecessary_to_owned
     require_as_ref_path(path.to_owned());
+    //~^ unnecessary_to_owned
     require_as_ref_str(s.to_owned());
+    //~^ unnecessary_to_owned
     require_as_ref_str(x.to_owned());
+    //~^ unnecessary_to_owned
     require_as_ref_slice(array.to_owned());
+    //~^ unnecessary_to_owned
     require_as_ref_slice(array_ref.to_owned());
+    //~^ unnecessary_to_owned
     require_as_ref_slice(slice.to_owned());
+    //~^ unnecessary_to_owned
 
     require_impl_as_ref_c_str(c_str.to_owned());
+    //~^ unnecessary_to_owned
     require_impl_as_ref_os_str(os_str.to_owned());
+    //~^ unnecessary_to_owned
     require_impl_as_ref_path(path.to_owned());
+    //~^ unnecessary_to_owned
     require_impl_as_ref_str(s.to_owned());
+    //~^ unnecessary_to_owned
     require_impl_as_ref_str(x.to_owned());
+    //~^ unnecessary_to_owned
     require_impl_as_ref_slice(array.to_owned());
+    //~^ unnecessary_to_owned
     require_impl_as_ref_slice(array_ref.to_owned());
+    //~^ unnecessary_to_owned
     require_impl_as_ref_slice(slice.to_owned());
+    //~^ unnecessary_to_owned
 
     require_as_ref_str_slice(s.to_owned(), array.to_owned());
+    //~^ unnecessary_to_owned
+    //~| unnecessary_to_owned
     require_as_ref_str_slice(s.to_owned(), array_ref.to_owned());
+    //~^ unnecessary_to_owned
+    //~| unnecessary_to_owned
     require_as_ref_str_slice(s.to_owned(), slice.to_owned());
+    //~^ unnecessary_to_owned
+    //~| unnecessary_to_owned
     require_as_ref_slice_str(array.to_owned(), s.to_owned());
+    //~^ unnecessary_to_owned
+    //~| unnecessary_to_owned
     require_as_ref_slice_str(array_ref.to_owned(), s.to_owned());
+    //~^ unnecessary_to_owned
+    //~| unnecessary_to_owned
     require_as_ref_slice_str(slice.to_owned(), s.to_owned());
+    //~^ unnecessary_to_owned
+    //~| unnecessary_to_owned
 
     let _ = x.join(&x_ref.to_string());
+    //~^ unnecessary_to_owned
 
     let _ = slice.to_vec().into_iter();
+    //~^ unnecessary_to_owned
     let _ = slice.to_owned().into_iter();
-    let _ = [std::path::PathBuf::new()][..].to_vec().into_iter();
-    let _ = [std::path::PathBuf::new()][..].to_owned().into_iter();
+    //~^ unnecessary_to_owned
 
     let _ = IntoIterator::into_iter(slice.to_vec());
+    //~^ unnecessary_to_owned
     let _ = IntoIterator::into_iter(slice.to_owned());
-    let _ = IntoIterator::into_iter([std::path::PathBuf::new()][..].to_vec());
-    let _ = IntoIterator::into_iter([std::path::PathBuf::new()][..].to_owned());
+    //~^ unnecessary_to_owned
 
     let _ = check_files(&[FileType::Account]);
 
@@ -155,15 +216,23 @@ fn main() {
 
     // The following should be flagged by `redundant_clone`, but not by this lint.
     require_c_str(&CString::from_vec_with_nul(vec![0]).unwrap().to_owned());
+    //~^ redundant_clone
     require_os_str(&OsString::from("x").to_os_string());
+    //~^ redundant_clone
     require_path(&std::path::PathBuf::from("x").to_path_buf());
+    //~^ redundant_clone
     require_str(&String::from("x").to_string());
+    //~^ redundant_clone
     require_slice(&[String::from("x")].to_owned());
+    //~^ redundant_clone
 
     let slice = [0u8; 1024];
     let _ref_str: &str = &String::from_utf8(slice.to_vec()).expect("not UTF-8");
+    //~^ unnecessary_to_owned
     let _ref_str: &str = &String::from_utf8(b"foo".to_vec()).unwrap();
+    //~^ unnecessary_to_owned
     let _ref_str: &str = &String::from_utf8(b"foo".as_slice().to_owned()).unwrap();
+    //~^ unnecessary_to_owned
     // Expression is of type `&String`, can't suggest `str::from_utf8` here
     let _ref_string = &String::from_utf8(b"foo".to_vec()).unwrap();
     macro_rules! arg_from_macro {
@@ -221,6 +290,7 @@ fn require_as_ref_slice_str<T, U: AsRef<[T]>, V: AsRef<str>>(_: U, _: V) {}
 // https://github.com/breard-r/acmed/blob/1f0dcc32aadbc5e52de6d23b9703554c0f925113/acmed/src/storage.rs#L262
 fn check_files(file_types: &[FileType]) -> bool {
     for t in file_types.to_vec() {
+        //~^ unnecessary_to_owned
         let path = match get_file_path(&t) {
             Ok(p) => p,
             Err(_) => {
@@ -240,17 +310,6 @@ fn get_file_path(_file_type: &FileType) -> Result<std::path::PathBuf, std::io::E
 
 fn require_string(_: &String) {}
 
-#[clippy::msrv = "1.35"]
-fn _msrv_1_35() {
-    // `copied` was stabilized in 1.36, so clippy should use `cloned`.
-    let _ = &["x"][..].to_vec().into_iter();
-}
-
-#[clippy::msrv = "1.36"]
-fn _msrv_1_36() {
-    let _ = &["x"][..].to_vec().into_iter();
-}
-
 // https://github.com/rust-lang/rust-clippy/issues/8507
 mod issue_8507 {
     #![allow(dead_code)]
@@ -297,6 +356,7 @@ mod issue_8507 {
     // Should lint because Y is copy.
     fn test_y(y: Y) -> Box<dyn Abstracted> {
         Box::new(build(y.to_string()))
+        //~^ unnecessary_to_owned
     }
 }
 
@@ -406,6 +466,7 @@ mod issue_9351 {
     // Should lint
     fn single_return() -> impl AsRef<str> {
         id("abc".to_string())
+        //~^ unnecessary_to_owned
     }
 
     // Should not lint
@@ -549,6 +610,7 @@ mod issue_11952 {
 
     fn bar() {
         IntoFuture::into_future(foo([].to_vec(), &0));
+        //~^ unnecessary_to_owned
     }
 }
 
@@ -558,18 +620,23 @@ fn borrow_checks() {
 
     fn inner(a: &[&str]) {
         let mut s = HashSet::from([vec!["a"]]);
-        s.remove(&a.to_vec()); //~ ERROR: unnecessary use of `to_vec`
+        s.remove(&a.to_vec());
+        //~^ unnecessary_to_owned
     }
 
     let mut s = HashSet::from(["a".to_string()]);
-    s.remove(&"b".to_owned()); //~ ERROR: unnecessary use of `to_owned`
-    s.remove(&"b".to_string()); //~ ERROR: unnecessary use of `to_string`
+    s.remove(&"b".to_owned());
+    //~^ unnecessary_to_owned
+    s.remove(&"b".to_string());
+    //~^ unnecessary_to_owned
     // Should not warn.
     s.remove("b");
 
     let mut s = HashSet::from([vec!["a"]]);
-    s.remove(&["b"].to_vec()); //~ ERROR: unnecessary use of `to_vec`
-    s.remove(&(&["b"]).to_vec()); //~ ERROR: unnecessary use of `to_vec`
+    s.remove(&["b"].to_vec());
+    //~^ unnecessary_to_owned
+    s.remove(&(&["b"]).to_vec());
+    //~^ unnecessary_to_owned
 
     // Should not warn.
     s.remove(&["b"].to_vec().clone());
@@ -587,3 +654,24 @@ fn borrow_checks() {
     HashSet::<i32>::new().foo::<&str>(&"".to_owned());
     HashSet::<String>::new().get(&1.to_string());
 }
+
+fn issue13624() -> impl IntoIterator {
+    let cow: Cow<'_, Vec<String>> = Cow::Owned(vec![String::from("foo")]);
+
+    cow.into_owned().into_iter()
+}
+
+mod issue_14242 {
+    use std::rc::Rc;
+
+    #[derive(Copy, Clone)]
+    struct Foo;
+
+    fn rc_slice_provider() -> Rc<[Foo]> {
+        Rc::from([Foo])
+    }
+
+    fn iterator_provider() -> impl Iterator<Item = Foo> {
+        rc_slice_provider().to_vec().into_iter()
+    }
+}
