@@ -1,11 +1,16 @@
 #![deny(clippy::useless_conversion)]
-#![allow(clippy::needless_if, clippy::unnecessary_fallible_conversions)]
+#![allow(
+    clippy::needless_if,
+    clippy::unnecessary_fallible_conversions,
+    clippy::manual_unwrap_or_default
+)]
 
 fn test_generic<T: Copy>(val: T) -> T {
     let _ = T::try_from(val).unwrap();
-    //~^ ERROR: useless conversion to the same type: `T`
+    //~^ useless_conversion
+
     val.try_into().unwrap()
-    //~^ ERROR: useless conversion to the same type: `T`
+    //~^ useless_conversion
 }
 
 fn test_generic2<T: Copy + Into<i32> + Into<U>, U: From<T>>(val: T) {
@@ -28,19 +33,25 @@ fn main() {
         let _: String = "foo".try_into().unwrap();
     }
     let _: String = "foo".to_string().try_into().unwrap();
-    //~^ ERROR: useless conversion to the same type: `std::string::String`
+    //~^ useless_conversion
+
     let _: String = TryFrom::try_from("foo".to_string()).unwrap();
-    //~^ ERROR: useless conversion to the same type: `std::string::String`
+    //~^ useless_conversion
+
     let _ = String::try_from("foo".to_string()).unwrap();
-    //~^ ERROR: useless conversion to the same type: `std::string::String`
+    //~^ useless_conversion
+
     let _ = String::try_from(format!("A: {:04}", 123)).unwrap();
-    //~^ ERROR: useless conversion to the same type: `std::string::String`
+    //~^ useless_conversion
+
     let _: String = format!("Hello {}", "world").try_into().unwrap();
-    //~^ ERROR: useless conversion to the same type: `std::string::String`
+    //~^ useless_conversion
+
     let _: String = String::new().try_into().unwrap();
-    //~^ ERROR: useless conversion to the same type: `std::string::String`
+    //~^ useless_conversion
+
     let _: String = match String::from("_").try_into() {
-        //~^ ERROR: useless conversion to the same type: `std::string::String`
+        //~^ useless_conversion
         Ok(a) => a,
         Err(_) => String::new(),
     };
