@@ -221,7 +221,7 @@ std::optional<Value> getStored(Operation *op);
 std::optional<Value> getCopySource(Operation *op);
 
 void enzyme::PointsToPointerAnalysis::processCapturingStore(
-    ProgramPoint dependent, PointsToSets *after, Value capturedValue,
+    ProgramPoint *dependent, PointsToSets *after, Value capturedValue,
     Value destinationAddress, bool isMustStore) {
   auto *srcClasses =
       getOrCreateFor<AliasClassLattice>(dependent, capturedValue);
@@ -270,9 +270,8 @@ static bool isNoOp(Operation *op) {
              LLVM::LifetimeEndOp, LLVM::AssumeOp, LLVM::UnreachableOp>(op);
 }
 
-void enzyme::PointsToPointerAnalysis::visitOperation(Operation *op,
-                                                     const PointsToSets &before,
-                                                     PointsToSets *after) {
+LogicalResult enzyme::PointsToPointerAnalysis::visitOperation(
+    Operation *op, const PointsToSets &before, PointsToSets *after) {
   join(after, before);
 
   // If we know nothing about memory effects, record reaching the pessimistic
@@ -280,9 +279,9 @@ void enzyme::PointsToPointerAnalysis::visitOperation(Operation *op,
   auto memory = dyn_cast<MemoryEffectOpInterface>(op);
   if (!memory) {
     if (isNoOp(op))
-      return;
+      return success();
     propagateIfChanged(after, after->markAllPointToUnknown());
-    return;
+    return success();
   }
 
   SmallVector<MemoryEffects::EffectInstance> effects;
@@ -293,11 +292,11 @@ void enzyme::PointsToPointerAnalysis::visitOperation(Operation *op,
   if (effects.size() == 1 &&
       isa<MemoryEffects::Allocate>(effects.front().getEffect()) &&
       effects.front().getValue()) {
-    const auto *destClasses =
-        getOrCreateFor<AliasClassLattice>(op, effects.front().getValue());
+    const auto *destClasses = getOrCreateFor<AliasClassLattice>(
+        getProgramPointAfter(op), effects.front().getValue());
     propagateIfChanged(
         after, after->setPointingToEmpty(destClasses->getAliasClassesObject()));
-    return;
+    return success();
   }
 
   for (const auto &effect : effects) {
@@ -333,11 +332,12 @@ void enzyme::PointsToPointerAnalysis::visitOperation(Operation *op,
 
     for (Value address : targetValues) {
       for (Value stored : storedValues) {
-        processCapturingStore(op, after, stored, address,
+        processCapturingStore(getProgramPointAfter(op), after, stored, address,
                               isMustStore(op, address));
       }
     }
   }
+  return success();
 }
 
 constexpr static llvm::StringLiteral kLLVMMemoryAttrName = "memory";
@@ -467,7 +467,8 @@ void enzyme::PointsToPointerAnalysis::processCallToSummarizedFunc(
   ChangeResult changed = ChangeResult::NoChange;
   // Unify the points-to summary with the actual lattices of function arguments
   for (auto &&[i, argOperand] : llvm::enumerate(call.getArgOperands())) {
-    auto *arg = getOrCreateFor<AliasClassLattice>(call, argOperand);
+    auto *arg = getOrCreateFor<AliasClassLattice>(getProgramPointAfter(call),
+                                                  argOperand);
 
     std::optional<AliasClassSet> calleePointsTo = lookup(i, /*depth=*/0);
     // If the argument class isn't in the summary, it hasn't changed what
@@ -520,7 +521,8 @@ void enzyme::PointsToPointerAnalysis::visitCallControlFlowTransfer(
       // memalign deals with nested pointers and thus must be handled here
       // memalign points to a value
       OperandRange arguments = call.getArgOperands();
-      auto *memPtr = getOrCreateFor<AliasClassLattice>(call, arguments[0]);
+      auto *memPtr = getOrCreateFor<AliasClassLattice>(
+          getProgramPointAfter(call), arguments[0]);
 
       // Note that this is a "must write" kind of situation, so we can
       // directly set the classes pointed to, rather than inserting them.
@@ -581,7 +583,7 @@ void enzyme::PointsToPointerAnalysis::visitCallControlFlowTransfer(
             continue;
 
           const auto *srcClasses = getOrCreateFor<AliasClassLattice>(
-              call, call.getArgOperands()[pointerAsData]);
+              getProgramPointAfter(call), call.getArgOperands()[pointerAsData]);
           (void)functionMayCapture.join(srcClasses->getAliasClassesObject());
         }
       }
@@ -593,7 +595,7 @@ void enzyme::PointsToPointerAnalysis::visitCallControlFlowTransfer(
       ChangeResult changed = ChangeResult::NoChange;
       for (int pointerOperand : pointerLikeOperands) {
         auto *destClasses = getOrCreateFor<AliasClassLattice>(
-            call, call.getArgOperands()[pointerOperand]);
+            getProgramPointAfter(call), call.getArgOperands()[pointerOperand]);
 
         // If the argument cannot be stored into, just preserve it as is.
         if (!mayWriteArg(callee, pointerOperand, argModRef)) {
@@ -670,8 +672,8 @@ void enzyme::PointsToPointerAnalysis::visitCallControlFlowTransfer(
         // marked as potentially pointing to some other classes, this marking
         // is *not* rolled back. Since points-to-pointer analysis is a may-
         // analysis, this is not problematic.
-        const auto *destClasses =
-            getOrCreateFor<AliasClassLattice>(call, result);
+        const auto *destClasses = getOrCreateFor<AliasClassLattice>(
+            getProgramPointAfter(call), result);
         AliasClassSet resultWithoutNonWritableOperands =
             AliasClassSet::getUndefined();
         if (destClasses->isUnknown() || nonWritableOperandClasses.isUnknown()) {
@@ -696,7 +698,7 @@ void enzyme::PointsToPointerAnalysis::visitCallControlFlowTransfer(
 
         for (int operandNo : pointerLikeOperands) {
           const auto *srcClasses = getOrCreateFor<AliasClassLattice>(
-              call, call.getArgOperands()[operandNo]);
+              getProgramPointAfter(call), call.getArgOperands()[operandNo]);
           if (mayReadArg(callee, operandNo, argModRef)) {
             changed |= after->addSetsFrom(resultWithoutNonWritableOperands,
                                           srcClasses->getAliasClassesObject());
@@ -746,7 +748,7 @@ enzyme::AliasClassLattice::alias(const AbstractSparseLattice &other) const {
 
   assert(!isUndefined() && !rhs->isUndefined() && "incomplete alias analysis");
 
-  if (getPoint() == rhs->getPoint())
+  if (getAnchor() == rhs->getAnchor())
     return AliasResult::MustAlias;
 
   if (elements.isUnknown() || rhs->elements.isUnknown())
@@ -783,7 +785,7 @@ enzyme::AliasClassLattice::join(const AbstractSparseLattice &other) {
 //===----------------------------------------------------------------------===//
 
 void enzyme::AliasAnalysis::setToEntryState(AliasClassLattice *lattice) {
-  if (auto arg = dyn_cast<BlockArgument>(lattice->getPoint())) {
+  if (auto arg = dyn_cast<BlockArgument>(lattice->getAnchor())) {
     if (auto funcOp =
             dyn_cast<FunctionOpInterface>(arg.getOwner()->getParentOp())) {
       // TODO: Not safe in general, integers can be a result of ptrtoint. We
@@ -805,15 +807,15 @@ void enzyme::AliasAnalysis::setToEntryState(AliasClassLattice *lattice) {
                 PseudoAliasClassAttr::get(FlatSymbolRefAttr::get(funcOp),
                                           arg.getArgNumber(), /*depth=*/0);
           }
-          DistinctAttr argClass =
-              originalClasses.getOriginalClass(lattice->getPoint(), debugLabel);
+          DistinctAttr argClass = originalClasses.getOriginalClass(
+              lattice->getAnchor(), debugLabel);
           return propagateIfChanged(lattice,
                                     lattice->join(AliasClassLattice::single(
-                                        lattice->getPoint(), argClass)));
+                                        lattice->getAnchor(), argClass)));
         } else {
           return propagateIfChanged(lattice,
                                     lattice->join(AliasClassLattice::single(
-                                        lattice->getPoint(), entryClass)));
+                                        lattice->getAnchor(), entryClass)));
         }
       }
     }
@@ -857,7 +859,7 @@ void enzyme::AliasAnalysis::transfer(
     if (isa<MemoryEffects::Allocate>(effect.getEffect())) {
       // Mark the result of the allocation as a fresh memory location.
       for (AliasClassLattice *result : results) {
-        if (result->getPoint() == value) {
+        if (result->getAnchor() == value) {
           std::string debugLabel;
           llvm::raw_string_ostream sstream(debugLabel);
           if (relative)
@@ -871,13 +873,14 @@ void enzyme::AliasAnalysis::transfer(
             }
           }
           auto fresh = AliasClassLattice::single(
-              result->getPoint(),
-              originalClasses.getOriginalClass(result->getPoint(), debugLabel));
+              result->getAnchor(), originalClasses.getOriginalClass(
+                                       result->getAnchor(), debugLabel));
           propagateIfChanged(result, result->join(fresh));
         }
       }
     } else if (isa<MemoryEffects::Read>(effect.getEffect())) {
-      auto *pointsToSets = getOrCreateFor<PointsToSets>(op, op);
+      auto *pointsToSets = getOrCreateFor<PointsToSets>(
+          getProgramPointAfter(op), getProgramPointAfter(op));
       AliasClassLattice *latticeElement = getLatticeElement(value);
       if (latticeElement->isUnknown()) {
         for (AliasClassLattice *result : results) {
@@ -889,7 +892,7 @@ void enzyme::AliasAnalysis::transfer(
         for (auto srcClass : latticeElement->getAliasClasses()) {
           const auto &srcPointsTo = pointsToSets->getPointsTo(srcClass);
           for (AliasClassLattice *result : results) {
-            if (!isPointerLike(result->getPoint().getType()))
+            if (!isPointerLike(result->getAnchor().getType()))
               continue;
 
             // TODO: consider some sort of "point join" or better insert that
@@ -929,7 +932,8 @@ void enzyme::AliasAnalysis::transfer(
     // which is not true in general but can result in improved analysis speed.
     // We need a type analysis for full correctness.
     constexpr bool pruneNonPointers = false;
-    if (pruneNonPointers && !isPointerLike(resultLattice->getPoint().getType()))
+    if (pruneNonPointers &&
+        !isPointerLike(resultLattice->getAnchor().getType()))
       continue;
 
     ChangeResult r = ChangeResult::NoChange;
@@ -945,11 +949,12 @@ void enzyme::AliasAnalysis::createImplicitArgDereference(
   assert(relative && "only valid to create implicit argument dereferences when "
                      "operating in relative mode");
 
-  Value readResult = result->getPoint();
+  Value readResult = result->getAnchor();
   auto parent = op->getParentOfType<FunctionOpInterface>();
   assert(parent && "failed to find function parent");
-  auto *entryPointsToSets =
-      getOrCreateFor<PointsToSets>(op, &parent.getCallableRegion()->front());
+  auto *entryPointsToSets = getOrCreateFor<PointsToSets>(
+      getProgramPointAfter(op),
+      getProgramPointAfter(&parent.getCallableRegion()->front()));
   if (!entryPointsToSets->getPointsTo(srcClass).isUndefined()) {
     // Only create the pseudo class if another load hasn't already created the
     // implicitly dereferenced pseudo class.
@@ -965,8 +970,8 @@ void enzyme::AliasAnalysis::createImplicitArgDereference(
     propagateIfChanged(result, result->join(AliasClassLattice::single(
                                    readResult, derefClass)));
     // The read source points to the dereferenced class
-    auto *pointsToState =
-        getOrCreate<PointsToSets>(&parent.getCallableRegion()->front());
+    auto *pointsToState = getOrCreate<PointsToSets>(
+        getProgramPointAfter(&parent.getCallableRegion()->front()));
     propagateIfChanged(pointsToState,
                        pointsToState->insert(source->getAliasClassesObject(),
                                              AliasClassSet(derefClass)));
@@ -1011,7 +1016,7 @@ LogicalResult getEffectsForExternalCall(
   return failure();
 }
 
-void enzyme::AliasAnalysis::visitOperation(
+LogicalResult enzyme::AliasAnalysis::visitOperation(
     Operation *op, ArrayRef<const AliasClassLattice *> operands,
     ArrayRef<AliasClassLattice *> results) {
 
@@ -1025,7 +1030,7 @@ void enzyme::AliasAnalysis::visitOperation(
 
       (void)results[result.getResultNumber()]->markUnknown();
     }
-    return;
+    return success();
   }
 
   SmallVector<MemoryEffects::EffectInstance> effects;
@@ -1036,6 +1041,7 @@ void enzyme::AliasAnalysis::visitOperation(
     // that non-overlapping subviews of the same buffer don't alias could be a
     // promising extension.
   }
+  return success();
 }
 
 static void
@@ -1142,7 +1148,8 @@ void enzyme::AliasAnalysis::visitExternalCall(
 
     // If can read from argument, collect the alias classes that can this
     // argument may be pointing to.
-    const auto *pointsToLattice = getOrCreateFor<PointsToSets>(call, call);
+    const auto *pointsToLattice = getOrCreateFor<PointsToSets>(
+        getProgramPointAfter(call), getProgramPointAfter(call));
     (void)srcClasses->getAliasClassesObject().foreachElement(
         [&](DistinctAttr srcClass, AliasClassSet::State state) {
           // Nothing to do in top/bottom case. In the top case, we have already
@@ -1177,8 +1184,8 @@ void enzyme::AliasAnalysis::visitExternalCall(
                                     std::to_string(result.getResultNumber()))
               : nullptr;
       auto individualAlloc = AliasClassLattice::single(
-          resultLattice->getPoint(),
-          originalClasses.getOriginalClass(resultLattice->getPoint(),
+          resultLattice->getAnchor(),
+          originalClasses.getOriginalClass(resultLattice->getAnchor(),
                                            individualDebugLabel));
       propagateIfChanged(resultLattice, resultLattice->join(individualAlloc));
     } else if (!modRefMayRef(otherModRef) &&
@@ -1192,8 +1199,8 @@ void enzyme::AliasAnalysis::visitExternalCall(
             originalClasses.getSameOriginalClass(aliasGroupResults, label);
       }
       AliasClassSet commonClass(commonResultAttr);
-      ChangeResult changed = resultLattice->join(
-          AliasClassLattice(resultLattice->getPoint(), std::move(commonClass)));
+      ChangeResult changed = resultLattice->join(AliasClassLattice(
+          resultLattice->getAnchor(), std::move(commonClass)));
 
       // If the function is known not to read other (or inaccessible mem), its
       // results may only alias what we know it can read, e.g. other arguments
@@ -1201,7 +1208,7 @@ void enzyme::AliasAnalysis::visitExternalCall(
       // FIXME: note the explicit copy, we need to simplify the relation between
       // AliasClassSet and AliasClassLattice.
       changed |= resultLattice->join(AliasClassLattice(
-          resultLattice->getPoint(), AliasClassSet(operandAliasClasses)));
+          resultLattice->getAnchor(), AliasClassSet(operandAliasClasses)));
       propagateIfChanged(resultLattice, changed);
     } else {
       propagateIfChanged(resultLattice, resultLattice->markUnknown());
