@@ -4,12 +4,12 @@ use std::process::{Command, Stdio};
 use std::thread;
 use std::time::Duration;
 
-use anyhow::{bail, Context as _};
+use anyhow::{Context as _, bail};
 use directories::ProjectDirs;
 use stdx::JodChild;
-use xshell::{cmd, Shell};
+use xshell::{Shell, cmd};
 
-use crate::{codegen, date_iso, flags, is_release_tag, project_root};
+use crate::{date_iso, flags, is_release_tag, project_root};
 
 impl flags::Release {
     pub(crate) fn run(self, sh: &Shell) -> anyhow::Result<()> {
@@ -29,10 +29,6 @@ impl flags::Release {
             cmd!(sh, "git push --force").run()?;
         }
 
-        // Generates bits of manual.adoc.
-        codegen::diagnostics_docs::generate(false);
-        codegen::assists_doc_tests::generate(false);
-
         let website_root = project_root().join("../rust-analyzer.github.io");
         {
             let _dir = sh.push_dir(&website_root);
@@ -53,22 +49,8 @@ impl flags::Release {
             .max()
             .unwrap_or_default();
 
-        for adoc in [
-            "manual.adoc",
-            "generated_assists.adoc",
-            "generated_config.adoc",
-            "generated_diagnostic.adoc",
-            "generated_features.adoc",
-        ] {
-            let src = project_root().join("./docs/user/").join(adoc);
-            let dst = website_root.join(adoc);
-
-            let contents = sh.read_file(src)?;
-            sh.write_file(dst, contents)?;
-        }
-
         let tags = cmd!(sh, "git tag --list").read()?;
-        let prev_tag = tags.lines().filter(|line| is_release_tag(line)).last().unwrap();
+        let prev_tag = tags.lines().filter(|line| is_release_tag(line)).next_back().unwrap();
 
         let contents = changelog::get_changelog(sh, changelog_n, &commit, prev_tag, &today)?;
         let path = changelog_dir.join(format!("{today}-changelog-{changelog_n}.adoc"));
@@ -119,12 +101,11 @@ impl flags::RustcPull {
         // Fetch given rustc commit.
         cmd!(sh, "git fetch http://localhost:{JOSH_PORT}/rust-lang/rust.git@{commit}{JOSH_FILTER}.git")
             .run()
-            .map_err(|e| {
+            .inspect_err(|_| {
                 // Try to un-do the previous `git commit`, to leave the repo in the state we found it it.
                 cmd!(sh, "git reset --hard HEAD^")
                     .run()
                     .expect("FAILED to clean up again after failed `git fetch`, sorry for that");
-                e
             })
             .context("FAILED to fetch new commits, something went wrong (committing the rust-version file has been undone)")?;
 
@@ -202,9 +183,14 @@ impl flags::RustcPush {
         let head = cmd!(sh, "git rev-parse HEAD").read()?;
         let fetch_head = cmd!(sh, "git rev-parse FETCH_HEAD").read()?;
         if head != fetch_head {
-            bail!("Josh created a non-roundtrip push! Do NOT merge this into rustc!");
+            bail!(
+                "Josh created a non-roundtrip push! Do NOT merge this into rustc!\n\
+                Expected {head}, got {fetch_head}."
+            );
         }
-        println!("Confirmed that the push round-trips back to rust-analyzer properly. Please create a rustc PR:");
+        println!(
+            "Confirmed that the push round-trips back to rust-analyzer properly. Please create a rustc PR:"
+        );
         // https://github.com/github-linguist/linguist/compare/master...octocat:linguist:master
         let fork_path = rust_fork.replace('/', ":");
         println!(
@@ -217,8 +203,7 @@ impl flags::RustcPush {
 }
 
 /// Used for rustc syncs.
-const JOSH_FILTER: &str =
-    ":rev(55d9a533b309119c8acd13061581b43ae8840823:prefix=src/tools/rust-analyzer):/src/tools/rust-analyzer";
+const JOSH_FILTER: &str = ":rev(55d9a533b309119c8acd13061581b43ae8840823:prefix=src/tools/rust-analyzer):/src/tools/rust-analyzer";
 const JOSH_PORT: &str = "42042";
 
 fn start_josh() -> anyhow::Result<impl Drop> {
