@@ -1,5 +1,5 @@
 // rustfmt-edition: 2018
-#![feature(rust_2018_preview, uniform_paths)]
+#![feature(uniform_paths)]
 use ::log::{error, info, log};
 use futures::prelude::*;
 use http_03::cli::Cli;
