@@ -7,7 +7,6 @@ use ignore::DirEntry;
 
 /// The default directory filter.
 pub fn filter_dirs(path: &Path) -> bool {
-    // FIXME: sync submodule exclusion list with rustfmt.toml
     // bootstrap/etc
     let skip = [
         "tidy-test-file",
@@ -33,8 +32,6 @@ pub fn filter_dirs(path: &Path) -> bool {
         "src/doc/rustc-dev-guide",
         "src/doc/reference",
         "src/gcc",
-        // Filter RLS output directories
-        "target/rls",
         "src/bootstrap/target",
         "vendor",
     ];
