@@ -1,4 +1,4 @@
-// compile-flags: -C no-prepopulate-passes -Copt-level=0
+//@ compile-flags: -C no-prepopulate-passes -Copt-level=0
 
 #![crate_type = "lib"]
 #![feature(const_eval_select)]
@@ -6,10 +6,14 @@
 
 use std::intrinsics::const_eval_select;
 
-const fn foo(_: i32) -> i32 { 1 }
+const fn foo(_: i32) -> i32 {
+    1
+}
 
 #[no_mangle]
-pub fn hi(n: i32) -> i32 { n }
+pub fn hi(n: i32) -> i32 {
+    n
+}
 
 #[no_mangle]
 pub unsafe fn hey() {
