@@ -8,15 +8,15 @@
 // CHECK: define dso_local i8 @call_foreign_fn()
 #[no_mangle]
 pub fn call_foreign_fn() -> u8 {
-    unsafe {
-        foreign_fn()
-    }
+    unsafe { foreign_fn() }
 }
 
 // External functions are still marked as non-dso_local, since we don't know if the symbol
 // is defined in the binary or in the shared library.
 // CHECK: declare zeroext i8 @foreign_fn()
-extern "C" {fn foreign_fn() -> u8;}
+extern "C" {
+    fn foreign_fn() -> u8;
+}
 
 // CHECK: !{i32 {{[78]}}, !"PIC Level", i32 2}
 // CHECK: !{i32 7, !"PIE Level", i32 2}
