@@ -1,21 +1,18 @@
-#![feature(rustc_private)]
-
-#[cfg(unix)]
-extern crate libc;
-
-use run_make_support::{aux_build, rfs};
-
 #[cfg(unix)]
 use std::os::unix::fs::PermissionsExt;
 use std::path::Path;
 
+#[cfg(unix)]
+use run_make_support::libc;
+use run_make_support::{rfs, rustc};
+
 fn main() {
     #[cfg(unix)]
     unsafe {
         libc::umask(0o002);
     }
 
-    aux_build().arg("foo.rs").run();
+    rustc().crate_type("lib").arg("foo.rs").run();
     verify(Path::new("libfoo.rlib"));
 }
 
