@@ -1,8 +1,6 @@
 //@ aux-build:block-on.rs
 //@ edition:2021
-//@ check-pass
-
-#![feature(async_closure)]
+//@ build-pass
 
 extern crate block_on;
 
