@@ -1,9 +1,7 @@
 //@ build-pass
 //@ edition: 2021
 
-#![feature(async_closure)]
-
-async fn foo(x: impl async Fn(&str) -> &str) {}
+async fn foo(x: impl AsyncFn(&str) -> &str) {}
 
 fn main() {
     foo(async |x| x);
