@@ -1,4 +1,4 @@
-// edition:2021
+//@ edition:2021
 // gate-test-async_for_loop
 
 #![feature(async_iter_from_iter, async_iterator)]
@@ -11,7 +11,7 @@ fn f() {
     };
 }
 
-#[cfg(FALSE)]
+#[cfg(false)]
 fn g() {
     let _ = async {
         for await _i in core::async_iter::from_iter(0..3) {
