@@ -1,8 +1,7 @@
-// run-pass
+//@ run-pass
 // main is conditionally compiled, but the conditional compilation
 // is conditional too!
 
-// pretty-expanded FIXME #23616
 
-#[cfg_attr(foo, cfg(bar))]
+#[cfg_attr(FALSE, cfg(bar))]
 fn main() { }
